@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: cli.c 7664 2009-01-11 17:46:51Z charles $
+ * $Id: cli.c 8411 2009-05-15 15:42:13Z charles $
  *
  * Copyright (c) 2005-2006 Transmission authors and contributors
  *
@@ -33,6 +33,7 @@
 #include <libtransmission/makemeta.h>
 #include <libtransmission/tr-getopt.h>
 #include <libtransmission/utils.h> /* tr_wait */
+#include <libtransmission/version.h>
 #include <libtransmission/web.h> /* tr_webRun */
 
 #define LINEWIDTH 80
@@ -47,10 +48,13 @@ static sig_atomic_t manualUpdate     = 0;
 
 static const char * torrentPath  = NULL;
 static const char * finishCall   = NULL;
-static const char * announce     = NULL;
 static const char * sourceFile   = NULL;
 static const char * comment      = NULL;
 
+#define MAX_ANNOUNCE 128
+static tr_tracker_info announce[MAX_ANNOUNCE];
+static int announceCount = 0;
+
 static const struct tr_option options[] =
 {
     { 'a', "announce",             "Set the new torrent's announce URL", "a",  1, "<url>"     },
@@ -331,11 +335,8 @@ main( int     argc,
     if( sourceFile && *sourceFile ) /* creating a torrent */
     {
         int                   err;
-        tr_metainfo_builder * b = tr_metaInfoBuilderCreate( h, sourceFile );
-        tr_tracker_info       ti;
-        ti.tier = 0;
-        ti.announce = (char*) announce;
-        tr_makeMetaInfo( b, torrentPath, &ti, 1, comment, isPrivate );
+        tr_metainfo_builder * b = tr_metaInfoBuilderCreate( sourceFile );
+        tr_makeMetaInfo( b, torrentPath, announce, announceCount, comment, isPrivate );
         while( !b->isDone )
         {
             tr_wait( 1000 );
@@ -355,7 +356,7 @@ main( int     argc,
     {
         tr_info info;
 
-        if( !tr_torrentParse( h, ctor, &info ) )
+        if( !tr_torrentParse( ctor, &info ) )
         {
             int          i;
             const time_t start = time( NULL );
@@ -391,7 +392,7 @@ main( int     argc,
     {
         tr_info info;
 
-        if( !tr_torrentParse( h, ctor, &info ) )
+        if( !tr_torrentParse( ctor, &info ) )
         {
             dumpInfo( stdout, &info );
             tr_metainfoFree( &info );
@@ -401,7 +402,7 @@ main( int     argc,
         goto cleanup;
     }
 
-    tor = tr_torrentNew( h, ctor, &error );
+    tor = tr_torrentNew( ctor, &error );
     tr_ctorFree( ctor );
     if( !tor )
     {
@@ -488,19 +489,22 @@ parseCommandLine( tr_benc * d, int argc, const char ** argv )
     {
         switch( c )
         {
-            case 'a':
-                announce = optarg; break;
-
-            case 'b': tr_bencDictAddInt( d, TR_PREFS_KEY_BLOCKLIST_ENABLED, 1 );
+            case 'a': if( announceCount + 1 < MAX_ANNOUNCE ) {
+                          announce[announceCount].tier = announceCount;
+                          announce[announceCount].announce = (char*) optarg;
+                          ++announceCount;
+                      }
                       break;
-            case 'B': tr_bencDictAddInt( d, TR_PREFS_KEY_BLOCKLIST_ENABLED, 0 );
+            case 'b': tr_bencDictAddBool( d, TR_PREFS_KEY_BLOCKLIST_ENABLED, TRUE );
+                      break;
+            case 'B': tr_bencDictAddBool( d, TR_PREFS_KEY_BLOCKLIST_ENABLED, FALSE );
                       break;
             case 'c': comment = optarg;
                       break;
-            case 'd': tr_bencDictAddInt( d, TR_PREFS_KEY_DSPEED, atoi( optarg ) );
-                      tr_bencDictAddInt( d, TR_PREFS_KEY_DSPEED_ENABLED, 1 );
+            case 'd': tr_bencDictAddInt ( d, TR_PREFS_KEY_DSPEED, atoi( optarg ) );
+                      tr_bencDictAddBool( d, TR_PREFS_KEY_DSPEED_ENABLED, TRUE );
                       break;
-            case 'D': tr_bencDictAddInt( d, TR_PREFS_KEY_DSPEED_ENABLED, 0 );
+            case 'D': tr_bencDictAddBool( d, TR_PREFS_KEY_DSPEED_ENABLED, FALSE );
                       break;
             case 'f': finishCall = optarg;
                       break;
@@ -508,9 +512,9 @@ parseCommandLine( tr_benc * d, int argc, const char ** argv )
                       break;
             case 'i': showInfo = 1;
                       break;
-            case 'm': tr_bencDictAddInt( d, TR_PREFS_KEY_PORT_FORWARDING, 1 );
+            case 'm': tr_bencDictAddBool( d, TR_PREFS_KEY_PORT_FORWARDING, TRUE );
                       break;
-            case 'M': tr_bencDictAddInt( d, TR_PREFS_KEY_PORT_FORWARDING, 0 );
+            case 'M': tr_bencDictAddBool( d, TR_PREFS_KEY_PORT_FORWARDING, FALSE );
                       break;
             case 'n': sourceFile = optarg; break;
             case 'p': tr_bencDictAddInt( d, TR_PREFS_KEY_PEER_PORT, atoi( optarg ) );
@@ -522,9 +526,9 @@ parseCommandLine( tr_benc * d, int argc, const char ** argv )
             case 't': tr_bencDictAddInt( d, TR_PREFS_KEY_PEER_SOCKET_TOS, atoi( optarg ) );
                       break;
             case 'u': tr_bencDictAddInt( d, TR_PREFS_KEY_USPEED, atoi( optarg ) );
-                      tr_bencDictAddInt( d, TR_PREFS_KEY_USPEED_ENABLED, 1 );
+                      tr_bencDictAddBool( d, TR_PREFS_KEY_USPEED_ENABLED, TRUE );
                       break;
-            case 'U': tr_bencDictAddInt( d, TR_PREFS_KEY_USPEED_ENABLED, 0 );
+            case 'U': tr_bencDictAddBool( d, TR_PREFS_KEY_USPEED_ENABLED, FALSE );
                       break;
             case 'v': verify = 1;
                       break;
