@@ -7,10 +7,12 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: remote.c 8044 2009-03-10 00:10:21Z charles $
+ * $Id: remote.c 8658 2009-06-09 15:45:09Z charles $
  */
 
+#include <ctype.h> /* isspace */
 #include <errno.h>
+#include <math.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h> /* strcmp */
@@ -21,7 +23,7 @@
  #include <unistd.h> /* getcwd */
 #endif
 
-#include <libevent/event.h>
+#include <event.h>
 
 #define CURL_DISABLE_TYPECHECK /* otherwise -Wunreachable-code goes insane */
 #include <curl/curl.h>
@@ -73,11 +75,16 @@ static tr_option opts[] =
     { 'i', "info",                 "Show the current torrent(s)' details", "i",  0, NULL },
     { 920, "session-info",         "Show the session's details", "si", 0, NULL },
     { 'l', "list",                 "List all torrents", "l",  0, NULL },
+    { 960, "move",                 "Move current torrent's data to a new folder", NULL, 1, "<path>" },
+    { 961, "find",                 "Tell Transmission where to find a torrent's data", NULL, 1, "<path>" },
     { 'm', "portmap",              "Enable portmapping via NAT-PMP or UPnP", "m",  0, NULL },
     { 'M', "no-portmap",           "Disable portmapping", "M",  0, NULL },
     { 'n', "auth",                 "Set authentication info", "n",  1, "<username:password>" },
     { 'N', "netrc",                "Set authentication info from a .netrc file", "N",  1, "<filename>" },
+    { 'o', "dht",                  "Enable distributed hash tables (DHT)", "o", 0, NULL },
+    { 'O', "no-dht",               "Disable distributed hash tables (DHT)", "O", 0, NULL },
     { 'p', "port",                 "Port for incoming peers (Default: " TR_DEFAULT_PEER_PORT_STR ")", "p", 1, "<port>" },
+    { 'P', "random-port",          "Random port for incomping peers", "P", 0, NULL },
     { 900, "priority-high",        "Set the files' priorities as high", "ph", 1, "<files>" },
     { 901, "priority-normal",      "Set the files' priorities as normal", "pn", 1, "<files>" },
     { 902, "priority-low",         "Set the files' priorities as low", "pl", 1, "<files>" },
@@ -85,6 +92,11 @@ static tr_option opts[] =
     { 930, "peers",                "Set the current torrent(s)' maximum number of peers each", "pr", 1, "<max>" },
     { 931, "global-peers",         "Set the global maximum number of peers", "gpr", 1, "<max>" },
     { 'R', "remove-and-delete",    "Remove the current torrent(s) and delete local data", NULL, 0, NULL },
+    { 950, "seedratio",            "Let the current torrent(s) seed until a specific ratio", "sr", 1, "ratio" },
+    { 951, "seedratio-default",    "Let the current torrent(s) use the global seedratio settings", "srd", 0, NULL },
+    { 952, "no-seedratio",         "Let the current torrent(s) seed regardless of ratio", "SR", 0, NULL },
+    { 953, "global-seedratio",     "All torrents, unless overridden by a per-torrent setting, should seed until a specific ratio", "gsr", 1, "ratio" },
+    { 954, "no-global-seedratio",  "All torrents, unless overridden by a per-torrent setting, should seed regardless of ratio", "GSR", 0, NULL },
     { 's', "start",                "Start the current torrent(s)", "s",  0, NULL },
     { 'S', "stop",                 "Stop the current torrent(s)", "S",  0, NULL },
     { 't', "torrent",              "Set the current torrent(s)", "t",  1, "<torrent>" },
@@ -103,7 +115,6 @@ static void
 showUsage( void )
 {
     tr_getopt_usage( MY_NAME, getUsage( ), opts );
-    exit( 0 );
 }
 
 static int
@@ -116,6 +127,7 @@ numarg( const char * arg )
     {
         fprintf( stderr, "Not a number: \"%s\"\n", arg );
         showUsage( );
+        exit( EXIT_FAILURE );
     }
     return num;
 }
@@ -125,6 +137,7 @@ static int    reqCount = 0;
 static int    debug = 0;
 static char * auth = NULL;
 static char * netrc = NULL;
+static char * sessionId = NULL;
 
 static char*
 tr_getcwd( void )
@@ -183,8 +196,16 @@ addIdArg( tr_benc *    args,
     }
     if( strcmp( id, "all" ) )
     {
-        tr_rpc_parse_list_str( tr_bencDictAdd( args,
-                                               "ids" ), id, strlen( id ) );
+        const char * pch;
+        tr_bool isList = strchr(id,',') || strchr(id,'-');
+        tr_bool isNum = TRUE;
+        for( pch=id; isNum && *pch; ++pch )
+            if( !isdigit( *pch ) )
+                isNum = FALSE;
+        if( isNum || isList )
+            tr_rpc_parse_list_str( tr_bencDictAdd( args, "ids" ), id, strlen( id ) );
+        else
+            tr_bencDictAddStr( args, "ids", id ); /* it's a torrent sha hash */
     }
 }
 
@@ -262,6 +283,7 @@ static const char * details_keys[] = {
     "timesCompleted",
     "totalSize",
     "uploadedEver",
+    "pieces",
     "webseeds",
     "webseedsSendingToUs"
 };
@@ -282,14 +304,15 @@ static const char * list_keys[] = {
     "uploadRatio"
 };
 
-static void
+static int
 readargs( int           argc,
           const char ** argv )
 {
-    int          c;
-    int          addingTorrents = 0;
+    int c;
+    int addingTorrents = 0;
+    int status = EXIT_SUCCESS;
+    char id[4096];
     const char * optarg;
-    char         id[4096];
 
     *id = '\0';
 
@@ -323,6 +346,7 @@ readargs( int           argc,
                 {
                     fprintf( stderr, "Unknown option: %s\n", optarg );
                     addArg = FALSE;
+                    status |= EXIT_FAILURE;
                 }
                 break;
 
@@ -339,12 +363,12 @@ readargs( int           argc,
             case 'd':
                 tr_bencDictAddStr( &top, "method", "session-set" );
                 tr_bencDictAddInt( args, "speed-limit-down", numarg( optarg ) );
-                tr_bencDictAddInt( args, "speed-limit-down-enabled", 1 );
+                tr_bencDictAddBool( args, "speed-limit-down-enabled", TRUE );
                 break;
 
             case 'D':
                 tr_bencDictAddStr( &top, "method", "session-set" );
-                tr_bencDictAddInt( args, "speed-limit-down-enabled", 0 );
+                tr_bencDictAddBool( args, "speed-limit-down-enabled", FALSE );
                 break;
 
             case 'f':
@@ -390,12 +414,12 @@ readargs( int           argc,
 
             case 'm':
                 tr_bencDictAddStr( &top, "method", "session-set" );
-                tr_bencDictAddInt( args, "port-forwarding-enabled", 1 );
+                tr_bencDictAddBool( args, "port-forwarding-enabled", TRUE );
                 break;
 
             case 'M':
                 tr_bencDictAddStr( &top, "method", "session-set" );
-                tr_bencDictAddInt( args, "port-forwarding-enabled", 0 );
+                tr_bencDictAddBool( args, "port-forwarding-enabled", FALSE );
                 break;
 
             case 'n':
@@ -410,7 +434,12 @@ readargs( int           argc,
 
             case 'p':
                 tr_bencDictAddStr( &top, "method", "session-set" );
-                tr_bencDictAddInt( args, "port", numarg( optarg ) );
+                tr_bencDictAddInt( args, TR_PREFS_KEY_PEER_PORT, numarg( optarg ) );
+                break;
+
+            case 'P':
+                tr_bencDictAddStr( &top, "method", "session-set" );
+                tr_bencDictAddBool( args, TR_PREFS_KEY_PEER_PORT_RANDOM_ON_START, TRUE);
                 break;
 
             case 'r':
@@ -421,7 +450,7 @@ readargs( int           argc,
             case 'R':
                 tr_bencDictAddStr( &top, "method", "torrent-remove" );
                 addIdArg( args, id );
-                tr_bencDictAddInt( args, "delete-local-data", 1 );
+                tr_bencDictAddBool( args, "delete-local-data", TRUE );
                 break;
 
             case 's':
@@ -442,12 +471,12 @@ readargs( int           argc,
             case 'u':
                 tr_bencDictAddStr( &top, "method", "session-set" );
                 tr_bencDictAddInt( args, "speed-limit-up", numarg( optarg ) );
-                tr_bencDictAddInt( args, "speed-limit-up-enabled", 1 );
+                tr_bencDictAddBool( args, "speed-limit-up-enabled", TRUE );
                 break;
 
             case 'U':
                 tr_bencDictAddStr( &top, "method", "session-set" );
-                tr_bencDictAddInt( args, "speed-limit-up-enabled", 0 );
+                tr_bencDictAddBool( args, "speed-limit-up-enabled", FALSE );
                 break;
 
             case 'v':
@@ -463,19 +492,29 @@ readargs( int           argc,
             case 'w': {
                 char * path = absolutify( optarg );
                 tr_bencDictAddStr( &top, "method", "session-set" );
-                tr_bencDictAddStr( args, "download-dir", path );
+                tr_bencDictAddStr( args, TR_PREFS_KEY_DOWNLOAD_DIR, path );
                 tr_free( path );
                 break;
             }
 
+            case 'o':
+                tr_bencDictAddStr( &top, "method", "session-set" );
+                tr_bencDictAddBool( args, TR_PREFS_KEY_DHT_ENABLED, TRUE );
+                break;
+
+            case 'O':
+                tr_bencDictAddStr( &top, "method", "session-set" );
+                tr_bencDictAddBool( args, TR_PREFS_KEY_DHT_ENABLED, FALSE );
+                break;
+
             case 'x':
                 tr_bencDictAddStr( &top, "method", "session-set" );
-                tr_bencDictAddInt( args, "pex-allowed", 1 );
+                tr_bencDictAddBool( args, TR_PREFS_KEY_PEX_ENABLED, TRUE );
                 break;
 
             case 'X':
                 tr_bencDictAddStr( &top, "method", "session-set" );
-                tr_bencDictAddInt( args, "pex-allowed", 0 );
+                tr_bencDictAddBool( args, TR_PREFS_KEY_PEX_ENABLED, FALSE );
                 break;
 
             case 900:
@@ -498,17 +537,17 @@ readargs( int           argc,
 
             case 910:
                 tr_bencDictAddStr( &top, "method", "session-set" );
-                tr_bencDictAddStr( args, "encryption", "required" );
+                tr_bencDictAddStr( args, TR_PREFS_KEY_ENCRYPTION, "required" );
                 break;
 
             case 911:
                 tr_bencDictAddStr( &top, "method", "session-set" );
-                tr_bencDictAddStr( args, "encryption", "preferred" );
+                tr_bencDictAddStr( args, TR_PREFS_KEY_ENCRYPTION, "preferred" );
                 break;
 
             case 912:
                 tr_bencDictAddStr( &top, "method", "session-set" );
-                tr_bencDictAddStr( args, "encryption", "tolerated" );
+                tr_bencDictAddStr( args, TR_PREFS_KEY_ENCRYPTION, "tolerated" );
                 break;
 
             case 920:
@@ -524,7 +563,7 @@ readargs( int           argc,
 
             case 931:
                 tr_bencDictAddStr( &top, "method", "session-set" );
-                tr_bencDictAddInt( args, "peer-limit", atoi(optarg) );
+                tr_bencDictAddInt( args, TR_PREFS_KEY_PEER_LIMIT_GLOBAL, atoi(optarg) );
                 break;
 
             case 940:
@@ -535,9 +574,54 @@ readargs( int           argc,
                 tr_bencListAddStr( fields, "peers" );
                 break;
 
+            case 950:
+                tr_bencDictAddStr( &top, "method", "torrent-set" );
+                tr_bencDictAddReal( args, "seedRatioLimit", atof(optarg) );
+                tr_bencDictAddInt( args, "seedRatioMode", TR_RATIOLIMIT_SINGLE );
+                addIdArg( args, id );
+                break;
+
+            case 951:
+                tr_bencDictAddStr( &top, "method", "torrent-set" );
+                tr_bencDictAddInt( args, "seedRatioMode", TR_RATIOLIMIT_GLOBAL );
+                addIdArg( args, id );
+                break;
+
+            case 952:
+                tr_bencDictAddStr( &top, "method", "torrent-set" );
+                tr_bencDictAddInt( args, "seedRatioMode", TR_RATIOLIMIT_UNLIMITED );
+                addIdArg( args, id );
+                break;
+
+            case 953:
+                tr_bencDictAddStr( &top, "method", "session-set" );
+                tr_bencDictAddReal( args, "seedRatioLimit", atof(optarg) );
+                tr_bencDictAddBool( args, "seedRatioLimited", TRUE );
+                break;
+
+            case 954:
+                tr_bencDictAddStr( &top, "method", "session-set" );
+                tr_bencDictAddBool( args, "seedRatioLimited", FALSE );
+                break;
+
+            case 960:
+                tr_bencDictAddStr( &top, "method", "torrent-set-location" );
+                tr_bencDictAddStr( args, "location", optarg );
+                tr_bencDictAddBool( args, "move", TRUE );
+                addIdArg( args, id );
+                break;
+
+            case 961:
+                tr_bencDictAddStr( &top, "method", "torrent-set-location" );
+                tr_bencDictAddStr( args, "location", optarg );
+                tr_bencDictAddBool( args, "move", FALSE );
+                addIdArg( args, id );
+                break;
+
             case TR_OPT_ERR:
                 fprintf( stderr, "invalid option\n" );
                 showUsage( );
+                status |= EXIT_FAILURE;
                 break;
 
             default:
@@ -548,21 +632,18 @@ readargs( int           argc,
 
         if( addArg )
         {
-            struct evbuffer * buf = tr_getBuffer( );
-            reqs[reqCount++] = tr_strdup( tr_bencSaveAsJSON( &top, buf ) );
-            tr_releaseBuffer( buf );
+            reqs[reqCount++] = tr_bencToStr( &top, TR_FMT_JSON_LEAN, NULL );
         }
 
         tr_bencFree( &top );
     }
+
+    return status;
 }
 
 /* [host:port] or [host] or [port] */
 static void
-getHostAndPort( int *   argc,
-                char ** argv,
-                char ** host,
-                int *   port )
+getHostAndPort( int * argc, char ** argv, char ** host, int * port )
 {
     if( *argv[1] != '-' )
     {
@@ -625,17 +706,7 @@ etaToString( char *  buf,
 static char*
 strlratio2( char * buf, double ratio, size_t buflen )
 {
-    if( (int)ratio == TR_RATIO_NA )
-        tr_strlcpy( buf, "None", buflen );
-    else if( (int)ratio == TR_RATIO_INF )
-        tr_strlcpy( buf, "Inf", buflen );
-    else if( ratio < 10.0 )
-        tr_snprintf( buf, buflen, "%'.2f", ratio );
-    else if( ratio < 100.0 )
-        tr_snprintf( buf, buflen, "%'.1f", ratio );
-    else
-        tr_snprintf( buf, buflen, "%'.0f", ratio );
-    return buf;
+    return tr_strratio( buf, buflen, ratio, "Inf" );
 }
 
 static char*
@@ -706,8 +777,8 @@ getStatusString( tr_benc * t, char * buf, size_t buflen )
                              ? "Will Verify"
                              : "Verifying";
             double percent;
-            if( tr_bencDictFindDouble( t, "recheckProgress", &percent ) )
-                tr_snprintf( buf, buflen, "%s (%.0f%%)", str, percent*100.0 );
+            if( tr_bencDictFindReal( t, "recheckProgress", &percent ) )
+                tr_snprintf( buf, buflen, "%s (%.0f%%)", str, floor(percent*100.0) );
             else
                 tr_strlcpy( buf, str, buflen );
 
@@ -716,23 +787,23 @@ getStatusString( tr_benc * t, char * buf, size_t buflen )
 
         case TR_STATUS_DOWNLOAD:
         case TR_STATUS_SEED: {
-            int64_t fromUs = 0;
-            int64_t toUs = 0;
-            tr_bencDictFindInt( t, "peersGettingFromUs", &fromUs );
-            tr_bencDictFindInt( t, "peersSendingToUs", &toUs );
-            if( fromUs && toUs )
-                tr_strlcpy( buf, "Up & Down", buflen );
-            else if( toUs )
-                tr_strlcpy( buf, "Downloading", buflen );
-            else if( fromUs ) {
-                int64_t leftUntilDone = 0;
-                tr_bencDictFindInt( t, "leftUntilDone", &leftUntilDone );
-                if( leftUntilDone > 0 )
-                    tr_strlcpy( buf, "Uploading", buflen );
-                else
-                    tr_strlcpy( buf, "Seeding", buflen );
-            } else {
-                tr_strlcpy( buf, "Idle", buflen );
+	    int64_t fromUs = 0; 
+	    int64_t toUs = 0; 
+ 	    tr_bencDictFindInt( t, "peersGettingFromUs", &fromUs ); 
+ 	    tr_bencDictFindInt( t, "peersSendingToUs", &toUs ); 
+ 	    if( fromUs && toUs ) 
+ 	        tr_strlcpy( buf, "Up & Down", buflen ); 
+ 	    else if( toUs ) 
+ 	        tr_strlcpy( buf, "Downloading", buflen ); 
+ 	    else if( fromUs ) { 
+ 	        int64_t leftUntilDone = 0; 
+ 	        tr_bencDictFindInt( t, "leftUntilDone", &leftUntilDone ); 
+ 	        if( leftUntilDone > 0 )
+ 	            tr_strlcpy( buf, "Uploading", buflen ); 
+ 	        else
+ 	            tr_strlcpy( buf, "Seeding", buflen ); 
+ 	    } else {
+ 	        tr_strlcpy( buf, "Idle", buflen ); 
             }
             break;
         }
@@ -761,6 +832,7 @@ printSession( tr_benc * top )
     {
         const char * str;
         int64_t      i;
+        tr_bool      boolVal;
 
         printf( "VERSION\n" );
         if( tr_bencDictFindStr( args,  "version", &str ) )
@@ -776,25 +848,73 @@ printSession( tr_benc * top )
             printf( "  Download directory: %s\n", str );
         if( tr_bencDictFindInt( args, "port", &i ) )
             printf( "  Listenport: %" PRId64 "\n", i );
-        if( tr_bencDictFindInt( args, "port-forwarding-enabled", &i ) )
-            printf( "  Portforwarding enabled: %s\n", ( i ? "Yes" : "No" ) );
-        if( tr_bencDictFindInt( args, "pex-allowed", &i ) )
-            printf( "  Peer exchange allowed: %s\n", ( i ? "Yes" : "No" ) );
+        if( tr_bencDictFindBool( args, "port-forwarding-enabled", &boolVal ) )
+            printf( "  Portforwarding enabled: %s\n", ( boolVal ? "Yes" : "No" ) );
+        if( tr_bencDictFindBool( args, "pex-allowed", &boolVal ) )
+            printf( "  Peer exchange allowed: %s\n", ( boolVal ? "Yes" : "No" ) );
         if( tr_bencDictFindStr( args,  "encryption", &str ) )
             printf( "  Encryption: %s\n", str );
         printf( "\n" );
 
-        printf( "LIMITS\n" );
-        if( tr_bencDictFindInt( args, "peer-limit", &i ) )
-            printf( "  Peer limit: %" PRId64 "\n", i );
-        if( tr_bencDictFindInt( args, "speed-limit-down-enabled", &i ) )
-            printf( "  Downloadlimit enabled: %s\n", ( i ? "Yes" : "No" ) );
-        if( tr_bencDictFindInt( args, "speed-limit-down", &i ) )
-            printf( "  Downloadlimit: %6" PRId64 " KB/sec\n", i );
-        if( tr_bencDictFindInt( args, "speed-limit-up-enabled", &i ) )
-            printf( "  Uploadlimit enabled:   %s\n", ( i ? "Yes" : "No" ) );
-        if( tr_bencDictFindInt( args, "speed-limit-up", &i ) )
-            printf( "  Uploadlimit:   %6" PRId64 " KB/sec\n", i );
+        {
+            tr_bool altEnabled, altTimeEnabled, upEnabled, downEnabled;
+            int64_t altDown, altUp, altBegin, altEnd, altDay, upLimit, downLimit, peerLimit;
+
+            if( tr_bencDictFindInt ( args, "alt-speed-down", &altDown ) &&
+                tr_bencDictFindBool( args, "alt-speed-enabled", &altEnabled ) &&
+                tr_bencDictFindInt ( args, "alt-speed-time-begin", &altBegin ) &&
+                tr_bencDictFindBool( args, "alt-speed-time-enabled", &altTimeEnabled ) &&
+                tr_bencDictFindInt ( args, "alt-speed-time-end", &altEnd ) &&
+                tr_bencDictFindInt ( args, "alt-speed-time-day", &altDay ) &&
+                tr_bencDictFindInt ( args, "alt-speed-up", &altUp ) &&
+                tr_bencDictFindInt ( args, "peer-limit-global", &peerLimit ) &&
+                tr_bencDictFindInt ( args, "speed-limit-down", &downLimit ) &&
+                tr_bencDictFindBool( args, "speed-limit-down-enabled", &downEnabled ) &&
+                tr_bencDictFindInt ( args, "speed-limit-up", &upLimit ) &&
+                tr_bencDictFindBool( args, "speed-limit-up-enabled", &upEnabled ) )
+            {
+                char buf[128];
+
+                printf( "LIMITS\n" );
+                printf( "  Peer limit: %" PRId64 "\n", peerLimit );
+
+                if( altEnabled )
+                    tr_snprintf( buf, sizeof( buf ), "%"PRId64" KB/s", altUp );
+                else if( upEnabled )
+                    tr_snprintf( buf, sizeof( buf ), "%"PRId64" KB/s", upLimit );
+                else
+                    tr_strlcpy( buf, "Unlimited", sizeof( buf ) );
+                printf( "  Upload speed limit: %s  (%s limit: %"PRId64" KB/s; %s turtle limit: %"PRId64" KB/s)\n",
+                        buf,
+                        (upEnabled?"Enabled":"Disabled"), upLimit,
+                        (altEnabled?"Enabled":"Disabled"), altUp );
+
+                if( altEnabled )
+                    tr_snprintf( buf, sizeof( buf ), "%"PRId64" KB/s", altDown );
+                else if( downEnabled )
+                    tr_snprintf( buf, sizeof( buf ), "%"PRId64" KB/s", downLimit );
+                else
+                    tr_strlcpy( buf, "unlimited", sizeof( buf ) );
+                printf( "  Download speed limit: %s  (%s limit: %"PRId64" KB/s; %s turtle limit: %"PRId64" KB/s)\n",
+                        buf,
+                        (downEnabled?"Enabled":"Disabled"), downLimit,
+                        (altEnabled?"Enabled":"Disabled"), altDown );
+
+                if( altTimeEnabled ) {
+                    printf( "  Turtle schedule: %02d:%02d - %02d:%02d  ",
+                            (int)(altBegin/60), (int)(altBegin%60),
+                            (int)(altEnd/60), (int)(altEnd%60) );
+                    if( altDay & TR_SCHED_SUN )   printf( "Sun " );
+                    if( altDay & TR_SCHED_MON )   printf( "Mon " );
+                    if( altDay & TR_SCHED_TUES )  printf( "Tue " );
+                    if( altDay & TR_SCHED_WED )   printf( "Wed " );
+                    if( altDay & TR_SCHED_THURS ) printf( "Thu " );
+                    if( altDay & TR_SCHED_FRI )   printf( "Fri " );
+                    if( altDay & TR_SCHED_SAT )   printf( "Sat " );
+                    printf( "\n" );
+                }
+            }
+        }
     }
 }
 
@@ -812,11 +932,14 @@ printDetails( tr_benc * top )
         {
             tr_benc *    t = tr_bencListChild( torrents, ti );
             tr_benc *    l;
+            const uint8_t * raw;
+            size_t       rawlen;
             const char * str;
             char         buf[512];
             char         buf2[512];
             int64_t      i, j, k;
             tr_bool      isStopped;
+            tr_bool      boolVal;
 
             isStopped = tr_bencDictFindInt( t, "status", &i ) && (i==TR_STATUS_STOPPED);
 
@@ -961,8 +1084,8 @@ printDetails( tr_benc * top )
                 const time_t tt = i;
                 printf( "  Date created: %s", ctime( &tt ) );
             }
-            if( tr_bencDictFindInt( t, "isPrivate", &i ) )
-                printf( "  Public torrent: %s\n", ( i ? "No" : "Yes" ) );
+            if( tr_bencDictFindBool( t, "isPrivate", &boolVal ) )
+                printf( "  Public torrent: %s\n", ( boolVal ? "No" : "Yes" ) );
             if( tr_bencDictFindStr( t, "comment", &str ) && str && *str )
                 printf( "  Comment: %s\n", str );
             if( tr_bencDictFindStr( t, "creator", &str ) && str && *str )
@@ -971,6 +1094,23 @@ printDetails( tr_benc * top )
                 printf( "  Piece Count: %" PRId64 "\n", i );
             if( tr_bencDictFindInt( t, "pieceSize", &i ) )
                 printf( "  Piece Size: %" PRId64 "\n", i );
+            printf( "\n" );
+
+            printf("PIECES\n  ");
+            if( tr_bencDictFindRaw( t, "pieces", &raw, &rawlen ) && tr_bencDictFindInt( t, "pieceCount", &j ) ) {
+                int len;
+                char * str = tr_base64_decode( raw, rawlen, &len );
+                for( i=k=0; k<len; ++k ) {
+                    int e;
+                    for( e=0; i<j && e<8; ++e, ++i )
+                        printf( str[k] & (1<<(7-e)) ? "1" : "0" );
+                    printf( " " );
+                    if( !(i%64) )
+                        printf( "\n  " );
+                }
+                tr_free( str );
+            }
+            printf( "\n" );
         }
     }
 }
@@ -1032,7 +1172,7 @@ printFileList( tr_benc * top )
                         }
                         printf( "%3d: %3.0f%% %-8s %-3s %9s  %s\n",
                                 j,
-                                ( 100.0 * percent ),
+                                floor( 100.0 * percent ),
                                 pristr,
                                 ( wanted ? "Yes" : "No" ),
                                 sizestr,
@@ -1059,7 +1199,7 @@ printPeersImpl( tr_benc * peers )
 
         if( tr_bencDictFindStr( d, "address", &address )
           && tr_bencDictFindStr( d, "clientName", &client )
-          && tr_bencDictFindDouble( d, "progress", &progress )
+          && tr_bencDictFindReal( d, "progress", &progress )
           && tr_bencDictFindStr( d, "flagStr", &flagstr )
           && tr_bencDictFindInt( d, "rateToClient", &rateToClient )
           && tr_bencDictFindInt( d, "rateToPeer", &rateToPeer ) )
@@ -1110,6 +1250,7 @@ printTorrentList( tr_benc * top )
         printf( "%-4s   %-4s  %9s  %-8s  %6s  %6s  %-5s  %-11s  %s\n",
                 "ID", "Done", "Have", "ETA", "Up", "Down", "Ratio", "Status",
                 "Name" );
+
         for( i = 0, n = tr_bencListSize( list ); i < n; ++i )
         {
             int64_t      id, eta, status, up, down;
@@ -1125,7 +1266,7 @@ printTorrentList( tr_benc * top )
               && tr_bencDictFindInt( d, "rateUpload", &up )
               && tr_bencDictFindInt( d, "sizeWhenDone", &sizeWhenDone )
               && tr_bencDictFindInt( d, "status", &status )
-              && tr_bencDictFindDouble( d, "uploadRatio", &ratio ) )
+              && tr_bencDictFindReal( d, "uploadRatio", &ratio ) )
             {
                 char etaStr[16];
                 char statusStr[64];
@@ -1167,28 +1308,32 @@ printTorrentList( tr_benc * top )
             }
         }
 
-        printf( "Sum:         %9s             %6.1f  %6.1f\n",
+        printf( "Sum:         %9s            %6.1f  %6.1f\n",
                 strlsize( haveStr, total_size, sizeof( haveStr ) ),
                 total_up / 1024.0,
                 total_down / 1024.0 );
     }
 }
 
-static void
+static int
 processResponse( const char * host,
                  int          port,
                  const void * response,
                  size_t       len )
 {
     tr_benc top;
+    int status = EXIT_SUCCESS;
 
     if( debug )
-        fprintf( stderr, "got response:\n--------\n%*.*s\n--------\n",
-                 (int)len, (int)len, (const char*) response );
+        fprintf( stderr, "got response (len %d):\n--------\n%*.*s\n--------\n",
+                 (int)len, (int)len, (int)len, (const char*) response );
 
-    if( tr_jsonParse( response, len, &top, NULL ) )
+    if( tr_jsonParse( NULL, response, len, &top, NULL ) )
+    {
         tr_nerr( MY_NAME, "Unable to parse response \"%*.*s\"", (int)len,
                  (int)len, (char*)response );
+        status |= EXIT_FAILURE;
+    }
     else
     {
         int64_t      tag = -1;
@@ -1213,64 +1358,132 @@ processResponse( const char * host,
                 printPeers( &top ); break;
 
             default:
-                if( tr_bencDictFindStr( &top, "result", &str ) )
+                if( !tr_bencDictFindStr( &top, "result", &str ) )
+                    status |= EXIT_FAILURE;
+                else {
                     printf( "%s:%d responded: \"%s\"\n", host, port, str );
+                    if( strcmp( str, "success") )
+                        status |= EXIT_FAILURE;
+                }
         }
 
         tr_bencFree( &top );
     }
+
+    return status;
 }
 
-static void
+/* look for a session id in the header in case the server gives back a 409 */
+static size_t
+parseResponseHeader( void *ptr, size_t size, size_t nmemb, void * stream UNUSED )
+{
+    const char * line = ptr;
+    const size_t line_len = size * nmemb;
+    const char * key = TR_RPC_SESSION_ID_HEADER ": ";
+    const size_t key_len = strlen( key );
+
+    if( ( line_len >= key_len ) && !memcmp( line, key, key_len ) )
+    {
+        const char * begin = line + key_len;
+        const char * end = begin;
+        while( !isspace( *end ) )
+            ++end;
+        tr_free( sessionId );
+        sessionId = tr_strndup( begin, end-begin );
+    }
+
+    return line_len;
+}
+
+static CURL*
+tr_curl_easy_init( struct evbuffer * writebuf )
+{
+    CURL * curl = curl_easy_init( );
+    curl_easy_setopt( curl, CURLOPT_USERAGENT, MY_NAME "/" LONG_VERSION_STRING );
+    curl_easy_setopt( curl, CURLOPT_WRITEFUNCTION, writeFunc );
+    curl_easy_setopt( curl, CURLOPT_WRITEDATA, writebuf );
+    curl_easy_setopt( curl, CURLOPT_HEADERFUNCTION, parseResponseHeader );
+    curl_easy_setopt( curl, CURLOPT_POST, 1 );
+    curl_easy_setopt( curl, CURLOPT_NETRC, CURL_NETRC_OPTIONAL );
+    curl_easy_setopt( curl, CURLOPT_HTTPAUTH, CURLAUTH_ANY );
+    curl_easy_setopt( curl, CURLOPT_TIMEOUT, 60L );
+    curl_easy_setopt( curl, CURLOPT_VERBOSE, debug );
+    curl_easy_setopt( curl, CURLOPT_ENCODING, "" ); /* "" tells curl to fill in the blanks with what it was compiled to support */
+    if( netrc )
+        curl_easy_setopt( curl, CURLOPT_NETRC_FILE, netrc );
+    if( auth )
+        curl_easy_setopt( curl, CURLOPT_USERPWD, auth );
+    if( sessionId ) {
+        char * h = tr_strdup_printf( "%s: %s", TR_RPC_SESSION_ID_HEADER, sessionId );
+        struct curl_slist * custom_headers = curl_slist_append( NULL, h );
+        curl_easy_setopt( curl, CURLOPT_HTTPHEADER, custom_headers );
+        /* fixme: leaks */
+    }
+    return curl;
+}
+    
+
+static int
 processRequests( const char *  host,
                  int           port,
                  const char ** reqs,
                  int           reqCount )
 {
-    int               i;
-    CURL *            curl;
+    int i;
+    CURL * curl = NULL;
     struct evbuffer * buf = evbuffer_new( );
-    char *            url = tr_strdup_printf(
-        "http://%s:%d/transmission/rpc", host, port );
+    char * url = tr_strdup_printf( "http://%s:%d/transmission/rpc", host, port );
+    int status = EXIT_SUCCESS;
 
-    curl = curl_easy_init( );
-    curl_easy_setopt( curl, CURLOPT_VERBOSE, debug );
-#ifdef HAVE_LIBZ
-    curl_easy_setopt( curl, CURLOPT_ENCODING, "deflate" );
-#endif
-    curl_easy_setopt( curl, CURLOPT_USERAGENT, MY_NAME "/" LONG_VERSION_STRING );
-    curl_easy_setopt( curl, CURLOPT_WRITEFUNCTION, writeFunc );
-    curl_easy_setopt( curl, CURLOPT_WRITEDATA, buf );
-    curl_easy_setopt( curl, CURLOPT_POST, 1 );
-    curl_easy_setopt( curl, CURLOPT_URL, url );
-    curl_easy_setopt( curl, CURLOPT_NETRC, CURL_NETRC_OPTIONAL );
-    curl_easy_setopt( curl, CURLOPT_HTTPAUTH, CURLAUTH_ANY );
-    curl_easy_setopt( curl, CURLOPT_TIMEOUT, 60L );
-    if( netrc )
-        curl_easy_setopt( curl, CURLOPT_NETRC_FILE, netrc );
-    if( auth )
-        curl_easy_setopt( curl, CURLOPT_USERPWD, auth );
-
-    for( i = 0; i < reqCount; ++i )
+    for( i=0; i<reqCount; ++i )
     {
         CURLcode res;
+        evbuffer_drain( buf, EVBUFFER_LENGTH( buf ) );
+
+        if( curl == NULL )
+        {
+            curl = tr_curl_easy_init( buf );
+            curl_easy_setopt( curl, CURLOPT_URL, url );
+        }
+
         curl_easy_setopt( curl, CURLOPT_POSTFIELDS, reqs[i] );
+
         if( debug )
             fprintf( stderr, "posting:\n--------\n%s\n--------\n", reqs[i] );
         if( ( res = curl_easy_perform( curl ) ) )
-            tr_nerr( MY_NAME, "(%s:%d) %s", host, port,
-                    curl_easy_strerror( res ) );
-        else
-            processResponse( host, port, EVBUFFER_DATA(
-                                buf ), EVBUFFER_LENGTH( buf ) );
-
-        evbuffer_drain( buf, EVBUFFER_LENGTH( buf ) );
+        {
+            tr_nerr( MY_NAME, "(%s:%d) %s", host, port, curl_easy_strerror( res ) );
+            status |= EXIT_FAILURE;
+        }
+        else {
+            long response;
+            curl_easy_getinfo( curl, CURLINFO_RESPONSE_CODE, &response );
+            switch( response ) {
+                case 200:
+                    status |= processResponse( host, port, EVBUFFER_DATA(buf), EVBUFFER_LENGTH(buf) );
+                    break;
+                case 409:
+                    /* session id failed.  our curl header func has already
+                     * pulled the new session id from this response's headers,
+                     * build a new CURL* and try again */
+                    curl_easy_cleanup( curl );
+                    curl = NULL;
+                    --i;
+                    break;
+                default:
+                    fprintf( stderr, "Unexpected response: %s\n", (char*)EVBUFFER_DATA(buf) );
+                    status |= EXIT_FAILURE;
+                    break;
+            }
+        }
     }
 
     /* cleanup */
     tr_free( url );
     evbuffer_free( buf );
-    curl_easy_cleanup( curl );
+    if( curl != NULL )
+        curl_easy_cleanup( curl );
+    return status;
 }
 
 int
@@ -1280,24 +1493,30 @@ main( int     argc,
     int    i;
     int    port = DEFAULT_PORT;
     char * host = NULL;
+    int    exit_status = EXIT_SUCCESS;
 
-    if( argc < 2 )
+    if( argc < 2 ) {
         showUsage( );
+        return EXIT_FAILURE;
+    }
 
     getHostAndPort( &argc, argv, &host, &port );
     if( host == NULL )
         host = tr_strdup( DEFAULT_HOST );
 
-    readargs( argc, (const char**)argv );
+    exit_status |= readargs( argc, (const char**)argv );
     if( reqCount )
-        processRequests( host, port, (const char**)reqs, reqCount );
-    else
+        exit_status = processRequests( host, port, (const char**)reqs, reqCount );
+    else {
         showUsage( );
+        return EXIT_FAILURE;
+    }
+        
 
-    for( i = 0; i < reqCount; ++i )
+    for( i=0; i<reqCount; ++i )
         tr_free( reqs[i] );
 
     tr_free( host );
-    return 0;
+    return exit_status;
 }
 
