@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: actions.c 8205 2009-04-10 17:44:17Z charles $
+ * $Id: actions.c 8420 2009-05-16 19:41:41Z charles $
  */
 
 #include <string.h>
@@ -20,6 +20,8 @@
 #include "tr-prefs.h"
 #include "lock.h"
 #include "logo.h"
+#include "options-icon.h"
+#include "turtles.h"
 
 #define UNUSED G_GNUC_UNUSED
 
@@ -50,7 +52,9 @@ static GtkRadioActionEntry sort_radio_entries[] =
     { "sort-by-ratio",    NULL, N_( "Sort by _Ratio" ),    NULL, NULL, 3 },
     { "sort-by-state",    NULL, N_( "Sort by _State" ),    NULL, NULL, 4 },
     { "sort-by-tracker",  NULL, N_( "Sort by _Tracker" ),  NULL, NULL, 5 },
-    { "sort-by-age",      NULL, N_( "Sort by A_ge" ),      NULL, NULL, 6 }
+    { "sort-by-age",      NULL, N_( "Sort by A_ge" ),      NULL, NULL, 6 },
+    { "sort-by-eta",      NULL, N_( "Sort by _ETA" ),      NULL, NULL, 7 },
+    { "sort-by-size",     NULL, N_( "Sort by Si_ze" ),     NULL, NULL, 8 }
 };
 
 static void
@@ -83,21 +87,15 @@ toggle_pref_cb( GtkToggleAction *  action,
 
 static GtkToggleActionEntry  pref_toggle_entries[] =
 {
-    { "minimal-view",          NULL,
-      N_( "_Minimal View" ), "<alt>M", NULL, G_CALLBACK( toggle_pref_cb ),
-      FALSE },
-    { "sort-reversed",         NULL,
-      N_( "_Reverse Sort Order" ), NULL, NULL, G_CALLBACK( toggle_pref_cb ),
-      FALSE },
-    { "show-filterbar",        NULL,
-      N_( "_Filterbar" ), NULL, NULL, G_CALLBACK( toggle_pref_cb ), FALSE },
-    { "show-statusbar",        NULL,
-      N_( "_Statusbar" ), NULL, NULL, G_CALLBACK( toggle_pref_cb ), FALSE },
-    { "show-toolbar",          NULL,
-      N_( "_Toolbar" ), NULL, NULL, G_CALLBACK( toggle_pref_cb ), FALSE }
+    { "alt-speed-enabled", NULL, N_( "Speed _Limit Mode" ), NULL, NULL, G_CALLBACK( toggle_pref_cb ), FALSE },
+    { "minimal-view",      NULL, N_( "_Minimal View" ), "<alt>M", NULL, G_CALLBACK( toggle_pref_cb ), FALSE },
+    { "sort-reversed",     NULL, N_( "_Reverse Sort Order" ), NULL, NULL, G_CALLBACK( toggle_pref_cb ), FALSE },
+    { "show-filterbar",    NULL, N_( "_Filterbar" ), NULL, NULL, G_CALLBACK( toggle_pref_cb ), FALSE },
+    { "show-statusbar",    NULL, N_( "_Statusbar" ), NULL, NULL, G_CALLBACK( toggle_pref_cb ), FALSE },
+    { "show-toolbar",      NULL, N_( "_Toolbar" ), NULL, NULL, G_CALLBACK( toggle_pref_cb ), FALSE }
 };
 
-static GtkActionEntry        entries[] =
+static GtkActionEntry entries[] =
 {
     { "torrent-menu", NULL, N_( "_Torrent" ), NULL, NULL, NULL  },
     { "view-menu", NULL, N_( "_View" ), NULL, NULL, NULL  },
@@ -112,6 +110,7 @@ static GtkActionEntry        entries[] =
     { "pause-torrent", GTK_STOCK_MEDIA_PAUSE, N_( "_Pause" ), "<control>P", N_( "Pause torrent" ), G_CALLBACK( action_cb ) },
     { "pause-all-torrents", GTK_STOCK_MEDIA_PAUSE, N_( "_Pause All" ), NULL, N_( "Pause all torrents" ), G_CALLBACK( action_cb ) },
     { "start-all-torrents", GTK_STOCK_MEDIA_PLAY, N_( "_Start All" ), NULL, N_( "Start all torrents" ), G_CALLBACK( action_cb ) },
+    { "relocate-torrent", NULL, N_("Set _Location" ), NULL, NULL, G_CALLBACK( action_cb ) },
     { "remove-torrent", GTK_STOCK_REMOVE, NULL, "Delete", N_( "Remove torrent" ), G_CALLBACK( action_cb ) },
     { "delete-torrent", GTK_STOCK_DELETE, N_( "_Delete Files and Remove" ), "<shift>Delete", NULL, G_CALLBACK( action_cb ) },
     { "new-torrent", GTK_STOCK_NEW, N_( "_New..." ), NULL, N_( "Create a torrent" ), G_CALLBACK( action_cb ) },
@@ -137,7 +136,10 @@ BuiltinIconInfo;
 static const BuiltinIconInfo my_fallback_icons[] =
 {
     { tr_icon_logo, "transmission"      },
-    { tr_icon_lock, "transmission-lock" }
+    { tr_icon_lock, "transmission-lock" },
+    { options_icon, "options"           },
+    { blue_turtle,  "alt-speed-on"      },
+    { grey_turtle,  "alt-speed-off"     }
 };
 
 static void
