@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: conf.h 8048 2009-03-10 15:24:41Z charles $
+ * $Id: conf.h 8047 2009-03-10 15:24:40Z charles $
  *
  * Copyright (c) 2005-2008 Transmission authors and contributors
  *
@@ -31,6 +31,9 @@
 int64_t          pref_int_get           ( const char * key );
 void             pref_int_set           ( const char * key, int64_t value );
 
+double           pref_double_get        ( const char * key );
+void             pref_double_set        ( const char * key, double value );
+
 gboolean         pref_flag_get          ( const char * key );
 void             pref_flag_set          ( const char * key, gboolean value );
 
