@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: tr-core.h 6978 2008-10-28 19:49:33Z charles $
+ * $Id: tr-core.h 8671 2009-06-11 16:17:48Z charles $
  *
  * Copyright (c) 2007-2008 Transmission authors and contributors
  *
@@ -64,26 +64,30 @@ typedef struct TrCoreClass
 {
     GObjectClass    parent;
 
-    /* "blocklist" signal:
-       void  handler( TrCore *, const char *, gpointer userData ); */
-    int    blocksig;
+    /* "blocklist-updated" signal with a callback type of
+        void (*callback )( TrCore*, int ruleCount, gpointer userData ). */
+    int blocklistSignal;
 
-    /* "error" signal:
-       void handler( TrCore *, enum tr_core_err, const char *, gpointer ) */
-    int    errsig;
+    /* "port-tested" signal with a callback type of
+       void( *callback )( TrCore*, gboolean isOpen, gpointer userData ). */
+    int portSignal;
 
-    /* "add-torrent-prompt" signal:
-       void handler( TrCore *, gpointer ctor, gpointer userData )
+    /* "error" signal with a callback type of
+       void( *callback )( TrCore*, enum tr_core_err, const char * humanReadable, gpointer userData ). */
+    int errsig;
+
+    /* "add-torrent-prompt" signal with a callback type of
+       void ( *callback)( TrCore *, gpointer ctor, gpointer userData )
        The handler assumes ownership of ctor and must free when done */
-    int    promptsig;
+    int promptsig;
 
     /* "quit" signal:
        void handler( TrCore *, gpointer ) */
-    int    quitsig;
+    int quitsig;
 
     /* "prefs-changed" signal:
        void handler( TrCore *, int, gpointer ) */
-    int    prefsig;
+    int prefsig;
 }
 TrCoreClass;
 
@@ -130,30 +134,25 @@ int      tr_core_load( TrCore * self,
 void     tr_core_add_list( TrCore *    self,
                            GSList *    torrentFiles,
                            pref_flag_t start,
-                           pref_flag_t prompt );
+                           pref_flag_t prompt,
+                           gboolean    doNotify );
 
-#define tr_core_add_list_defaults( c, l ) \
-    tr_core_add_list( c, l, PREF_FLAG_DEFAULT, PREF_FLAG_DEFAULT )
+#define tr_core_add_list_defaults( c, l, doNotify ) \
+    tr_core_add_list( c, l, PREF_FLAG_DEFAULT, PREF_FLAG_DEFAULT, doNotify )
 
 
+/** @brief Add a torrent. */
+gboolean tr_core_add_metainfo( TrCore      * core,
+                               const char  * base64_metainfo,
+                               gboolean    * setme_success,
+                               GError     ** err );
+
 /** Add a torrent. */
-gboolean tr_core_add_file(
-                 TrCore*,
-    const char * filename,
-    gboolean *
-                 setme_success,
-    GError **    err );
+void tr_core_add_torrent( TrCore*, TrTorrent*, gboolean doNotify );
 
 /** Present the main window */
-gboolean tr_core_present_window(
-              TrCore*,
-    gboolean *
-              setme_success,
-    GError ** err );
+gboolean tr_core_present_window( TrCore*, gboolean * setme_success, GError ** err ); 
 
-/** Add a torrent. */
-void     tr_core_add_torrent( TrCore*,
-                              TrTorrent* );
 
 /**
  * Notifies listeners that torrents have been added.
@@ -167,42 +166,37 @@ void     tr_core_torrents_added( TrCore * self );
 
 /* we've gotten notice from RPC that a torrent has been destroyed;
    update our gui accordingly */
-void     tr_core_torrent_destroyed( TrCore * self,
-                                    int      id );
+void  tr_core_torrent_destroyed( TrCore * self, int torrentId );
 
 /* remove a torrent */
-void     tr_core_remove_torrent( TrCore *    self,
-                                 TrTorrent * gtor,
-                                 int         deleteFiles );
+void  tr_core_remove_torrent( TrCore * self, TrTorrent * gtor, int deleteFiles );
 
 /* update the model with current torrent status */
-void     tr_core_update( TrCore * self );
+void  tr_core_update( TrCore * self );
 
 /* emit the "quit" signal */
-void     tr_core_quit( TrCore * self );
+void  tr_core_quit( TrCore * self );
 
-/* emit the "blocklist changed" signal */
-void     tr_core_blocksig( TrCore *     core,
-                           gboolean     isDone,
-                           const char * status );
+/**
+***  Set a preference value, save the prefs file, and emit the "prefs-changed" signal
+**/
 
-/* Set a preference value, save the prefs file, and emit the
-   "prefs-changed" signal */
-void     tr_core_set_pref( TrCore *     self,
-                           const char * key,
-                           const char * val );
+void tr_core_set_pref     ( TrCore * self, const char * key, const char * val );
+void tr_core_set_pref_bool( TrCore * self, const char * key, gboolean val );
+void tr_core_set_pref_int ( TrCore * self, const char * key, int val );
+void tr_core_set_pref_double( TrCore * self, const char * key, double val );
 
-/* Set a boolean preference value, save the prefs file, and emit the
-   "prefs-changed" signal */
-void     tr_core_set_pref_bool( TrCore *     self,
-                                const char * key,
-                                gboolean     val );
+/**
+***
+**/
 
-/* Set an integer preference value, save the prefs file, and emit the
-   "prefs-changed" signal */
-void     tr_core_set_pref_int( TrCore *     self,
-                               const char * key,
-                               int          val );
+void tr_core_port_test( TrCore * core );
+
+void tr_core_blocklist_update( TrCore * core );
+
+void tr_core_exec( TrCore * core, const tr_benc * benc );
+
+void tr_core_exec_json( TrCore * core, const char * json );
 
 
 /**
