@@ -7,12 +7,13 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: tr-prefs.c 8143 2009-04-05 14:19:27Z charles $
+ * $Id: tr-prefs.c 8671 2009-06-11 16:17:48Z charles $
  */
 
 #include <ctype.h> /* isspace */
 #include <errno.h>
 #include <stdarg.h>
+#include <limits.h> /* USHRT_MAX */
 #include <stdlib.h> /* free() */
 #include <unistd.h>
 #include <glib/gi18n.h>
@@ -21,7 +22,6 @@
 #include <libtransmission/utils.h>
 #include <libtransmission/version.h>
 #include <libtransmission/web.h>
-#include "blocklist.h"
 #include "conf.h"
 #include "hig.h"
 #include "tr-core.h"
@@ -83,6 +83,7 @@ struct spin_idle_data
 {
     gpointer    core;
     GTimer *    last_change;
+    gboolean    isDouble;
 };
 
 static void
@@ -106,9 +107,17 @@ spun_cb_idle( gpointer spin )
     {
         /* update the core */
         const char * key = g_object_get_data( o, PREF_KEY );
-        const int    value = gtk_spin_button_get_value_as_int(
-             GTK_SPIN_BUTTON( spin ) );
-        tr_core_set_pref_int( TR_CORE( data->core ), key, value );
+        if (data->isDouble)
+        {
+            const double value = gtk_spin_button_get_value( GTK_SPIN_BUTTON( spin ) );
+            tr_core_set_pref_double( TR_CORE( data->core ), key, value );
+        }
+        else
+        {
+            const int    value = gtk_spin_button_get_value_as_int(
+                                 GTK_SPIN_BUTTON( spin ) );
+            tr_core_set_pref_int( TR_CORE( data->core ), key, value );
+        }
 
         /* cleanup */
         g_object_set_data( o, IDLE_DATA, NULL );
@@ -121,7 +130,8 @@ spun_cb_idle( gpointer spin )
 
 static void
 spun_cb( GtkSpinButton * w,
-         gpointer        core )
+         gpointer        core,
+         gboolean        isDouble )
 {
     /* user may be spinning through many values, so let's hold off
        for a moment to keep from flooding the core with changes */
@@ -133,6 +143,7 @@ spun_cb( GtkSpinButton * w,
         data = g_new( struct spin_idle_data, 1 );
         data->core = core;
         data->last_change = g_timer_new( );
+        data->isDouble = isDouble;
         g_object_set_data_full( o, IDLE_DATA, data, spin_idle_data_free );
         g_object_ref( G_OBJECT( o ) );
         g_timeout_add( 100, spun_cb_idle, w );
@@ -140,6 +151,20 @@ spun_cb( GtkSpinButton * w,
     g_timer_start( data->last_change );
 }
 
+static void
+spun_cb_int( GtkSpinButton * w,
+             gpointer        core )
+{
+    spun_cb( w, core, FALSE );
+}
+
+static void
+spun_cb_double( GtkSpinButton * w,
+                gpointer        core )
+{
+    spun_cb( w, core, TRUE );
+}
+
 static GtkWidget*
 new_spin_button( const char * key,
                  gpointer     core,
@@ -153,7 +178,24 @@ new_spin_button( const char * key,
                                 key ), g_free );
     gtk_spin_button_set_digits( GTK_SPIN_BUTTON( w ), 0 );
     gtk_spin_button_set_value( GTK_SPIN_BUTTON( w ), pref_int_get( key ) );
-    g_signal_connect( w, "value-changed", G_CALLBACK( spun_cb ), core );
+    g_signal_connect( w, "value-changed", G_CALLBACK( spun_cb_int ), core );
+    return w;
+}
+
+static GtkWidget*
+new_spin_button_double( const char * key,
+                       gpointer      core,
+                       double        low,
+                       double        high,
+                       double        step )
+{
+    GtkWidget * w = gtk_spin_button_new_with_range( low, high, step );
+
+    g_object_set_data_full( G_OBJECT( w ), PREF_KEY, g_strdup(
+                                key ), g_free );
+    gtk_spin_button_set_digits( GTK_SPIN_BUTTON( w ), 2 );
+    gtk_spin_button_set_value( GTK_SPIN_BUTTON( w ), pref_double_get( key ) );
+    g_signal_connect( w, "value-changed", G_CALLBACK( spun_cb_double ), core );
     return w;
 }
 
@@ -229,6 +271,7 @@ torrentPage( GObject * core )
     const char * s;
     GtkWidget *  t;
     GtkWidget *  w;
+    GtkWidget *  w2;
 
 #ifdef HAVE_GIO
     GtkWidget *  l;
@@ -262,6 +305,16 @@ torrentPage( GObject * core )
     w = new_path_chooser_button( TR_PREFS_KEY_DOWNLOAD_DIR, core );
     hig_workarea_add_row( t, &row, _( "_Destination folder:" ), w, NULL );
 
+    hig_workarea_add_section_divider( t, &row );
+    hig_workarea_add_section_title( t, &row, _( "Limits" ) );
+
+    s = _( "_Stop seeding torrents at ratio:" );
+    w = new_check_button( s, TR_PREFS_KEY_RATIO_ENABLED, core );
+    w2 = new_spin_button_double( TR_PREFS_KEY_RATIO, core, 0, INT_MAX, .05 );
+    gtk_widget_set_sensitive( GTK_WIDGET( w2 ), pref_flag_get( TR_PREFS_KEY_RATIO_ENABLED ) );
+    g_signal_connect( w, "toggled", G_CALLBACK( target_cb ), w2 );
+    hig_workarea_add_row_w( t, &row, w, w2, NULL );
+
     hig_workarea_finish( t, &row );
     return t;
 }
@@ -303,102 +356,134 @@ desktopPage( GObject * core )
 
 struct blocklist_data
 {
-    GtkWidget *  check;
-    GtkWidget *  dialog;
-    TrCore *     core;
-    gulong       id;
-    int          abortFlag;
-    char         secondary[256];
+    gulong      updateBlocklistTag;
+    GtkWidget * updateBlocklistButton;
+    GtkWidget * updateBlocklistDialog;
+    GtkWidget * check;
+    TrCore    * core;
 };
 
 static void
-updateBlocklistText( GtkWidget * w,
-                     TrCore *    core )
+updateBlocklistText( GtkWidget * w, TrCore * core )
 {
     const int n = tr_blocklistGetRuleCount( tr_core_session( core ) );
     char      buf[512];
-
     g_snprintf( buf, sizeof( buf ),
                 ngettext( "Enable _blocklist (contains %'d rule)",
                           "Enable _blocklist (contains %'d rules)", n ), n );
     gtk_button_set_label( GTK_BUTTON( w ), buf );
 }
 
+/* prefs dialog is being destroyed, so stop listening to blocklist updates */
 static void
-onBlocklistDialogResponse( GtkDialog *  d,
-                           int response UNUSED,
-                           gpointer     gdata )
+peerPageDestroyed( gpointer gdata, GObject * dead UNUSED )
 {
     struct blocklist_data * data = gdata;
-
-    g_signal_handler_disconnect( data->core, data->id );
-    gtk_widget_destroy( GTK_WIDGET( d ) );
+    if( data->updateBlocklistTag > 0 )
+        g_signal_handler_disconnect( data->core, data->updateBlocklistTag );
+    g_free( data );
 }
 
+/* user hit "close" in the blocklist-update dialog */
 static void
-onBlocklistStatus( TrCore * core UNUSED,
-                   gboolean      isDone,
-                   const char *  status,
-                   gpointer      gdata )
+onBlocklistUpdateResponse( GtkDialog * dialog, gint response UNUSED, gpointer gdata )
 {
     struct blocklist_data * data = gdata;
-
-    gdk_threads_enter( );
-    gtk_message_dialog_format_secondary_text( GTK_MESSAGE_DIALOG( data-> dialog ),
-                                              "%s", status );
-    gtk_dialog_set_response_sensitive( GTK_DIALOG( data->dialog ),
-                                       GTK_RESPONSE_CANCEL, !isDone );
-    gtk_dialog_set_response_sensitive( GTK_DIALOG( data->dialog ),
-                                       GTK_RESPONSE_CLOSE, isDone );
-    if( isDone )
-        updateBlocklistText( data->check, core );
-    gdk_threads_leave( );
+    gtk_widget_destroy( GTK_WIDGET( dialog ) );
+    gtk_widget_set_sensitive( data->updateBlocklistButton, TRUE );
+    data->updateBlocklistDialog = NULL;
+    g_signal_handler_disconnect( data->core, data->updateBlocklistTag );
 }
 
+/* core says the blocklist was updated */
 static void
-onUpdateBlocklistCB( GtkButton * w,
-                     gpointer    gdata )
+onBlocklistUpdated( TrCore * core, int n, gpointer gdata )
 {
-    GtkWidget *             d;
+    const char * s = ngettext( "Blocklist now has %'d rule.", "Blocklist now has %'d rules.", n );
     struct blocklist_data * data = gdata;
+    GtkMessageDialog * d = GTK_MESSAGE_DIALOG( data->updateBlocklistDialog );
+    gtk_widget_set_sensitive( data->updateBlocklistButton, TRUE );
+    gtk_message_dialog_set_markup( d, _( "<b>Update succeeded!</b>" ) );
+    gtk_message_dialog_format_secondary_text( d, s, n );
+    updateBlocklistText( data->check, core );
+}
 
-    d =
-        gtk_message_dialog_new( GTK_WINDOW( gtk_widget_get_toplevel( 
-                                                                    GTK_WIDGET(
-                                                                        w ) ) ),
-                               GTK_DIALOG_DESTROY_WITH_PARENT,
-                               GTK_MESSAGE_INFO,
-                               GTK_BUTTONS_NONE,
-                               _( "Updating Blocklist" ) );
-
-    data->dialog = d;
-    data->id =
-        g_signal_connect( data->core, "blocklist-status", G_CALLBACK(
-                              onBlocklistStatus ), data );
-
-    gtk_dialog_add_buttons( GTK_DIALOG( d ),
-                            GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-                            GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
-                            NULL );
-    gtk_dialog_set_response_sensitive( GTK_DIALOG(
-                                           d ), GTK_RESPONSE_CLOSE, FALSE );
-
-    g_signal_connect( d, "response", G_CALLBACK(
-                          onBlocklistDialogResponse ), data );
+/* user pushed a button to update the blocklist */
+static void
+onBlocklistUpdate( GtkButton * w, gpointer gdata )
+{
+    GtkWidget * d;
+    struct blocklist_data * data = gdata;
+    d = gtk_message_dialog_new( GTK_WINDOW( gtk_widget_get_toplevel( GTK_WIDGET( w ) ) ),
+                                GTK_DIALOG_DESTROY_WITH_PARENT,
+                                GTK_MESSAGE_INFO,
+                                GTK_BUTTONS_CLOSE,
+                               _( "Update Blocklist" ) );
+    gtk_widget_set_sensitive( data->updateBlocklistButton, FALSE );
+    gtk_message_dialog_format_secondary_text( GTK_MESSAGE_DIALOG( d ), _( "Getting new blocklist..." ) );
+    data->updateBlocklistDialog = d;
+    g_signal_connect( d, "response", G_CALLBACK(onBlocklistUpdateResponse), data );
     gtk_widget_show( d );
-
-    gtr_blocklist_update( data->core );
+    tr_core_blocklist_update( data->core );
+    data->updateBlocklistTag = g_signal_connect( data->core, "blocklist-updated", G_CALLBACK( onBlocklistUpdated ), data );
 }
 
 static void
-onEncryptionToggled( GtkToggleButton * w,
-                     gpointer          core )
+onIntComboChanged( GtkComboBox * w, gpointer core )
 {
-    const int val = gtk_toggle_button_get_active( w )
-                    ? TR_ENCRYPTION_REQUIRED
-                    : TR_ENCRYPTION_PREFERRED;
+    GtkTreeIter iter;
 
-    tr_core_set_pref_int( TR_CORE( core ), TR_PREFS_KEY_ENCRYPTION, val );
+    if( gtk_combo_box_get_active_iter( w, &iter ) )
+    {
+        int val = 0;
+        const char * key = g_object_get_data( G_OBJECT( w ), PREF_KEY );
+        gtk_tree_model_get( gtk_combo_box_get_model( w ), &iter, 0, &val, -1 );
+        tr_core_set_pref_int( TR_CORE( core ), key, val );
+    }
+}
+
+static GtkWidget*
+new_encryption_combo( GObject * core, const char * key )
+{
+    int i;
+    int selIndex;
+    GtkWidget * w;
+    GtkCellRenderer * r;
+    GtkListStore * store;
+    const int currentValue = pref_int_get( key );
+    const struct {
+        int value;
+        const char * text;
+    } items[] = {
+        { TR_CLEAR_PREFERRED,      N_( "Plaintext Preferred" )  },
+        { TR_ENCRYPTION_PREFERRED, N_( "Encryption Preferred" ) },
+        { TR_ENCRYPTION_REQUIRED,  N_( "Encryption Required" )  }
+    };
+
+    /* build a store for encryption */
+    selIndex = -1;
+    store = gtk_list_store_new( 2, G_TYPE_INT, G_TYPE_STRING );
+    for( i=0; i<(int)G_N_ELEMENTS(items); ++i ) {
+        GtkTreeIter iter;
+        gtk_list_store_append( store, &iter );
+        gtk_list_store_set( store, &iter, 0, items[i].value, 1, _( items[i].text ), -1 );
+        if( items[i].value == currentValue )
+            selIndex = i;
+    }
+
+    /* build the widget */
+    w = gtk_combo_box_new_with_model( GTK_TREE_MODEL( store ) );
+    r = gtk_cell_renderer_text_new( );
+    gtk_cell_layout_pack_start( GTK_CELL_LAYOUT( w ), r, TRUE );
+    gtk_cell_layout_set_attributes( GTK_CELL_LAYOUT( w ), r, "text", 1, NULL );
+    g_object_set_data_full( G_OBJECT( w ), PREF_KEY, tr_strdup( key ), g_free );
+    if( selIndex >= 0 )
+        gtk_combo_box_set_active( GTK_COMBO_BOX( w ), selIndex );
+    g_signal_connect( w, "changed", G_CALLBACK( onIntComboChanged ), core );
+
+    /* cleanup */
+    g_object_unref( G_OBJECT( store ) );
+    return w;
 }
 
 static GtkWidget*
@@ -422,11 +507,11 @@ peerPage( GObject * core )
     updateBlocklistText( w, TR_CORE( core ) );
     h = gtk_hbox_new( FALSE, GUI_PAD_BIG );
     gtk_box_pack_start( GTK_BOX( h ), w, TRUE, TRUE, 0 );
-    b = gtr_button_new_from_stock( GTK_STOCK_REFRESH, _( "_Update" ) );
+    b = data->updateBlocklistButton = gtr_button_new_from_stock( GTK_STOCK_REFRESH, _( "_Update" ) );
     data->check = w;
     g_object_set_data( G_OBJECT( b ), "session",
                       tr_core_session( TR_CORE( core ) ) );
-    g_signal_connect( b, "clicked", G_CALLBACK( onUpdateBlocklistCB ), data );
+    g_signal_connect( b, "clicked", G_CALLBACK( onBlocklistUpdate ), data );
     gtk_box_pack_start( GTK_BOX( h ), b, FALSE, FALSE, 0 );
     g_signal_connect( w, "toggled", G_CALLBACK( target_cb ), b );
     target_cb( w, b );
@@ -447,21 +532,26 @@ peerPage( GObject * core )
     hig_workarea_add_row( t, &row, _( "Maximum peers per _torrent:" ), w, NULL );
 
     hig_workarea_add_section_divider( t, &row );
-    hig_workarea_add_section_title ( t, &row, _( "Options" ) );
+    hig_workarea_add_section_title ( t, &row, _( "Privacy" ) );
 
-    s = _( "_Ignore unencrypted peers" );
-    w = gtk_check_button_new_with_mnemonic( s );
-    gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( w ),
-                                  pref_int_get( TR_PREFS_KEY_ENCRYPTION ) ==
-                                  TR_ENCRYPTION_REQUIRED );
-    g_signal_connect( w, "toggled", G_CALLBACK( onEncryptionToggled ), core );
+    s = _( "_Encryption mode" );
+    w = new_encryption_combo( core, "encryption" );
+    hig_workarea_add_row( t, &row, s, w, NULL );
+
+    s = _( "Use PE_X to find more peers" );
+    w = new_check_button( s, TR_PREFS_KEY_PEX_ENABLED, core );
+    s = _( "PEX is a tool for exchanging peer lists with the peers you're connected to." );
+    gtr_widget_set_tooltip_text( w, s );
     hig_workarea_add_wide_control( t, &row, w );
 
-    s = _( "Use peer e_xchange" );
-    w = new_check_button( s, TR_PREFS_KEY_PEX_ENABLED, core );
+    s = _( "Use _DHT to find more peers" );
+    w = new_check_button( s, TR_PREFS_KEY_DHT_ENABLED, core );
+    s = _( "DHT is a tool for finding peers without a tracker." );
+    gtr_widget_set_tooltip_text( w, s );
     hig_workarea_add_wide_control( t, &row, w );
 
     hig_workarea_finish( t, &row );
+    g_object_weak_ref( G_OBJECT( t ), peerPageDestroyed, data );
     return t;
 }
 
@@ -682,7 +772,7 @@ webPage( GObject * core )
     hig_workarea_add_wide_control( t, &row, h );
 
     /* port */
-    w = new_spin_button( TR_PREFS_KEY_RPC_PORT, core, 0, 65535, 1 );
+    w = new_spin_button( TR_PREFS_KEY_RPC_PORT, core, 0, USHRT_MAX, 1 );
     page->widgets = g_slist_append( page->widgets, w );
     w = hig_workarea_add_row( t, &row, _( "Listening _port:" ), w, NULL );
     page->widgets = g_slist_append( page->widgets, w );
@@ -887,7 +977,7 @@ trackerPage( GObject * core )
     w = hig_workarea_add_row( t, &row, s, w, NULL );
     page->proxy_widgets = g_slist_append( page->proxy_widgets, w );
 
-    w = new_spin_button( TR_PREFS_KEY_PROXY_PORT, core, 0, 65535, 1 );
+    w = new_spin_button( TR_PREFS_KEY_PROXY_PORT, core, 0, USHRT_MAX, 1 );
     page->proxy_widgets = g_slist_append( page->proxy_widgets, w );
     w = hig_workarea_add_row( t, &row, _( "Proxy _port:" ), w, NULL );
     page->proxy_widgets = g_slist_append( page->proxy_widgets, w );
@@ -945,8 +1035,7 @@ static void
 refreshSchedSensitivity( struct BandwidthPage * p )
 {
     GSList *       l;
-    const gboolean sched_enabled = pref_flag_get(
-        PREF_KEY_SCHED_LIMIT_ENABLED );
+    const gboolean sched_enabled = pref_flag_get( TR_PREFS_KEY_ALT_SPEED_TIME_ENABLED );
 
     for( l = p->sched_widgets; l != NULL; l = l->next )
         gtk_widget_set_sensitive( GTK_WIDGET( l->data ), sched_enabled );
@@ -1018,6 +1107,57 @@ new_time_combo( GObject *    core,
     return w;
 }
 
+static GtkWidget*
+new_week_combo( GObject * core, const char * key )
+{
+    int i;
+    int selIndex;
+    GtkWidget * w;
+    GtkCellRenderer * r;
+    GtkListStore * store;
+    const int currentValue = pref_int_get( key );
+    const struct {
+        int value;
+        const char * text;
+    } items[] = {
+        { TR_SCHED_ALL,     N_( "Every Day" ) },
+        { TR_SCHED_WEEKDAY, N_( "Weekdays" ) },
+        { TR_SCHED_WEEKEND, N_( "Weekends" ) },
+        { TR_SCHED_SUN,     N_( "Sunday" ) },
+        { TR_SCHED_MON,     N_( "Monday" ) },
+        { TR_SCHED_TUES,    N_( "Tuesday" ) },
+        { TR_SCHED_WED,     N_( "Wednesday" ) },
+        { TR_SCHED_THURS,   N_( "Thursday" ) },
+        { TR_SCHED_FRI,     N_( "Friday" ) },
+        { TR_SCHED_SAT,     N_( "Saturday" ) }
+    };
+
+    /* build a store for the days of the week */
+    selIndex = -1;
+    store = gtk_list_store_new( 2, G_TYPE_INT, G_TYPE_STRING );
+    for( i=0; i<(int)G_N_ELEMENTS(items); ++i ) {
+        GtkTreeIter iter;
+        gtk_list_store_append( store, &iter );
+        gtk_list_store_set( store, &iter, 0, items[i].value, 1, _( items[i].text ), -1 );
+        if( items[i].value == currentValue )
+            selIndex = i;
+    }
+
+    /* build the widget */
+    w = gtk_combo_box_new_with_model( GTK_TREE_MODEL( store ) );
+    r = gtk_cell_renderer_text_new( );
+    gtk_cell_layout_pack_start( GTK_CELL_LAYOUT( w ), r, TRUE );
+    gtk_cell_layout_set_attributes( GTK_CELL_LAYOUT( w ), r, "text", 1, NULL );
+    g_object_set_data_full( G_OBJECT( w ), PREF_KEY, tr_strdup( key ), g_free );
+    if( selIndex >= 0 )
+        gtk_combo_box_set_active( GTK_COMBO_BOX( w ), selIndex );
+    g_signal_connect( w, "changed", G_CALLBACK( onIntComboChanged ), core );
+
+    /* cleanup */
+    g_object_unref( G_OBJECT( store ) );
+    return w;
+}
+
 static void
 bandwidthPageFree( gpointer gpage )
 {
@@ -1033,56 +1173,74 @@ bandwidthPage( GObject * core )
     int                    row = 0;
     const char *           s;
     GtkWidget *            t;
-    GtkWidget *            w, * w2, * h, * l;
+    GtkWidget *            w, * w2, * h;
+    char buf[512];
     struct BandwidthPage * page = tr_new0( struct BandwidthPage, 1 );
 
     page->core = TR_CORE( core );
 
     t = hig_workarea_create( );
-    hig_workarea_add_section_title( t, &row, _( "Limits" ) );
+    hig_workarea_add_section_title( t, &row, _( "Global Bandwidth Limits" ) );
 
-    s = _( "Limit _download speed (KB/s):" );
-    w = new_check_button( s, TR_PREFS_KEY_DSPEED_ENABLED, core );
-    w2 = new_spin_button( TR_PREFS_KEY_DSPEED, core, 0, INT_MAX, 5 );
-    gtk_widget_set_sensitive( GTK_WIDGET( w2 ), pref_flag_get( TR_PREFS_KEY_DSPEED_ENABLED ) );
-    g_signal_connect( w, "toggled", G_CALLBACK( target_cb ), w2 );
-    hig_workarea_add_row_w( t, &row, w, w2, NULL );
+        s = _( "Limit _download speed (KB/s):" );
+        w = new_check_button( s, TR_PREFS_KEY_DSPEED_ENABLED, core );
+        w2 = new_spin_button( TR_PREFS_KEY_DSPEED, core, 0, INT_MAX, 5 );
+        gtk_widget_set_sensitive( GTK_WIDGET( w2 ), pref_flag_get( TR_PREFS_KEY_DSPEED_ENABLED ) );
+        g_signal_connect( w, "toggled", G_CALLBACK( target_cb ), w2 );
+        hig_workarea_add_row_w( t, &row, w, w2, NULL );
 
-    s = _( "Limit _upload speed (KB/s):" );
-    w = new_check_button( s, TR_PREFS_KEY_USPEED_ENABLED, core );
-    w2 = new_spin_button( TR_PREFS_KEY_USPEED, core, 0, INT_MAX, 5 );
-    gtk_widget_set_sensitive( GTK_WIDGET( w2 ), pref_flag_get( TR_PREFS_KEY_USPEED_ENABLED ) );
-    g_signal_connect( w, "toggled", G_CALLBACK( target_cb ), w2 );
-    hig_workarea_add_row_w( t, &row, w, w2, NULL );
+        s = _( "Limit _upload speed (KB/s):" );
+        w = new_check_button( s, TR_PREFS_KEY_USPEED_ENABLED, core );
+        w2 = new_spin_button( TR_PREFS_KEY_USPEED, core, 0, INT_MAX, 5 );
+        gtk_widget_set_sensitive( GTK_WIDGET( w2 ), pref_flag_get( TR_PREFS_KEY_USPEED_ENABLED ) );
+        g_signal_connect( w, "toggled", G_CALLBACK( target_cb ), w2 );
+        hig_workarea_add_row_w( t, &row, w, w2, NULL );
 
     hig_workarea_add_section_divider( t, &row );
-    hig_workarea_add_section_title( t, &row, _( "Scheduled Limits" ) );
+    h = gtk_hbox_new( FALSE, GUI_PAD );
+    w = gtk_image_new_from_stock( "alt-speed-off", -1 );
+    gtk_box_pack_start( GTK_BOX( h ), w, FALSE, FALSE, 0 );
+    g_snprintf( buf, sizeof( buf ), "<b>%s</b>", _( "Speed Limit Mode" ) );
+    w = gtk_label_new( buf );
+    gtk_misc_set_alignment( GTK_MISC( w ), 0.0f, 0.5f );
+    gtk_label_set_use_markup( GTK_LABEL( w ), TRUE );
+    gtk_box_pack_start( GTK_BOX( h ), w, FALSE, FALSE, 0 );
+    hig_workarea_add_section_title_widget( t, &row, h );
 
-    h = gtk_hbox_new( FALSE, 0 );
-    w2 = new_time_combo( core, PREF_KEY_SCHED_BEGIN );
-    page->sched_widgets = g_slist_append( page->sched_widgets, w2 );
-    gtk_box_pack_start( GTK_BOX( h ), w2, FALSE, FALSE, 0 );
-    w2 = gtk_label_new ( _( " and " ) );
-    page->sched_widgets = g_slist_append( page->sched_widgets, w2 );
-    gtk_box_pack_start( GTK_BOX( h ), w2, FALSE, FALSE, 0 );
-    w2 = new_time_combo( core, PREF_KEY_SCHED_END );
-    page->sched_widgets = g_slist_append( page->sched_widgets, w2 );
-    gtk_box_pack_start( GTK_BOX( h ), w2, FALSE, FALSE, 0 );
+        s = _( "Limit do_wnload speed (KB/s):" );
+        w = new_spin_button( TR_PREFS_KEY_ALT_SPEED_DOWN, core, 0, INT_MAX, 5 );
+        hig_workarea_add_row( t, &row, s, w, NULL );
 
-    s = _( "_Limit bandwidth between" );
-    w = new_check_button( s, PREF_KEY_SCHED_LIMIT_ENABLED, core );
-    g_signal_connect( w, "toggled", G_CALLBACK( onSchedToggled ), page );
-    hig_workarea_add_row_w( t, &row, w, h, NULL );
+        s = _( "Limit u_pload speed (KB/s):" );
+        w = new_spin_button( TR_PREFS_KEY_ALT_SPEED_UP, core, 0, INT_MAX, 5 );
+        hig_workarea_add_row( t, &row, s, w, NULL );
 
-    w = new_spin_button( PREF_KEY_SCHED_DL_LIMIT, core, 0, INT_MAX, 5 );
-    page->sched_widgets = g_slist_append( page->sched_widgets, w );
-    l = hig_workarea_add_row( t, &row, _( "Limit d_ownload speed (KB/s):" ), w, NULL );
-    page->sched_widgets = g_slist_append( page->sched_widgets, l );
+        g_snprintf( buf, sizeof( buf ), "<small>%s</small>", _( "When enabled, Speed Limit Mode overrides the Global Bandwidth Limits" ) );
+        w = gtk_label_new( buf );
+        gtk_label_set_use_markup( GTK_LABEL( w ), TRUE );
+        gtk_misc_set_alignment( GTK_MISC( w ), 0.5f, 0.5f );
+        hig_workarea_add_wide_control( t, &row, w );
 
-    w = new_spin_button( PREF_KEY_SCHED_UL_LIMIT, core, 0, INT_MAX, 5 );
-    page->sched_widgets = g_slist_append( page->sched_widgets, w );
-    l = hig_workarea_add_row( t, &row, _( "Limit u_pload speed (KB/s):" ), w, NULL );
-    page->sched_widgets = g_slist_append( page->sched_widgets, l );
+        s = _( "Use Speed Limit Mode _between:" ); 
+        h = gtk_hbox_new( FALSE, 0 );
+        w2 = new_time_combo( core, TR_PREFS_KEY_ALT_SPEED_TIME_BEGIN );
+        page->sched_widgets = g_slist_append( page->sched_widgets, w2 );
+        gtk_box_pack_start( GTK_BOX( h ), w2, TRUE, TRUE, 0 );
+        w2 = gtk_label_new ( _( " and " ) );
+        page->sched_widgets = g_slist_append( page->sched_widgets, w2 );
+        gtk_box_pack_start( GTK_BOX( h ), w2, FALSE, FALSE, 0 );
+        w2 = new_time_combo( core, TR_PREFS_KEY_ALT_SPEED_TIME_END );
+        page->sched_widgets = g_slist_append( page->sched_widgets, w2 );
+        gtk_box_pack_start( GTK_BOX( h ), w2, TRUE, TRUE, 0 );
+        w = new_check_button( s, TR_PREFS_KEY_ALT_SPEED_TIME_ENABLED, core );
+        g_signal_connect( w, "toggled", G_CALLBACK( onSchedToggled ), page );
+        hig_workarea_add_row_w( t, &row, w, h, NULL );
+
+        s = _( "_On days:" );
+        w = new_week_combo( core, TR_PREFS_KEY_ALT_SPEED_TIME_DAY );
+        page->sched_widgets = g_slist_append( page->sched_widgets, w );
+        w = hig_workarea_add_row( t, &row, s, w, NULL );
+        page->sched_widgets = g_slist_append( page->sched_widgets, w );
 
     hig_workarea_finish( t, &row );
     g_object_set_data_full( G_OBJECT( t ), "page", page, bandwidthPageFree );
@@ -1095,109 +1253,60 @@ bandwidthPage( GObject * core )
 *****  Network Tab
 ****/
 
-struct test_port_data
-{
-    GtkWidget *  label;
-    gboolean *   alive;
-    char text[128];
-};
-
-/* this is invoked in the gtk main loop's thread */
-static gboolean
-testing_port_done_idle( gpointer gdata )
-{
-    struct test_port_data * data = gdata;
-
-    if( *data->alive )
-    {
-        gdk_threads_enter( );
-        gtk_label_set_markup( GTK_LABEL( data->label ), data->text );
-        gdk_threads_leave( );
-    }
-
-    return FALSE;
-}
-
-/* this is invoked in the libtransmission thread */
-static void
-testing_port_done( tr_session * session        UNUSED,
-                   long          response_code UNUSED,
-                   const void *                response,
-                   size_t                      response_len,
-                   void *                      gdata )
-{
-    struct test_port_data * data = gdata;
-
-    if( *data->alive )
-    {
-        const int isOpen = response_len && *(char*)response == '1';
-        g_snprintf( data->text, sizeof( data->text ), isOpen 
-                    ? _( "Port is <b>open</b>" )
-                    : _( "Port is <b>closed</b>" ) );
-        g_idle_add( testing_port_done_idle, data );
-    }
-}
-
-static gboolean
-testing_port_begin( gpointer gdata )
-{
-    struct test_port_data * data = gdata;
-
-    if( *data->alive )
-    {
-        char            url[256];
-        GObject       * o       = G_OBJECT( data->label );
-        GtkSpinButton * spin    = g_object_get_data( o, "tr-port-spin" );
-        tr_session    * session = g_object_get_data( o, "session" );
-        const int       port    = gtk_spin_button_get_value_as_int( spin );
-        g_snprintf( url, sizeof( url ),
-                    "http://portcheck.transmissionbt.com/%d",
-                    port );
-        tr_webRun( session, url, NULL, testing_port_done, data );
-    }
-    return FALSE;
-}
-
 struct network_page_data
 {
-    gboolean *   alive;
-    GtkWidget *  label;
-    guint        id;
-    TrCore *     core;
+    TrCore     * core;
+    GtkWidget  * portLabel;
+    GtkWidget  * portButton;
+    GtkWidget  * portSpin;
+    gulong       portTag;
+    gulong       prefsTag;
 };
 
 static void
-onCorePrefsChanged( TrCore * core UNUSED,
-                    const char *  key,
-                    gpointer      gdata )
+onCorePrefsChanged( TrCore * core UNUSED, const char *  key, gpointer gdata )
 {
     if( !strcmp( key, TR_PREFS_KEY_PEER_PORT ) )
     {
-        struct network_page_data * ndata = gdata;
-        struct test_port_data *    data;
-
-        gtk_label_set_markup( GTK_LABEL( ndata->label ),
-                             _( "<i>Testing port...</i>" ) );
-
-        /* wait three seconds to give the port forwarding time to kick in */
-        data = g_new0( struct test_port_data, 1 );
-        data->label = ndata->label;
-        data->alive = ndata->alive;
-        gtr_timeout_add_seconds( 3, testing_port_begin, data );
+        struct network_page_data * data = gdata;
+        gtk_label_set_text( GTK_LABEL( data->portLabel ), _( "Status unknown" ) );
+        gtk_widget_set_sensitive( data->portButton, TRUE );
+        gtk_widget_set_sensitive( data->portSpin, TRUE );
     }
 }
 
 static void
-networkPageDestroyed( gpointer       gdata,
-                      GObject * dead UNUSED )
+networkPageDestroyed( gpointer gdata, GObject * dead UNUSED )
 {
     struct network_page_data * data = gdata;
-
-    *data->alive = FALSE;
-    g_signal_handler_disconnect( data->core, data->id );
+    if( data->prefsTag > 0 )
+        g_signal_handler_disconnect( data->core, data->prefsTag );
+    if( data->portTag > 0 )
+        g_signal_handler_disconnect( data->core, data->portTag );
     g_free( data );
 }
 
+static void
+onPortTested( TrCore * core UNUSED, gboolean isOpen, gpointer vdata )
+{
+    struct network_page_data * data = vdata;
+    const char * markup = isOpen ? _( "Port is <b>open</b>" ) : _( "Port is <b>closed</b>" );
+    gtk_label_set_markup( GTK_LABEL( data->portLabel ), markup );
+    gtk_widget_set_sensitive( data->portButton, TRUE );
+    gtk_widget_set_sensitive( data->portSpin, TRUE );
+}
+
+static void
+onPortTest( GtkButton * button UNUSED, gpointer vdata )
+{
+    struct network_page_data * data = vdata;
+    gtk_widget_set_sensitive( data->portButton, FALSE );
+    gtk_widget_set_sensitive( data->portSpin, FALSE );
+    gtk_label_set_markup( GTK_LABEL( data->portLabel ), _( "<i>Testing...</i>" ) );
+    data->portTag = g_signal_connect( data->core, "port-tested", G_CALLBACK(onPortTested), data );
+    tr_core_port_test( data->core );
+}
+
 static GtkWidget*
 networkPage( GObject * core )
 {
@@ -1205,48 +1314,42 @@ networkPage( GObject * core )
     const char *               s;
     GtkWidget *                t;
     GtkWidget *                w;
-    GtkWidget *                w2;
     GtkWidget *                h;
     GtkWidget *                l;
     struct network_page_data * data;
 
-    /* register to stop listening to core prefs changes when the page is
-      destroyed */
+    /* register to stop listening to core prefs changes when the page is destroyed */
     data = g_new0( struct network_page_data, 1 );
     data->core = TR_CORE( core );
 
-    /* we leak this gboolean* s.t. we know it will still be alive when the port
-       check is done, whether the dialog was destroyed or not.  kind of
-       clumsy... */
-    data->alive = g_new( gboolean, 1 );
-    *data->alive = TRUE;
-
     /* build the page */
     t = hig_workarea_create( );
     hig_workarea_add_section_title( t, &row, _( "Incoming Peers" ) );
 
-    h = gtk_hbox_new( FALSE, GUI_PAD_BIG );
-    w2 = new_spin_button( TR_PREFS_KEY_PEER_PORT, core, 1, 65535, 1 );
-    gtk_box_pack_start( GTK_BOX( h ), w2, FALSE, FALSE, 0 );
-    data->label = l = gtk_label_new( NULL );
-    gtk_misc_set_alignment( GTK_MISC( l ), 0.0f, 0.5f );
-    gtk_box_pack_start( GTK_BOX( h ), l, FALSE, FALSE, 0 );
-    hig_workarea_add_row( t, &row, _( "Listening _port:" ), h, w2 );
+    s = _( "_Port for incoming connections:" );
+    w = data->portSpin = new_spin_button( TR_PREFS_KEY_PEER_PORT, core, 1, USHRT_MAX, 1 );
+    hig_workarea_add_row( t, &row, s, w, NULL );
 
-    g_object_set_data( G_OBJECT( l ), "tr-port-spin", w2 );
-    g_object_set_data( G_OBJECT( l ), "session",
-                      tr_core_session( TR_CORE( core ) ) );
-    data->id = g_signal_connect( TR_CORE( core ), "prefs-changed",
-                                 G_CALLBACK( onCorePrefsChanged ), data );
-    onCorePrefsChanged( NULL, TR_PREFS_KEY_PEER_PORT, data );
+    h = gtk_hbox_new( FALSE, GUI_PAD_BIG );
+    l = data->portLabel = gtk_label_new( _( "Status unknown" ) );
+    gtk_misc_set_alignment( GTK_MISC( l ), 0.0f, 0.5f );
+    gtk_box_pack_start( GTK_BOX( h ), l, TRUE, TRUE, 0 );
+    w = data->portButton = gtk_button_new_with_mnemonic( _( "_Test Port" ) );
+    gtk_box_pack_end( GTK_BOX( h ), w, FALSE, FALSE, 0 );
+    g_signal_connect( w, "clicked", G_CALLBACK(onPortTest), data );
+    hig_workarea_add_row( t, &row, NULL, h, NULL );
+    data->prefsTag = g_signal_connect( TR_CORE( core ), "prefs-changed", G_CALLBACK( onCorePrefsChanged ), data );
+    g_object_weak_ref( G_OBJECT( t ), networkPageDestroyed, data );
+
+    s = _( "_Randomize the port every launch" );
+    w = new_check_button( s, TR_PREFS_KEY_PEER_PORT_RANDOM_ON_START, core );
+    hig_workarea_add_wide_control( t, &row, w );
 
     s = _( "Use UPnP or NAT-PMP port _forwarding from my router" );
     w = new_check_button( s, TR_PREFS_KEY_PORT_FORWARDING, core );
     hig_workarea_add_wide_control( t, &row, w );
 
     hig_workarea_finish( t, &row );
-    g_object_weak_ref( G_OBJECT( t ), networkPageDestroyed, data );
-
     return t;
 }
 
@@ -1281,15 +1384,15 @@ tr_prefs_dialog_new( GObject *   core,
     gtk_notebook_append_page( GTK_NOTEBOOK( n ),
                               peerPage( core ),
                               gtk_label_new ( _( "Peers" ) ) );
+    gtk_notebook_append_page( GTK_NOTEBOOK( n ),
+                              bandwidthPage( core ),
+                              gtk_label_new ( _( "Speed" ) ) );
     gtk_notebook_append_page( GTK_NOTEBOOK( n ),
                               networkPage( core ),
                               gtk_label_new ( _( "Network" ) ) );
     gtk_notebook_append_page( GTK_NOTEBOOK( n ),
                               desktopPage( core ),
                               gtk_label_new ( _( "Desktop" ) ) );
-    gtk_notebook_append_page( GTK_NOTEBOOK( n ),
-                              bandwidthPage( core ),
-                              gtk_label_new ( _( "Bandwidth" ) ) );
     gtk_notebook_append_page( GTK_NOTEBOOK( n ),
                               webPage( core ),
                               gtk_label_new ( _( "Web" ) ) );
