@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: util.c 8205 2009-04-10 17:44:17Z charles $
+ * $Id: util.c 8669 2009-06-11 14:51:21Z charles $
  *
  * Copyright (c) 2005-2008 Transmission authors and contributors
  *
@@ -38,10 +38,11 @@
  #include <dbus/dbus-glib.h>
 #endif
 
-#include <libevent/evhttp.h>
+#include <evhttp.h>
 
 #include <libtransmission/transmission.h> /* TR_RATIO_NA, TR_RATIO_INF */
 #include <libtransmission/utils.h> /* tr_inf */
+#include <libtransmission/version.h> /* tr_inf */
 
 #include "conf.h"
 #include "hig.h"
@@ -49,21 +50,9 @@
 #include "util.h"
 
 char*
-tr_strlratio( char * buf,
-              double ratio,
-              size_t buflen )
+tr_strlratio( char * buf, double ratio, size_t buflen )
 {
-    if( (int)ratio == TR_RATIO_NA )
-        g_strlcpy( buf, _( "None" ), buflen );
-    else if( (int)ratio == TR_RATIO_INF )
-        g_strlcpy( buf, "\xE2\x88\x9E", buflen );
-    else if( ratio < 10.0 )
-        g_snprintf( buf, buflen, "%'.2f", ratio );
-    else if( ratio < 100.0 )
-        g_snprintf( buf, buflen, "%'.1f", ratio );
-    else
-        g_snprintf( buf, buflen, "%'.0f", ratio );
-    return buf;
+    return tr_strratio( buf, buflen, ratio, "\xE2\x88\x9E" );
 }
 
 #define KILOBYTE_FACTOR 1024.0
@@ -210,6 +199,15 @@ gtr_localtime( time_t time )
     return g_locale_to_utf8( buf, -1, NULL, NULL, NULL );
 }
 
+char *
+gtr_localtime2( char * buf, time_t time, size_t buflen )
+{
+    char * tmp = gtr_localtime( time );
+    g_strlcpy( buf, tmp, buflen );
+    g_free( tmp );
+    return buf;
+}
+
 int
 mkdir_p( const char * path,
          mode_t       mode )
@@ -583,29 +581,43 @@ gtr_open_file( const char * path )
 gboolean
 gtr_dbus_add_torrent( const char * filename )
 {
-    static gboolean   success = FALSE;
-
+    /* FIXME: why is this static? */
+    static gboolean success = FALSE;
 #ifdef HAVE_DBUS_GLIB
-    DBusGProxy *      proxy = NULL;
-    GError *          err = NULL;
+    DBusGProxy * proxy = NULL;
+    GError * err = NULL;
     DBusGConnection * conn;
-    if( ( conn = dbus_g_bus_get( DBUS_BUS_SESSION, &err ) ) )
-        proxy = dbus_g_proxy_new_for_name ( conn, VALUE_SERVICE_NAME,
-                                            VALUE_SERVICE_OBJECT_PATH,
-                                            VALUE_SERVICE_INTERFACE );
-    else if( err )
-        g_message( "err: %s", err->message );
-    if( proxy )
-        dbus_g_proxy_call( proxy, "AddFile", &err,
-                           G_TYPE_STRING, filename,
-                           G_TYPE_INVALID,
-                           G_TYPE_BOOLEAN, &success,
-                           G_TYPE_INVALID );
-    if( err )
-        g_message( "err: %s", err->message );
+    char * file_contents;
+    gsize file_length;
+
+    if( g_file_get_contents( filename, &file_contents, &file_length, NULL ) )
+    {
+        char * b64 = tr_base64_encode( file_contents, file_length, NULL );
+        if(( conn = dbus_g_bus_get( DBUS_BUS_SESSION, &err )))
+            proxy = dbus_g_proxy_new_for_name (conn, VALUE_SERVICE_NAME,
+                                                     VALUE_SERVICE_OBJECT_PATH,
+                                                     VALUE_SERVICE_INTERFACE );
+        else if( err )
+           g_message( "err: %s", err->message );
+        if( proxy )
+            dbus_g_proxy_call( proxy, "AddMetainfo", &err,
+                               G_TYPE_STRING, b64,
+                               G_TYPE_INVALID,
+                               G_TYPE_BOOLEAN, &success,
+                               G_TYPE_INVALID );
+        if( err )
+           g_message( "err: %s", err->message );
+
+        if( proxy )
+            g_object_unref( proxy );
+        if( conn )
+            dbus_g_connection_unref( conn );
+
+        tr_free( b64 );
+        g_free( file_contents );
+    }
+    else g_message( "couldn't read %s", filename );
 
-    g_object_unref( proxy );
-    dbus_g_connection_unref( conn );
 #endif
     return success;
 }
