@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: bandwidth.h 8204 2009-04-10 17:34:25Z charles $
+ * $Id: bandwidth.h 8709 2009-06-16 19:31:17Z charles $
  */
 
 #ifndef __TRANSMISSION__
@@ -23,13 +23,18 @@
 
 struct tr_peerIo;
 
+/**
+ * @addtogroup networked_io Networked IO
+ * @{
+ */
+
 /* these are PRIVATE IMPLEMENTATION details that should not be touched.
  * it's included in the header for inlining and composition. */
 enum
 {
     HISTORY_MSEC = 2000,
     INTERVAL_MSEC = HISTORY_MSEC,
-    GRANULARITY_MSEC = 50,
+    GRANULARITY_MSEC = 200,
     HISTORY_SIZE = ( INTERVAL_MSEC / GRANULARITY_MSEC ),
     MAGIC_NUMBER = 43143
 };
@@ -100,6 +105,7 @@ typedef struct tr_bandwidth
 
     struct tr_band band[2];
     struct tr_bandwidth * parent;
+    tr_priority_t priority;
     int magicNumber;
     tr_session * session;
     tr_ptrArray children; /* struct tr_bandwidth */
@@ -242,6 +248,15 @@ static TR_INLINE void tr_bandwidthHonorParentLimits ( tr_bandwidth        * band
     bandwidth->band[direction].honorParentLimits = isEnabled;
 }
 
+static TR_INLINE tr_bool tr_bandwidthAreParentLimitsHonored( tr_bandwidth  * bandwidth,
+                                                             tr_direction    direction )
+{
+    assert( tr_isBandwidth( bandwidth ) );
+    assert( tr_isDirection( direction ) );
+
+    return bandwidth->band[direction].honorParentLimits;
+}
+
 /******
 *******
 ******/
@@ -249,4 +264,5 @@ static TR_INLINE void tr_bandwidthHonorParentLimits ( tr_bandwidth        * band
 void tr_bandwidthSetPeer( tr_bandwidth        * bandwidth,
                           struct tr_peerIo    * peerIo );
 
+/* @} */
 #endif
