@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: clients.c 8204 2009-04-10 17:34:25Z charles $
+ * $Id: clients.c 8599 2009-06-03 15:47:08Z charles $
  */
 
 /* thanks amc1! */
@@ -17,7 +17,7 @@
 #include <stdlib.h> /* strtol */
 #include <string.h>
 
-#include <libevent/event.h> /* evbuffer */
+#include <event.h> /* evbuffer */
 
 #include "transmission.h"
 #include "clients.h"
@@ -171,18 +171,7 @@ tr_clientForId( char * buf, size_t buflen, const void * id_in )
     /* Azureus-style */
     if( id[0] == '-' && id[7] == '-' )
     {
-        if( !memcmp( id+1, "UT", 2 ) )
-        {
-            tr_snprintf( buf, buflen, "\xc2\xb5Torrent %d.%d.%d%s",
-                         strint(id+3,1), strint(id+4,1), strint(id+5,1), getMnemonicEnd(id[6]) );
-        }
-        if( !memcmp( id+1, "UM", 2 ) )
-        {
-            tr_snprintf( buf, buflen, "\xc2\xb5Torrent Mac %d.%d.%d%s",
-                         strint(id+3,1), strint(id+4,1), strint(id+5,1), getMnemonicEnd(id[6]) );
-        }
-
-        else if( !memcmp( id+1, "TR", 2 ) )
+        if( !memcmp( id+1, "TR", 2 ) )
         {
             if( !memcmp( id+3, "000", 3 ) ) /* very old client style: -TR0006- is 0.6 */
                 tr_snprintf( buf, buflen, "Transmission 0.%c", id[6] );
@@ -193,6 +182,17 @@ tr_clientForId( char * buf, size_t buflen, const void * id_in )
                           id[6]=='Z' || id[6]=='X' ? "+" : "" );
         }
         
+        else if( !memcmp( id+1, "UT", 2 ) )
+        {
+            tr_snprintf( buf, buflen, "\xc2\xb5Torrent %d.%d.%d%s",
+                         strint(id+3,1), strint(id+4,1), strint(id+5,1), getMnemonicEnd(id[6]) );
+        }
+        else if( !memcmp( id+1, "UM", 2 ) )
+        {
+            tr_snprintf( buf, buflen, "\xc2\xb5Torrent Mac %d.%d.%d%s",
+                         strint(id+3,1), strint(id+4,1), strint(id+5,1), getMnemonicEnd(id[6]) );
+        }
+        
         else if( !memcmp( id+1, "AZ", 2 ) )
         {
             if( id[3] > '3' || ( id[3] == '3' && id[4] >= '1' ) ) /* Vuze starts at version 3.1.0.0 */
@@ -240,6 +240,7 @@ tr_clientForId( char * buf, size_t buflen, const void * id_in )
         else if( !memcmp( id+1, "QD", 2 ) ) four_digits( buf, buflen, "QQDownload", id+3 );
         else if( !memcmp( id+1, "RS", 2 ) ) four_digits( buf, buflen, "Rufus", id+3 );
         else if( !memcmp( id+1, "RT", 2 ) ) four_digits( buf, buflen, "Retriever", id+3 );
+        else if( !memcmp( id+1, "RZ", 2 ) ) four_digits( buf, buflen, "RezTorrent", id+3 );
         else if( !memcmp( id+1, "SD", 2 ) ) four_digits( buf, buflen, "Xunlei", id+3 );
         else if( !memcmp( id+1, "SS", 2 ) ) four_digits( buf, buflen, "SwarmScope", id+3 );
         else if( !memcmp( id+1, "SZ", 2 ) ) four_digits( buf, buflen, "Shareaza", id+3 );
@@ -411,16 +412,17 @@ tr_clientForId( char * buf, size_t buflen, const void * id_in )
     /* No match */
     if( !*buf )
     {
-        struct evbuffer * out = tr_getBuffer( );
+        char out[32], *walk=out;
         const char *in, *in_end;
         for( in=(const char*)id, in_end=in+8; in!=in_end; ++in ) {
             if( isprint( *in ) )
-                evbuffer_add_printf( out, "%c", *in );
-            else
-                evbuffer_add_printf( out, "%%%02X", (unsigned int)*in );
+                *walk++ = *in;
+            else {
+                tr_snprintf( walk, out+sizeof(out)-walk, "%%%02X", (unsigned int)*in );
+                walk += 3;
+            }
         }
-
-        tr_strlcpy( buf, EVBUFFER_DATA( out ), buflen );
-        tr_releaseBuffer( out );
+        *walk = '\0';
+        tr_strlcpy( buf, out, buflen );
     }
 }
