@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: completion.h 8218 2009-04-12 15:01:29Z charles $
+ * $Id: completion.h 8695 2009-06-15 00:11:06Z charles $
  */
 
 #ifndef TR_COMPLETION_H
@@ -20,6 +20,7 @@
 #include <assert.h>
 
 #include "transmission.h"
+#include "bitfield.h"
 #include "utils.h" /* tr_bitfield */
 
 typedef struct tr_completion
@@ -109,7 +110,8 @@ static TR_INLINE float tr_cpPercentComplete( const tr_completion * cp )
 
 static TR_INLINE float tr_cpPercentDone( const tr_completion * cp )
 {
-    return tr_getRatio( cp->sizeNow, tr_cpSizeWhenDone( cp ) );
+    const double ratio = tr_getRatio( cp->sizeNow, tr_cpSizeWhenDone( cp ) );
+    return (ratio == TR_RATIO_NA ||  ratio == TR_RATIO_INF) ? 0.0f : ratio;
 }
 
 /**
