@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: fastresume.c 7455 2008-12-22 00:51:14Z charles $
+ * $Id: fastresume.c 8254 2009-04-18 23:17:30Z charles $
  *
  * Copyright (c) 2005-2008 Transmission authors and contributors
  *
@@ -22,6 +22,17 @@
  * DEALINGS IN THE SOFTWARE.
  *****************************************************************************/
 
+/*
+ * NOTE: THIS FILE IS DEPRECATED
+ *
+ *  The fastresume file format is brittle and was replaced in Transmission 1.20
+ *  with the benc-formatted ".resume" files implemented in resume.[ch].
+ *
+ *  This older format is kept only for reading older resume files for users
+ *  who upgrade from older versions of Transmission, and may be removed
+ *  after more time has passed.
+ */  
+
 /***********************************************************************
  * Fast resume
  ***********************************************************************
@@ -52,9 +63,9 @@
 #include <unistd.h> /* unlink */
 
 #include "transmission.h"
+#include "session.h"
 #include "completion.h"
 #include "fastresume.h"
-#include "net.h"
 #include "peer-mgr.h"
 #include "platform.h"
 #include "resume.h" /* TR_FR_ bitwise enum */
@@ -170,7 +181,7 @@ internalIdToPublicBitfield( uint8_t id )
             ret = TR_FR_PROGRESS;      break;
 
         case FR_ID_PRIORITY:
-            ret = TR_FR_PRIORITY;      break;
+            ret = TR_FR_FILE_PRIORITIES; break;
 
         case FR_ID_SPEED:
             ret = TR_FR_SPEEDLIMIT;    break;
@@ -282,7 +293,7 @@ parseProgress( tr_torrent *    tor,
         bitfield.byteCount = FR_BLOCK_BITFIELD_LEN( tor );
         bitfield.bitCount = bitfield.byteCount * 8;
         bitfield.bits = (uint8_t*) walk;
-        if( tr_cpBlockBitfieldSet( tor->completion, &bitfield ) )
+        if( tr_cpBlockBitfieldSet( &tor->completion, &bitfield ) )
             ret = TR_FR_PROGRESS;
         else {
             tr_torrentUncheck( tor );
@@ -296,7 +307,7 @@ parseProgress( tr_torrent *    tor,
         tr_piece_index_t i;
         for( i = 0; i < tor->info.pieceCount; ++i )
             if( !tr_torrentIsPieceChecked( tor, i ) )
-                tr_cpPieceRem( tor->completion, i );
+                tr_cpPieceRem( &tor->completion, i );
     }
 
     return ret;
@@ -312,11 +323,12 @@ parsePriorities( tr_torrent *    tor,
     if( len == (uint32_t)( 2 * tor->info.fileCount ) )
     {
         const size_t     n = tor->info.fileCount;
-        const size_t     len = 2 * n;
+        const uint8_t *  walk = buf;
         tr_file_index_t *dnd = NULL, dndCount = 0;
         tr_file_index_t *dl = NULL, dlCount = 0;
         size_t           i;
-        const uint8_t *  walk = buf;
+
+        len = 2 * n;
 
         /* set file priorities */
         for( i = 0; i < n; ++i )
@@ -354,7 +366,7 @@ parsePriorities( tr_torrent *    tor,
         tr_free( dnd );
         tr_free( dl );
 
-        ret = TR_FR_PRIORITY;
+        ret = TR_FR_FILE_PRIORITIES;
     }
 
     return ret;
@@ -375,11 +387,11 @@ parseSpeedLimit( tr_torrent *    tor,
         readBytes( &i16, &buf, sizeof( i16 ) );
         tr_torrentSetSpeedLimit( tor, TR_DOWN, i16 );
         readBytes( &i8, &buf, sizeof( i8 ) );
-        tr_torrentSetSpeedMode( tor, TR_DOWN, (tr_speedlimit)i8 );
+        /*tr_torrentSetSpeedMode( tor, TR_DOWN, (tr_speedlimit)i8 );*/
         readBytes( &i16, &buf, sizeof( i16 ) );
         tr_torrentSetSpeedLimit( tor, TR_UP, i16 );
         readBytes( &i8, &buf, sizeof( i8 ) );
-        tr_torrentSetSpeedMode( tor, TR_UP, (tr_speedlimit)i8 );
+        /*tr_torrentSetSpeedMode( tor, TR_UP, (tr_speedlimit)i8 );*/
 
         ret = TR_FR_SPEEDLIMIT;
     }
@@ -414,9 +426,7 @@ parsePeers( tr_torrent *    tor,
         {
             tr_pex pex;
             readBytes( &pex, &buf, sizeof( tr_pex ) );
-            tr_peerMgrAddPex( tor->session->peerMgr, tor->info.hash,
-                              TR_PEER_FROM_CACHE,
-                              &pex );
+            tr_peerMgrAddPex( tor, TR_PEER_FROM_CACHE, &pex );
         }
 
         tr_tordbg( tor, "Loaded %d peers from resume file", count );
