@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: fdlimit.c 8203 2009-04-10 15:56:40Z charles $
+ * $Id: fdlimit.c 8707 2009-06-16 17:10:47Z charles $
  *
  * Copyright (c) 2005-2008 Transmission authors and contributors
  *
@@ -43,8 +43,8 @@
  #include <fcntl.h>
 #endif
 
-#ifdef HAVE_FALLOCATE
- #include <linux/falloc.h>
+#ifdef HAVE_XFS_XFS_H
+ #include <xfs/xfs.h>
 #endif
 
 #include <sys/types.h>
@@ -79,9 +79,6 @@ enum
 {
     NOFILE_BUFFER = 512, /* the process' number of open files is
                             globalMaxPeers + NOFILE_BUFFER */
-
-    SYNC_INTERVAL = 15   /* (arbitrary number) how many seconds to go
-                            between fsync calls for files in heavy use */
 };
 
 struct tr_openfile
@@ -92,7 +89,6 @@ struct tr_openfile
     char       filename[MAX_PATH_LENGTH];
     int        fd;
     uint64_t   date;
-    time_t     syncAt;
 };
 
 struct tr_fd_s
@@ -158,31 +154,47 @@ preallocateFileFull( const char * filename, uint64_t length )
     int fd = open( filename, flags, 0666 );
     if( fd >= 0 )
     {
-        
-# ifdef HAVE_FALLOCATE
-
-        success = !fallocate( fd, FALLOC_FL_KEEP_SIZE, 0, length );
-
-# elif defined(HAVE_POSIX_FALLOCATE)
-
-        success = !posix_fallocate( fd, 0, length );
-
-# elif defined(SYS_DARWIN) 
-
-        fstore_t fst;
-        fst.fst_flags = F_ALLOCATECONTIG;
-        fst.fst_posmode = F_PEOFPOSMODE;
-        fst.fst_offset = 0;
-        fst.fst_length = length;
-        fst.fst_bytesalloc = 0;
-        success = !fcntl( fd, F_PREALLOCATE, &fst );
-
-# else
-
-        #warning no known method to preallocate files on this platform
-        success = 0;
-
+# ifdef HAVE_XFS_XFS_H
+        if( !success && platform_test_xfs_fd( fd ) )
+        {
+            xfs_flock64_t fl;
+            fl.l_whence = 0;
+            fl.l_start = 0;
+            fl.l_len = length;
+            success = !xfsctl( NULL, fd, XFS_IOC_RESVSP64, &fl );
+        }
 # endif
+# ifdef SYS_DARWIN
+        if( !success )
+        {
+            fstore_t fst;
+            fst.fst_flags = F_ALLOCATECONTIG;
+            fst.fst_posmode = F_PEOFPOSMODE;
+            fst.fst_offset = 0;
+            fst.fst_length = length;
+            fst.fst_bytesalloc = 0;
+            success = !fcntl( fd, F_PREALLOCATE, &fst );
+        }
+# endif
+# ifdef HAVE_POSIX_FALLOCATE
+        if( !success )
+        {
+            success = !posix_fallocate( fd, 0, length );
+        }
+# endif
+
+        if( !success ) /* if nothing else works, do it the old-fashioned way */
+        {
+            uint8_t buf[ 4096 ]; 
+            memset( buf, 0, sizeof( buf ) ); 
+            success = TRUE; 
+            while ( success && ( length > 0 ) ) 
+            { 
+                const int thisPass = MIN( length, sizeof( buf ) ); 
+                success = write( fd, buf, thisPass ) == thisPass; 
+                length -= thisPass; 
+            } 
+        }
 
         close( fd );
     }
@@ -192,6 +204,76 @@ preallocateFileFull( const char * filename, uint64_t length )
     return success;
 }
 
+tr_bool
+tr_preallocate_file( const char * filename, uint64_t length )
+{
+    return preallocateFileFull( filename, length );
+}
+
+int
+tr_open_file_for_writing( const char * filename )
+{
+    int flags = O_WRONLY | O_CREAT;
+#ifdef O_BINARY
+    flags |= O_BINARY;
+#endif
+#ifdef O_LARGEFILE
+    flags |= O_LARGEFILE;
+#endif
+    return open( filename, flags, 0666 );
+}
+
+int
+tr_open_file_for_scanning( const char * filename )
+{
+    int fd;
+    int flags;
+
+    /* build the flags */
+    flags = O_RDONLY;
+#ifdef O_SEQUENTIAL
+    flags |= O_SEQUENTIAL;
+#endif
+#ifdef O_BINARY
+    flags |= O_BINARY;
+#endif
+#ifdef O_LARGEFILE
+    flags |= O_LARGEFILE;
+#endif
+
+    /* open the file */
+    fd = open( filename, flags, 0666 );
+    if( fd >= 0 )
+    {
+        /* Set hints about the lookahead buffer and caching. It's okay
+           for these to fail silently, so don't let them affect errno */
+        const int err = errno;
+#ifdef HAVE_POSIX_FADVISE
+        posix_fadvise( fd, 0, 0, POSIX_FADV_SEQUENTIAL );
+#endif
+#ifdef SYS_DARWIN
+        fcntl( fd, F_NOCACHE, 1 );
+        fcntl( fd, F_RDAHEAD, 1 );
+#endif
+        errno = err;
+    }
+
+    return fd;
+}
+
+void
+tr_close_file( int fd )
+{
+#if defined(HAVE_POSIX_FADVISE)
+    /* Set hint about not caching this file.
+       It's okay for this to fail silently, so don't let it affect errno */
+    const int err = errno;
+    posix_fadvise( fd, 0, 0, POSIX_FADV_DONTNEED );
+    errno = err;
+#endif
+    close( fd );
+}
+
 /**
  * returns 0 on success, or an errno value on failure.
  * errno values include ENOENT if the parent folder doesn't exist,
@@ -241,8 +323,8 @@ TrOpenFile( int                      i,
     
     /* open the file */
     flags = doWrite ? ( O_RDWR | O_CREAT ) : O_RDONLY;
-#ifdef O_RANDOM
-    flags |= O_RANDOM
+#ifdef O_SEQUENTIAL
+    flags |= O_SEQUENTIAL;
 #endif
 #ifdef O_LARGEFILE
     flags |= O_LARGEFILE;
@@ -263,7 +345,7 @@ TrOpenFile( int                      i,
         preallocateFileSparse( file->fd, desiredFileSize );
 
 #ifdef HAVE_POSIX_FADVISE
-    posix_fadvise( file->fd, 0, 0, POSIX_FADV_RANDOM );
+    posix_fadvise( file->fd, 0, 0, POSIX_FADV_SEQUENTIAL );
 #endif
 
     tr_free( filename );
@@ -285,7 +367,7 @@ TrCloseFile( int i )
     assert( i < gFd->openFileLimit );
     assert( fileIsOpen( o ) );
 
-    close( o->fd );
+    tr_close_file( o->fd );
     o->fd = -1;
     o->isCheckedOut = 0;
 }
@@ -410,7 +492,6 @@ tr_fdFileCheckout( const char             * folder,
                 doWrite ? 'y' : 'n' );
         tr_strlcpy( o->filename, filename, sizeof( o->filename ) );
         o->isWritable = doWrite;
-        o->syncAt = time( NULL ) + SYNC_INTERVAL;
     }
 
     dbgmsg( "checking out '%s' in slot %d", filename, winner );
@@ -438,15 +519,6 @@ tr_fdFileReturn( int fd )
         o->isCheckedOut = 0;
         if( o->closeWhenDone )
             TrCloseFile( i );
-        else if( o->syncAt <= time( NULL ) ) {
-            dbgmsg( "fsync()ing file '%s' in slot #%d", o->filename, i );
-            fsync( o->fd );
-#ifdef HAVE_POSIX_FADVISE
-            /* TODO: test performance with and without this */
-            posix_fadvise( o->fd, 0, 0, POSIX_FADV_DONTNEED );
-#endif
-            o->syncAt = time( NULL ) + SYNC_INTERVAL;
-        }
 
         break;
     }
@@ -530,41 +602,51 @@ tr_fdSocketAccept( int           b,
                    tr_address  * addr,
                    tr_port     * port )
 {
-    int                s = -1;
-    unsigned int       len;
+    int s;
+    unsigned int len;
     struct sockaddr_storage sock;
+    tr_lockLock( gFd->lock );
 
     assert( addr );
     assert( port );
 
-    tr_lockLock( gFd->lock );
-    if( gFd->socketCount < getSocketMax( gFd ) )
+    len = sizeof( struct sockaddr_storage );
+    s = accept( b, (struct sockaddr *) &sock, &len );
+
+    if( ( s >= 0 ) && gFd->socketCount > getSocketMax( gFd ) )
     {
-        len = sizeof( struct sockaddr_storage );
-        s = accept( b, (struct sockaddr *) &sock, &len );
+        EVUTIL_CLOSESOCKET( s );
+        s = -1;
     }
-    if( s > -1 )
+
+    if( s >= 0 )
     {
         /* "The ss_family field of the sockaddr_storage structure will always 
          * align with the family field of any protocol-specific structure." */ 
         if( sock.ss_family == AF_INET ) 
-        { 
-            struct sockaddr_in * sock4 = (struct sockaddr_in *)&sock; 
+        {
+            struct sockaddr_in *si;
+            union { struct sockaddr_storage dummy; struct sockaddr_in si; } s;
+            s.dummy = sock;
+            si = &s.si;
             addr->type = TR_AF_INET; 
-            addr->addr.addr4.s_addr = sock4->sin_addr.s_addr; 
-            *port = sock4->sin_port; 
+            addr->addr.addr4.s_addr = si->sin_addr.s_addr; 
+            *port = si->sin_port; 
         } 
         else 
         { 
-            struct sockaddr_in6 * sock6 = (struct sockaddr_in6 *)&sock; 
+            struct sockaddr_in6 *si;
+            union { struct sockaddr_storage dummy; struct sockaddr_in6 si; } s;
+            s.dummy = sock;
+            si = &s.si;
             addr->type = TR_AF_INET6; 
-            addr->addr.addr6 = sock6->sin6_addr;
-            *port = sock6->sin6_port; 
+            addr->addr.addr6 = si->sin6_addr;
+            *port = si->sin6_port; 
         } 
         ++gFd->socketCount;
     }
-    tr_lockUnlock( gFd->lock );
 
+    tr_lockUnlock( gFd->lock );
     return s;
 }
 
