@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: handshake.c 7884 2009-02-12 20:43:09Z charles $
+ * $Id: handshake.c 8698 2009-06-15 03:24:40Z charles $
  */
 
 #include <assert.h>
@@ -28,13 +28,15 @@
 #include "peer-io.h"
 #include "peer-mgr.h"
 #include "torrent.h"
-#include "trevent.h"
+#include "tr-dht.h"
 #include "utils.h"
 
 /* enable LibTransmission extension protocol */
 #define ENABLE_LTEP * /
 /* fast extensions */
 #define ENABLE_FAST * /
+/* DHT */
+#define ENABLE_DHT * /
 
 /***
 ****
@@ -62,7 +64,7 @@ enum
     CRYPTO_PROVIDE_CRYPTO          = 2,
 
     /* how long to wait before giving up on a handshake */
-    HANDSHAKE_TIMEOUT_MSEC         = 60 * 1000
+    HANDSHAKE_TIMEOUT_SEC          = 60
 };
 
 
@@ -82,6 +84,14 @@ enum
  #define HANDSHAKE_SET_FASTEXT( bits ) ( (void)0 ) 
 #endif 
 
+#ifdef ENABLE_DHT
+ #define HANDSHAKE_HAS_DHT( bits ) ( ( ( bits )[7] & 0x01 ) ? 1 : 0 )
+ #define HANDSHAKE_SET_DHT( bits ) ( ( bits )[7] |= 0x01 )
+#else
+ #define HANDSHAKE_HAS_DHT( bits ) ( 0 )
+ #define HANDSHAKE_SET_DHT( bits ) ( (void)0 )
+#endif 
+
 /* http://www.azureuswiki.com/index.php/Extension_negotiation_protocol
    these macros are to be used if both extended messaging and the
    azureus protocol is supported, they indicate which protocol is preferred */
@@ -96,7 +106,6 @@ struct tr_handshake
     tr_peerIo *           io;
     tr_crypto *           crypto;
     tr_session *          session;
-    uint8_t               myPublicKey[KEY_LEN];
     uint8_t               mySecret[KEY_LEN];
     uint8_t               state;
     tr_encryption_mode    encryptionMode;
@@ -106,10 +115,9 @@ struct tr_handshake
     uint32_t              crypto_select;
     uint32_t              crypto_provide;
     uint8_t               myReq1[SHA_DIGEST_LENGTH];
-    uint8_t               peer_id[PEER_ID_LEN];
     handshakeDoneCB       doneCB;
     void *                doneUserData;
-    tr_timer *            timeout;
+    struct event          timeout_timer;
 };
 
 /**
@@ -203,11 +211,9 @@ setReadState( tr_handshake * handshake,
     setState( handshake, state );
 }
 
-static uint8_t *
-buildHandshakeMessage( tr_handshake * handshake,
-                       int *          setme_len )
+static void
+buildHandshakeMessage( tr_handshake * handshake, uint8_t * buf )
 {
-    uint8_t          * buf = tr_new0( uint8_t, HANDSHAKE_SIZE );
     uint8_t          * walk = buf;
     const uint8_t    * torrentHash = tr_cryptoGetTorrentHash( handshake->crypto );
     const tr_torrent * tor = tr_torrentFindFromHash( handshake->session, torrentHash );
@@ -219,6 +225,12 @@ buildHandshakeMessage( tr_handshake * handshake,
     HANDSHAKE_SET_LTEP( walk );
     HANDSHAKE_SET_FASTEXT( walk );
 
+    /* Note that this doesn't depend on whether the torrent is private.  We
+       don't accept DHT peers for a private torrent, but we participate in
+       the DHT regardless. */
+    if(tr_dhtEnabled(handshake->session))
+        HANDSHAKE_SET_DHT( walk );
+
     walk += HANDSHAKE_FLAGS_LEN;
     memcpy( walk, torrentHash, SHA_DIGEST_LENGTH );
     walk += SHA_DIGEST_LENGTH;
@@ -227,8 +239,6 @@ buildHandshakeMessage( tr_handshake * handshake,
 
     assert( strlen( ( const char* )peer_id ) == PEER_ID_LEN );
     assert( walk - buf == HANDSHAKE_SIZE );
-    *setme_len = walk - buf;
-    return buf;
 }
 
 static int tr_handshakeDone( tr_handshake * handshake,
@@ -250,7 +260,8 @@ parseHandshake( tr_handshake *    handshake,
     uint8_t reserved[HANDSHAKE_FLAGS_LEN];
     uint8_t hash[SHA_DIGEST_LENGTH];
     const tr_torrent * tor;
-    const uint8_t * peer_id;
+    const uint8_t * tor_peer_id;
+    uint8_t peer_id[PEER_ID_LEN];
 
     dbgmsg( handshake, "payload: need %d, got %zu",
             (int)HANDSHAKE_SIZE, EVBUFFER_LENGTH( inbuf ) );
@@ -278,18 +289,16 @@ parseHandshake( tr_handshake *    handshake,
     }
 
     /* peer_id */
-    tr_peerIoReadBytes( handshake->io, inbuf, handshake->peer_id,
-                       sizeof( handshake->peer_id ) );
-    tr_peerIoSetPeersId( handshake->io, handshake->peer_id );
+    tr_peerIoReadBytes( handshake->io, inbuf, peer_id, sizeof( peer_id ) );
+    tr_peerIoSetPeersId( handshake->io, peer_id );
 
     /* peer id */
     handshake->havePeerID = TRUE;
-    dbgmsg( handshake, "peer-id is [%*.*s]", PEER_ID_LEN, PEER_ID_LEN,
-            handshake->peer_id );
+    dbgmsg( handshake, "peer-id is [%*.*s]", PEER_ID_LEN, PEER_ID_LEN, peer_id );
 
     tor = tr_torrentFindFromHash( handshake->session, hash );
-    peer_id = tor && tor->peer_id ? tor->peer_id : tr_getPeerId( );
-    if( !memcmp( handshake->peer_id, peer_id, PEER_ID_LEN ) )
+    tor_peer_id = tor && tor->peer_id ? tor->peer_id : tr_getPeerId( );
+    if( !memcmp( peer_id, tor_peer_id, PEER_ID_LEN ) )
     {
         dbgmsg( handshake, "streuth!  we've connected to ourselves." );
         return HANDSHAKE_PEER_IS_SELF;
@@ -303,6 +312,10 @@ parseHandshake( tr_handshake *    handshake,
 
     tr_peerIoEnableFEXT( handshake->io, HANDSHAKE_HAS_FASTEXT( reserved ) );
 
+    /* This doesn't depend on whether the torrent is private. */
+    if( tor && tor->session->isDHTEnabled )
+        tr_peerIoEnableDHT( handshake->io, HANDSHAKE_HAS_DHT( reserved ) );
+
     return HANDSHAKE_OK;
 }
 
@@ -318,26 +331,23 @@ sendYa( tr_handshake * handshake )
 {
     int               len;
     const uint8_t *   public_key;
-    struct evbuffer * outbuf = tr_getBuffer( );
-    uint8_t           pad_a[PadA_MAXLEN];
+    char              outbuf[ KEY_LEN + PadA_MAXLEN ], *walk=outbuf;
 
     /* add our public key (Ya) */
     public_key = tr_cryptoGetMyPublicKey( handshake->crypto, &len );
     assert( len == KEY_LEN );
     assert( public_key );
-    evbuffer_add( outbuf, public_key, len );
+    memcpy( walk, public_key, len );
+    walk += len;
 
     /* add some bullshit padding */
     len = tr_cryptoRandInt( PadA_MAXLEN );
-    tr_cryptoRandBuf( pad_a, len );
-    evbuffer_add( outbuf, pad_a, len );
+    tr_cryptoRandBuf( walk, len );
+    walk += len;
 
     /* send it */
     setReadState( handshake, AWAITING_YB );
-    tr_peerIoWriteBuf( handshake->io, outbuf, FALSE );
-
-    /* cleanup */
-    tr_releaseBuffer( outbuf );
+    tr_peerIoWrite( handshake->io, outbuf, walk-outbuf, FALSE );
 }
 
 static uint32_t
@@ -430,7 +440,7 @@ readYb( tr_handshake *    handshake,
 
     /* now send these: HASH('req1', S), HASH('req2', SKEY) xor HASH('req3', S),
      * ENCRYPT(VC, crypto_provide, len(PadC), PadC, len(IA)), ENCRYPT(IA) */
-    outbuf = tr_getBuffer( );
+    outbuf = evbuffer_new( );
 
     /* HASH('req1', S) */
     {
@@ -472,14 +482,13 @@ readYb( tr_handshake *    handshake,
 
     /* ENCRYPT len(IA)), ENCRYPT(IA) */
     {
-        int       msglen;
-        uint8_t * msg = buildHandshakeMessage( handshake, &msglen );
+        uint8_t msg[HANDSHAKE_SIZE];
+        buildHandshakeMessage( handshake, msg );
 
-        tr_peerIoWriteUint16( handshake->io, outbuf, msglen );
-        tr_peerIoWriteBytes( handshake->io, outbuf, msg, msglen );
+        tr_peerIoWriteUint16( handshake->io, outbuf, sizeof( msg ) );
+        tr_peerIoWriteBytes( handshake->io, outbuf, msg, sizeof( msg ) );
 
         handshake->haveSentBitTorrentHandshake = 1;
-        tr_free( msg );
     }
 
     /* send it */
@@ -488,7 +497,7 @@ readYb( tr_handshake *    handshake,
     tr_peerIoWriteBuf( handshake->io, outbuf, FALSE );
 
     /* cleanup */
-    tr_releaseBuffer( outbuf );
+    evbuffer_free( outbuf );
     return READ_LATER;
 }
 
@@ -697,10 +706,9 @@ readHandshake( tr_handshake *    handshake,
 
     if( !handshake->haveSentBitTorrentHandshake )
     {
-        int       msgSize;
-        uint8_t * msg = buildHandshakeMessage( handshake, &msgSize );
-        tr_peerIoWrite( handshake->io, msg, msgSize, FALSE );
-        tr_free( msg );
+        uint8_t msg[HANDSHAKE_SIZE];
+        buildHandshakeMessage( handshake, msg );
+        tr_peerIoWrite( handshake->io, msg, sizeof( msg ), FALSE );
         handshake->haveSentBitTorrentHandshake = 1;
     }
 
@@ -715,23 +723,24 @@ readPeerId( tr_handshake    * handshake,
     int  peerIsGood;
     char client[128];
     tr_torrent * tor;
-    const uint8_t * peer_id;
+    const uint8_t * tor_peer_id;
+    uint8_t peer_id[PEER_ID_LEN];
 
     if( EVBUFFER_LENGTH( inbuf ) < PEER_ID_LEN )
         return READ_LATER;
 
     /* peer id */
-    tr_peerIoReadBytes( handshake->io, inbuf, handshake->peer_id, PEER_ID_LEN );
-    tr_peerIoSetPeersId( handshake->io, handshake->peer_id );
+    tr_peerIoReadBytes( handshake->io, inbuf, peer_id, PEER_ID_LEN );
+    tr_peerIoSetPeersId( handshake->io, peer_id );
     handshake->havePeerID = TRUE;
-    tr_clientForId( client, sizeof( client ), handshake->peer_id );
+    tr_clientForId( client, sizeof( client ), peer_id );
     dbgmsg( handshake, "peer-id is [%s] ... isIncoming is %d", client,
             tr_peerIoIsIncoming( handshake->io ) );
 
     /* if we've somehow connected to ourselves, don't keep the connection */
     tor = tr_torrentFindFromHash( handshake->session, tr_peerIoGetTorrentHash( handshake->io ) );
-    peer_id = tor && tor->peer_id ? tor->peer_id : tr_getPeerId( );
-    peerIsGood = memcmp( handshake->peer_id, peer_id, PEER_ID_LEN ) != 0;
+    tor_peer_id = tor && tor->peer_id ? tor->peer_id : tr_getPeerId( );
+    peerIsGood = memcmp( peer_id, tor_peer_id, PEER_ID_LEN ) != 0;
     dbgmsg( handshake, "isPeerGood == %d", peerIsGood );
     return tr_handshakeDone( handshake, peerIsGood );
 }
@@ -922,7 +931,7 @@ readIA( tr_handshake *    handshake,
     **/
 
     tr_cryptoEncryptInit( handshake->crypto );
-    outbuf = tr_getBuffer( );
+    outbuf = evbuffer_new( );
 
     dbgmsg( handshake, "sending vc" );
     /* send VC */
@@ -942,7 +951,7 @@ readIA( tr_handshake *    handshake,
     else
     {
         dbgmsg( handshake, "peer didn't offer an encryption mode we like." );
-        tr_releaseBuffer( outbuf );
+        evbuffer_free( outbuf );
         return tr_handshakeDone( handshake, FALSE );
     }
 
@@ -965,16 +974,16 @@ readIA( tr_handshake *    handshake,
     dbgmsg( handshake, "sending handshake" );
     /* send our handshake */
     {
-        int       msgSize;
-        uint8_t * msg = buildHandshakeMessage( handshake, &msgSize );
-        tr_peerIoWriteBytes( handshake->io, outbuf, msg, msgSize );
+        uint8_t msg[HANDSHAKE_SIZE];
+        buildHandshakeMessage( handshake, msg );
+
+        tr_peerIoWriteBytes( handshake->io, outbuf, msg, sizeof( msg ) );
         handshake->haveSentBitTorrentHandshake = 1;
-        tr_free( msg );
     }
 
     /* send it out */
     tr_peerIoWriteBuf( handshake->io, outbuf, FALSE );
-    tr_releaseBuffer( outbuf );
+    evbuffer_free( outbuf );
 
     /* now await the handshake */
     setState( handshake, AWAITING_PAYLOAD_STREAM );
@@ -1085,14 +1094,13 @@ fireDoneFunc( tr_handshake * handshake,
               tr_bool        isConnected )
 {
     const uint8_t * peer_id = isConnected && handshake->havePeerID
-                              ? handshake->peer_id
-                              : NULL;
-    const int       success = ( *handshake->doneCB )( handshake,
-                                                      handshake->io,
-                                                      isConnected,
-                                                      peer_id,
-                                                      handshake->
-                                                      doneUserData );
+                            ? tr_peerIoGetPeersId( handshake->io )
+                            : NULL;
+    const int success = ( *handshake->doneCB )( handshake,
+                                                handshake->io,
+                                                isConnected,
+                                                peer_id,
+                                                handshake->doneUserData );
 
     return success;
 }
@@ -1103,7 +1111,7 @@ tr_handshakeFree( tr_handshake * handshake )
     if( handshake->io )
         tr_peerIoUnref( handshake->io ); /* balanced by the ref in tr_handshakeNew */
 
-    tr_timerFree( &handshake->timeout );
+    evtimer_del( &handshake->timeout_timer );
 
     tr_free( handshake );
 }
@@ -1145,14 +1153,13 @@ gotError( tr_peerIo  * io UNUSED,
       && ( handshake->encryptionMode != TR_ENCRYPTION_REQUIRED )
       && ( !tr_peerIoReconnect( handshake->io ) ) )
     {
-        int       msgSize;
-        uint8_t * msg;
+        uint8_t msg[HANDSHAKE_SIZE];
+
         dbgmsg( handshake, "handshake failed, trying plaintext..." );
-        msg = buildHandshakeMessage( handshake, &msgSize );
+        buildHandshakeMessage( handshake, msg );
         handshake->haveSentBitTorrentHandshake = 1;
         setReadState( handshake, AWAITING_HANDSHAKE );
-        tr_peerIoWrite( handshake->io, msg, msgSize, FALSE );
-        tr_free( msg );
+        tr_peerIoWrite( handshake->io, msg, sizeof( msg ), FALSE );
     }
     else
     {
@@ -1166,11 +1173,10 @@ gotError( tr_peerIo  * io UNUSED,
 ***
 **/
 
-static int
-handshakeTimeout( void * handshake )
+static void
+handshakeTimeout( int foo UNUSED, short bar UNUSED, void * handshake )
 {
     tr_handshakeAbort( handshake );
-    return FALSE;
 }
 
 tr_handshake*
@@ -1188,7 +1194,9 @@ tr_handshakeNew( tr_peerIo *        io,
     handshake->doneCB = doneCB;
     handshake->doneUserData = doneUserData;
     handshake->session = tr_peerIoGetSession( io );
-    handshake->timeout = tr_timerNew( handshake->session, handshakeTimeout, handshake, HANDSHAKE_TIMEOUT_MSEC );
+
+    evtimer_set( &handshake->timeout_timer, handshakeTimeout, handshake );
+    tr_timerAdd( &handshake->timeout_timer, HANDSHAKE_TIMEOUT_SEC, 0 );
 
     tr_peerIoRef( io ); /* balanced by the unref in tr_handshakeFree */
     tr_peerIoSetIOFuncs( handshake->io, canRead, NULL, gotError, handshake );
@@ -1200,12 +1208,12 @@ tr_handshakeNew( tr_peerIo *        io,
         sendYa( handshake );
     else
     {
-        int       msgSize;
-        uint8_t * msg = buildHandshakeMessage( handshake, &msgSize );
+        uint8_t msg[HANDSHAKE_SIZE];
+        buildHandshakeMessage( handshake, msg );
+
         handshake->haveSentBitTorrentHandshake = 1;
         setReadState( handshake, AWAITING_HANDSHAKE );
-        tr_peerIoWrite( handshake->io, msg, msgSize, FALSE );
-        tr_free( msg );
+        tr_peerIoWrite( handshake->io, msg, sizeof( msg ), FALSE );
     }
 
     return handshake;
