@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: makemeta.c 7664 2009-01-11 17:46:51Z charles $
+ * $Id: makemeta.c 8588 2009-06-02 01:48:48Z charles $
  */
 
 #include <assert.h>
@@ -114,8 +114,7 @@ builderFileCompare( const void * va,
 }
 
 tr_metainfo_builder*
-tr_metaInfoBuilderCreate( tr_session * session,
-                          const char * topFile )
+tr_metaInfoBuilderCreate( const char * topFile )
 {
     int                   i;
     struct FileList *     files;
@@ -123,7 +122,7 @@ tr_metaInfoBuilderCreate( tr_session * session,
     tr_metainfo_builder * ret = tr_new0( tr_metainfo_builder, 1 );
 
     ret->top = tr_strdup( topFile );
-    ret->handle = session;
+
     {
         struct stat sb;
         stat( topFile, &sb );
@@ -418,7 +417,7 @@ tr_realMakeMetaInfo( tr_metainfo_builder * builder )
     /* save the file */
     if( !builder->result && !builder->abortFlag )
     {
-        if( tr_bencSaveFile( builder->outputFile, &top ) )
+        if( tr_bencToFile( &top, TR_FMT_BENC, builder->outputFile ) )
         {
             builder->my_errno = errno;
             tr_strlcpy( builder->errfile, builder->outputFile,
@@ -445,29 +444,25 @@ static tr_metainfo_builder * queue = NULL;
 static tr_thread *           workerThread = NULL;
 
 static tr_lock*
-getQueueLock( tr_session * session )
+getQueueLock( void )
 {
     static tr_lock * lock = NULL;
-    tr_globalLock( session );
 
     if( !lock )
         lock = tr_lockNew( );
 
-    tr_globalUnlock( session );
     return lock;
 }
 
 static void
-makeMetaWorkerFunc( void * user_data )
+makeMetaWorkerFunc( void * unused UNUSED )
 {
-    tr_session * session = user_data;
-
-    for( ; ; )
+    for( ;; )
     {
         tr_metainfo_builder * builder = NULL;
 
         /* find the next builder to process */
-        tr_lock * lock = getQueueLock( session );
+        tr_lock * lock = getQueueLock( );
         tr_lockLock( lock );
         if( queue )
         {
@@ -522,12 +517,12 @@ tr_makeMetaInfo( tr_metainfo_builder *   builder,
         builder->outputFile = tr_strdup_printf( "%s.torrent", builder->top );
 
     /* enqueue the builder */
-    lock = getQueueLock ( builder->handle );
+    lock = getQueueLock ( );
     tr_lockLock( lock );
     builder->nextBuilder = queue;
     queue = builder;
     if( !workerThread )
-        workerThread = tr_threadNew( makeMetaWorkerFunc, builder->handle );
+        workerThread = tr_threadNew( makeMetaWorkerFunc, NULL );
     tr_lockUnlock( lock );
 }
 
