@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: peer-io.c 8218 2009-04-12 15:01:29Z charles $
+ * $Id: peer-io.c 8605 2009-06-03 22:52:08Z charles $
  */
 
 #include <assert.h>
@@ -39,7 +39,7 @@
 #define MAGIC_NUMBER 206745
 
 static size_t
-getPacketOverhead( size_t d )
+guessPacketOverhead( size_t d )
 {
     /**
      * http://sd.wareonearth.com/~phil/net/overhead/
@@ -86,11 +86,11 @@ struct tr_datatype
 static void
 didWriteWrapper( tr_peerIo * io, size_t bytes_transferred )
 {
-    while( bytes_transferred )
-    {
+     while( bytes_transferred && tr_isPeerIo( io ) )
+     {
         struct tr_datatype * next = __tr_list_entry( io->outbuf_datatypes.next, struct tr_datatype, head );
         const size_t payload = MIN( next->length, bytes_transferred );
-        const size_t overhead = getPacketOverhead( payload );
+        const size_t overhead = guessPacketOverhead( payload );
 
         tr_bandwidthUsed( &io->bandwidth, TR_UP, payload, next->isPieceData );
 
@@ -99,13 +99,16 @@ didWriteWrapper( tr_peerIo * io, size_t bytes_transferred )
 
         if( io->didWrite )
             io->didWrite( io, payload, next->isPieceData, io->userData );
-
-        bytes_transferred -= payload;
-        next->length -= payload;
-        if( !next->length ) {
-            __tr_list_remove( io->outbuf_datatypes.next );
-            tr_free( next );
-	}
+        
+        if( tr_isPeerIo( io ) )
+        {
+            bytes_transferred -= payload;
+            next->length -= payload;
+            if( !next->length ) {
+                __tr_list_remove( io->outbuf_datatypes.next );
+                tr_free( next );
+            }
+        }
     }
 }
 
@@ -614,6 +617,16 @@ tr_peerIoEnableLTEP( tr_peerIo  * io,
     io->extendedProtocolSupported = flag;
 }
 
+void
+tr_peerIoEnableDHT( tr_peerIo * io, tr_bool flag )
+{
+    assert( tr_isPeerIo( io ) );
+    assert( tr_isBool( flag ) );
+
+    dbgmsg( io, "setting DHT support flag to %d", (flag!=0) );
+    io->dhtSupported = flag;
+}
+
 /**
 ***
 **/
@@ -627,8 +640,8 @@ getDesiredOutputBufferSize( const tr_peerIo * io, uint64_t now )
      * It's okay to tweak this as needed */
     const double maxBlockSize = 16 * 1024; /* 16 KiB is from BT spec */
     const double currentSpeed = tr_bandwidthGetPieceSpeed( &io->bandwidth, now, TR_UP );
-    const double period = 20; /* arbitrary */
-    const double numBlocks = 5.5; /* the 5 is arbitrary; the .5 is to leave room for messages */
+    const double period = 15; /* arbitrary */
+    const double numBlocks = 3.5; /* the 3 is arbitrary; the .5 is to leave room for messages */
     return MAX( maxBlockSize*numBlocks, currentSpeed*1024*period );
 }
 
@@ -670,6 +683,7 @@ tr_peerIoWrite( tr_peerIo   * io,
                 size_t        byteCount,
                 tr_bool       isPieceData )
 {
+    /* FIXME(libevent2): this implementation snould be moved to tr_peerIoWriteBuf.   This function should be implemented as evbuffer_new() + evbuffer_add_reference() + a call to tr_peerIoWriteBuf() + evbuffer_free() */
     struct tr_datatype * datatype;
 
     assert( tr_amInEventThread( io->session ) );
@@ -686,6 +700,7 @@ tr_peerIoWrite( tr_peerIo   * io,
     {
         case PEER_ENCRYPTION_RC4:
         {
+            /* FIXME(libevent2): use evbuffer_reserve_space() and evbuffer_commit_space() instead of tmp */
             uint8_t tmp[MAX_STACK_ARRAY_SIZE];
             const uint8_t * walk = bytes;
             evbuffer_expand( io->outbuf, byteCount );
@@ -715,6 +730,7 @@ tr_peerIoWriteBuf( tr_peerIo         * io,
                    struct evbuffer   * buf,
                    tr_bool             isPieceData )
 {
+    /* FIXME(libevent2): loop through calls to evbuffer_get_contiguous_space() + evbuffer_drain() */
     const size_t n = EVBUFFER_LENGTH( buf );
     tr_peerIoWrite( io, EVBUFFER_DATA( buf ), n, isPieceData );
     evbuffer_drain( buf, n );
@@ -731,6 +747,7 @@ tr_peerIoReadBytes( tr_peerIo       * io,
                     size_t            byteCount )
 {
     assert( tr_isPeerIo( io ) );
+    /* FIXME(libevent2): use evbuffer_get_length() */
     assert( EVBUFFER_LENGTH( inbuf ) >= byteCount );
 
     switch( io->encryptionMode )
@@ -740,6 +757,7 @@ tr_peerIoReadBytes( tr_peerIo       * io,
             break;
 
         case PEER_ENCRYPTION_RC4:
+            /* FIXME(libevent2): loop through calls to evbuffer_get_contiguous_space() + evbuffer_drain() */
             tr_cryptoDecrypt( io->crypto, byteCount, EVBUFFER_DATA(inbuf), bytes );
             evbuffer_drain(inbuf, byteCount );
             break;
@@ -844,6 +862,26 @@ tr_peerIoFlush( tr_peerIo  * io, tr_direction dir, size_t limit )
     return bytesUsed;
 }
 
+int
+tr_peerIoFlushOutgoingProtocolMsgs( tr_peerIo * io )
+{
+    size_t byteCount = 0;
+    struct __tr_list * walk;
+    struct __tr_list * fencepost = &io->outbuf_datatypes;
+
+    /* count up how many bytes are used by non-piece-data messages
+       at the front of our outbound queue */
+    for( walk=fencepost->next; walk!=fencepost; walk=walk->next ) {
+        struct tr_datatype * d = __tr_list_entry( walk, struct tr_datatype, head );
+        if( d->isPieceData )
+            break;
+        byteCount += d->length;
+    }
+
+    return tr_peerIoFlush( io, TR_UP, byteCount );
+}
+
+
 /***
 ****
 ****/
