@@ -8,7 +8,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: peer-mgr.c 8218 2009-04-12 15:01:29Z charles $
+ * $Id: peer-mgr.c 8709 2009-06-16 19:31:17Z charles $
  */
 
 #include <assert.h>
@@ -53,7 +53,7 @@ enum
     BANDWIDTH_PERIOD_MSEC = 500,
 
     /* how frequently to age out old piece request lists */
-    REFILL_UPKEEP_PERIOD_MSEC = 10000,
+    REFILL_UPKEEP_PERIOD_MSEC = ( 10 * 1000 ),
 
     /* how frequently to decide which peers live and die */
     RECONNECT_PERIOD_MSEC = 500,
@@ -360,16 +360,14 @@ peerDestructor( tr_peer * peer )
 }
 
 static void
-removePeer( Torrent * t,
-            tr_peer * peer )
+removePeer( Torrent * t, tr_peer * peer )
 {
-    tr_peer *          removed;
-    struct peer_atom * atom;
+    tr_peer * removed;
+    struct peer_atom * atom = peer->atom;
 
     assert( torrentIsLocked( t ) );
-
-    atom = getExistingAtom( t, &peer->addr );
     assert( atom );
+
     atom->time = time( NULL );
 
     removed = tr_ptrArrayRemoveSorted( &t->peers, peer, peerCompare );
@@ -452,28 +450,33 @@ tr_peerMgr*
 tr_peerMgrNew( tr_session * session )
 {
     tr_peerMgr * m = tr_new0( tr_peerMgr, 1 );
-
     m->session = session;
     m->incomingHandshakes = TR_PTR_ARRAY_INIT;
-    m->bandwidthTimer    = tr_timerNew( session, bandwidthPulse, m, BANDWIDTH_PERIOD_MSEC );
-    m->rechokeTimer      = tr_timerNew( session, rechokePulse,   m, RECHOKE_PERIOD_MSEC );
-    m->reconnectTimer    = tr_timerNew( session, reconnectPulse, m, RECONNECT_PERIOD_MSEC );
-    m->refillUpkeepTimer = tr_timerNew( session, refillUpkeep,   m, REFILL_UPKEEP_PERIOD_MSEC );
-
-    rechokePulse( m );
-
     return m;
 }
 
+static void
+deleteTimers( struct tr_peerMgr * m )
+{
+    if( m->bandwidthTimer )
+        tr_timerFree( &m->bandwidthTimer );
+
+    if( m->rechokeTimer )
+        tr_timerFree( &m->rechokeTimer );
+
+    if( m->reconnectTimer )
+        tr_timerFree( &m->reconnectTimer );
+
+    if( m->refillUpkeepTimer )
+        tr_timerFree( &m->refillUpkeepTimer );
+}
+
 void
 tr_peerMgrFree( tr_peerMgr * manager )
 {
     managerLock( manager );
 
-    tr_timerFree( &manager->refillUpkeepTimer );
-    tr_timerFree( &manager->reconnectTimer );
-    tr_timerFree( &manager->rechokeTimer );
-    tr_timerFree( &manager->bandwidthTimer );
+    deleteTimers( manager );
 
     /* free the handshakes.  Abort invokes handshakeDoneCB(), which removes
      * the item from manager->handshakes, so this is a little roundabout... */
@@ -911,8 +914,7 @@ broadcastGotBlock( Torrent * t, uint32_t index, uint32_t offset, uint32_t length
 }
 
 static void
-addStrike( Torrent * t,
-           tr_peer * peer )
+addStrike( Torrent * t, tr_peer * peer )
 {
     tordbg( t, "increasing peer %s strike count to %d",
             tr_peerIoAddrStr( &peer->addr,
@@ -920,7 +922,7 @@ addStrike( Torrent * t,
 
     if( ++peer->strikes >= MAX_BAD_PIECES_PER_PEER )
     {
-        struct peer_atom * atom = getExistingAtom( t, &peer->addr );
+        struct peer_atom * atom = peer->atom;
         atom->myflags |= MYFLAG_BANNED;
         peer->doPurge = 1;
         tordbg( t, "banning peer %s", tr_peerIoAddrStr( &atom->addr, atom->port ) );
@@ -1008,10 +1010,8 @@ peerCallbackFunc( void * vpeer, void * vevent, void * vt )
     {
         case TR_PEER_UPLOAD_ONLY:
             /* update our atom */
-            if( peer ) {
-                struct peer_atom * a = getExistingAtom( t, &peer->addr );
-                a->uploadOnly = e->uploadOnly ? UPLOAD_ONLY_YES : UPLOAD_ONLY_NO;
-            }
+            if( peer )
+                peer->atom->uploadOnly = e->uploadOnly ? UPLOAD_ONLY_YES : UPLOAD_ONLY_NO;
             break;
 
         case TR_PEER_NEED_REQ:
@@ -1027,7 +1027,7 @@ peerCallbackFunc( void * vpeer, void * vevent, void * vt )
             const time_t now = time( NULL );
             tr_torrent * tor = t->tor;
 
-            tor->activityDate = now;
+            tr_torrentSetActivityDate( tor, now );
 
             if( e->wasPieceData )
                 tor->uploadedCur += e->length;
@@ -1037,11 +1037,10 @@ peerCallbackFunc( void * vpeer, void * vevent, void * vt )
                 tr_statsAddUploaded( tor->session, e->length );
 
             /* update our atom */
-            if( peer ) {
-                struct peer_atom * a = getExistingAtom( t, &peer->addr );
-                if( e->wasPieceData )
-                    a->piece_data_time = now;
-            }
+            if( peer && e->wasPieceData )
+                peer->atom->piece_data_time = now;
+
+            tor->needsSeedRatioCheck = TRUE;
 
             break;
         }
@@ -1060,7 +1059,8 @@ peerCallbackFunc( void * vpeer, void * vevent, void * vt )
         {
             const time_t now = time( NULL );
             tr_torrent * tor = t->tor;
-            tor->activityDate = now;
+
+            tr_torrentSetActivityDate( tor, now );
 
             /* only add this to downloadedCur if we got it from a peer --
              * webseeds shouldn't count against our ratio.  As one tracker
@@ -1076,11 +1076,8 @@ peerCallbackFunc( void * vpeer, void * vevent, void * vt )
                 tr_statsAddDownloaded( tor->session, e->length );
 
             /* update our atom */
-            if( peer ) {
-                struct peer_atom * a = getExistingAtom( t, &peer->addr );
-                if( e->wasPieceData )
-                    a->piece_data_time = now;
-            }
+            if( peer && e->wasPieceData )
+                peer->atom->piece_data_time = now;
 
             break;
         }
@@ -1089,7 +1086,7 @@ peerCallbackFunc( void * vpeer, void * vevent, void * vt )
         {
             if( peer )
             {
-                struct peer_atom * atom = getExistingAtom( t, &peer->addr );
+                struct peer_atom * atom = peer->atom;
                 const int peerIsSeed = e->progress >= 1.0;
                 if( peerIsSeed ) {
                     tordbg( t, "marking peer %s as a seed", tr_peerIoAddrStr( &atom->addr, atom->port ) );
@@ -1223,7 +1220,7 @@ getMaxPeerCount( const tr_torrent * tor )
 static int
 getPeerCount( const Torrent * t )
 {
-    return tr_ptrArraySize( &t->peers );// + tr_ptrArraySize( &t->outgoingHandshakes );
+    return tr_ptrArraySize( &t->peers );/* + tr_ptrArraySize( &t->outgoingHandshakes ); */
 }
 
 /* FIXME: this is kind of a mess. */
@@ -1314,6 +1311,7 @@ myHandshakeDoneCB( tr_handshake  * handshake,
                 }
 
                 peer->port = port;
+                peer->atom = atom;
                 peer->io = tr_handshakeStealIO( handshake ); /* this steals its refcount too, which is
                                                                 balanced by our unref in peerDestructor()  */
                 tr_peerIoSetParent( peer->io, t->tor->bandwidth );
@@ -1551,7 +1549,7 @@ tr_peerMgrGetPeers( tr_torrent      * tor,
             const tr_peer * peer = peers[i];
             if( peer->addr.type == af )
             {
-                const struct peer_atom * atom = getExistingAtom( t, &peer->addr );
+                const struct peer_atom * atom = peer->atom;
 
                 assert( tr_isAddress( &peer->addr ) );
                 walk->addr = peer->addr;
@@ -1575,14 +1573,32 @@ tr_peerMgrGetPeers( tr_torrent      * tor,
     return peersReturning;
 }
 
+static void
+ensureMgrTimersExist( struct tr_peerMgr * m )
+{
+    tr_session * s = m->session;
+
+    if( m->bandwidthTimer == NULL )
+        m->bandwidthTimer = tr_timerNew( s, bandwidthPulse, m, BANDWIDTH_PERIOD_MSEC );
+
+    if( m->rechokeTimer == NULL )
+        m->rechokeTimer = tr_timerNew( s, rechokePulse, m, RECHOKE_PERIOD_MSEC );
+
+    if( m->reconnectTimer == NULL )
+        m->reconnectTimer = tr_timerNew( s, reconnectPulse, m, RECONNECT_PERIOD_MSEC );
+
+    if( m->refillUpkeepTimer == NULL )
+        m->refillUpkeepTimer = tr_timerNew( s, refillUpkeep, m, REFILL_UPKEEP_PERIOD_MSEC );
+}
+
 void
 tr_peerMgrStartTorrent( tr_torrent * tor )
 {
     Torrent * t = tor->torrentPeers;
 
+    assert( t != NULL );
     managerLock( t->manager );
-
-    assert( t );
+    ensureMgrTimersExist( t->manager );
 
     if( !t->isRunning )
     {
@@ -1592,6 +1608,7 @@ tr_peerMgrStartTorrent( tr_torrent * tor )
             refillSoon( t );
     }
 
+    rechokePulse( t->manager );
     managerUnlock( t->manager );
 }
 
@@ -1742,7 +1759,7 @@ tr_peerMgrTorrentStats( tr_torrent       * tor,
     for( i=0; i<size; ++i )
     {
         const tr_peer * peer = peers[i];
-        const struct peer_atom * atom = getExistingAtom( t, &peer->addr );
+        const struct peer_atom * atom = peer->atom;
 
         if( peer->io == NULL ) /* not connected */
             continue;
@@ -1770,6 +1787,25 @@ tr_peerMgrTorrentStats( tr_torrent       * tor,
     managerUnlock( t->manager );
 }
 
+float
+tr_peerMgrGetWebseedSpeed( const tr_torrent * tor, uint64_t now )
+{
+    int i;
+    float tmp;
+    float ret = 0;
+
+    const Torrent * t = tor->torrentPeers;
+    const int n = tr_ptrArraySize( &t->webseeds );
+    const tr_webseed ** webseeds = (const tr_webseed**) tr_ptrArrayBase( &t->webseeds );
+
+    for( i=0; i<n; ++i )
+        if( tr_webseedGetSpeed( webseeds[i], now, &tmp ) )
+            ret += tmp;
+
+    return ret;
+}
+
+
 float*
 tr_peerMgrWebSpeeds( const tr_torrent * tor )
 {
@@ -1822,11 +1858,11 @@ tr_peerMgrPeerStats( const tr_torrent    * tor,
     ret = tr_new0( tr_peer_stat, size );
     now = tr_date( );
 
-    for( i = 0; i < size; ++i )
+    for( i=0; i<size; ++i )
     {
         char *                   pch;
         const tr_peer *          peer = peers[i];
-        const struct peer_atom * atom = getExistingAtom( t, &peer->addr );
+        const struct peer_atom * atom = peer->atom;
         tr_peer_stat *           stat = ret + i;
 
         tr_ntop( &peer->addr, stat->addr, sizeof( stat->addr ) );
@@ -1856,6 +1892,7 @@ tr_peerMgrPeerStats( const tr_torrent    * tor,
         if( !stat->clientIsChoked && !stat->clientIsInterested ) *pch++ = 'K';
         if( !stat->peerIsChoked && !stat->peerIsInterested ) *pch++ = '?';
         if( stat->isEncrypted ) *pch++ = 'E';
+        if( stat->from == TR_PEER_FROM_DHT ) *pch++ = 'H';
         if( stat->from == TR_PEER_FROM_PEX ) *pch++ = 'X';
         if( stat->isIncoming ) *pch++ = 'I';
         *pch = '\0';
@@ -1929,7 +1966,7 @@ rechokeTorrent( Torrent * t )
     for( i = 0, size = 0; i < peerCount; ++i )
     {
         tr_peer * peer = peers[i];
-        struct peer_atom * atom = getExistingAtom( t, &peer->addr );
+        struct peer_atom * atom = peer->atom;
 
         if( peer->progress >= 1.0 ) /* choke all seeds */
         {
@@ -2048,7 +2085,7 @@ shouldPeerBeClosed( const Torrent    * t,
 {
     const tr_torrent *       tor = t->tor;
     const time_t             now = time( NULL );
-    const struct peer_atom * atom = getExistingAtom( t, &peer->addr );
+    const struct peer_atom * atom = peer->atom;
 
     /* if it's marked for purging, close it */
     if( peer->doPurge )
@@ -2146,8 +2183,8 @@ compareCandidates( const void * va,
     if( a->time != b->time )
         return a->time < b->time ? -1 : 1;
 
-    /* all other things being equal, prefer peers whose
-     * information comes from a more reliable source */
+    /* In order to avoid fragmenting the swarm, peers from trackers and
+     * from the DHT should be preferred to peers from PEX. */
     if( a->from != b->from )
         return a->from < b->from ? -1 : 1;
 
@@ -2252,10 +2289,11 @@ closePeer( Torrent * t, tr_peer * peer )
     assert( t != NULL );
     assert( peer != NULL );
 
+    atom = peer->atom;
+
     /* if we transferred piece data, then they might be good peers,
        so reset their `numFails' weight to zero.  otherwise we connected
        to them fruitlessly, so mark it as another fail */
-    atom = getExistingAtom( t, &peer->addr );
     if( atom->piece_data_time )
         atom->numFails = 0;
     else
@@ -2417,6 +2455,7 @@ pumpAllPeers( tr_peerMgr * mgr )
 static int
 bandwidthPulse( void * vmgr )
 {
+    tr_torrent * tor = NULL;
     tr_peerMgr * mgr = vmgr;
     managerLock( mgr );
 
@@ -2427,6 +2466,14 @@ bandwidthPulse( void * vmgr )
     tr_bandwidthAllocate( mgr->session->bandwidth, TR_UP, BANDWIDTH_PERIOD_MSEC );
     tr_bandwidthAllocate( mgr->session->bandwidth, TR_DOWN, BANDWIDTH_PERIOD_MSEC );
 
+    /* possibly stop torrents that have seeded enough */
+    while(( tor = tr_torrentNext( mgr->session, tor ))) {
+        if( tor->needsSeedRatioCheck ) {
+            tor->needsSeedRatioCheck = FALSE;
+            tr_torrentCheckSeedRatio( tor );
+        }
+    }
+
     managerUnlock( mgr );
     return TRUE;
 }
