@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: peer-msgs.c 8204 2009-04-10 17:34:25Z charles $
+ * $Id: peer-msgs.c 8698 2009-06-15 03:24:40Z charles $
  */
 
 #include <assert.h>
@@ -36,8 +36,9 @@
 #include "request-list.h"
 #include "stats.h"
 #include "torrent.h"
-#include "trevent.h"
+#include "tr-dht.h"
 #include "utils.h"
+#include "version.h"
 
 /**
 ***
@@ -75,7 +76,7 @@ enum
     /* idle seconds before we send a keepalive */
     KEEPALIVE_INTERVAL_SECS = 100,
 
-    PEX_INTERVAL            = ( 90 * 1000 ), /* msec between sendPex() calls */
+    PEX_INTERVAL_SECS       = 90, /* sec between sendPex() calls */
 
 
     MAX_BLOCK_SIZE          = ( 1024 * 16 ),
@@ -143,7 +144,7 @@ struct tr_peermsgs
     tr_bool         peerSupportsPex;
     tr_bool         clientSentLtepHandshake;
     tr_bool         peerSentLtepHandshake;
-    tr_bool         haveFastSet;
+    /*tr_bool         haveFastSet;*/
 
     uint8_t         state;
     uint8_t         ut_pex_id;
@@ -151,8 +152,10 @@ struct tr_peermsgs
     uint16_t        pexCount6;
     uint16_t        maxActiveRequests;
 
+#if 0
     size_t                 fastsetSize;
     tr_piece_index_t       fastset[MAX_FAST_SET_SIZE];
+#endif
 
     /* how long the outMessages batch should be allowed to grow before
      * it's flushed -- some messages (like requests >:) should be sent
@@ -172,11 +175,10 @@ struct tr_peermsgs
     struct request_list    clientAskedFor;
     struct request_list    clientWillAskFor;
 
-    tr_timer             * pexTimer;
     tr_pex               * pex;
     tr_pex               * pex6;
 
-    time_t                 clientSentPexAt;
+    /*time_t                 clientSentPexAt;*/
     time_t                 clientSentAnythingAt;
 
     /* when we started batching the outMessages */
@@ -188,6 +190,8 @@ struct tr_peermsgs
        supplied a reqq argument, it's stored here.  otherwise the
        value is zero and should be ignored. */
     int64_t               reqq;
+
+    struct event          pexTimer;
 };
 
 /**
@@ -205,7 +209,7 @@ myDebug( const char * file, int line,
     {
         va_list           args;
         char              timestr[64];
-        struct evbuffer * buf = tr_getBuffer( );
+        struct evbuffer * buf = evbuffer_new( );
         char *            base = tr_basename( file );
 
         evbuffer_add_printf( buf, "[%s] %s - %s [%s]: ",
@@ -217,10 +221,11 @@ myDebug( const char * file, int line,
         evbuffer_add_vprintf( buf, fmt, args );
         va_end( args );
         evbuffer_add_printf( buf, " (%s:%d)\n", base, line );
+        /* FIXME(libevent2) tr_getLog() should return an fd, then use evbuffer_write() here */
         fwrite( EVBUFFER_DATA( buf ), 1, EVBUFFER_LENGTH( buf ), fp );
 
         tr_free( base );
-        tr_releaseBuffer( buf );
+        evbuffer_free( buf );
     }
 }
 
@@ -305,6 +310,18 @@ protocolSendCancel( tr_peermsgs               * msgs,
     pokeBatchPeriod( msgs, IMMEDIATE_PRIORITY_INTERVAL_SECS );
 }
 
+static void
+protocolSendPort(tr_peermsgs *msgs, uint16_t port)
+{
+    tr_peerIo       * io  = msgs->peer->io;
+    struct evbuffer * out = msgs->outMessages;
+
+    dbgmsg( msgs, "sending Port %u", port);
+    tr_peerIoWriteUint32( io, out, 3 );
+    tr_peerIoWriteUint8 ( io, out, BT_PORT );
+    tr_peerIoWriteUint16( io, out, port);
+}
+
 static void
 protocolSendHave( tr_peermsgs * msgs,
                   uint32_t      index )
@@ -526,12 +543,16 @@ tr_generateAllowedSet( tr_piece_index_t * setmePieces,
 
     if( addr->type == TR_AF_INET )
     {
-        uint8_t w[SHA_DIGEST_LENGTH + 4];
+        uint8_t w[SHA_DIGEST_LENGTH + 4], *walk=w;
         uint8_t x[SHA_DIGEST_LENGTH];
 
-        *(uint32_t*)w = ntohl( htonl( addr->addr.addr4.s_addr ) & 0xffffff00 );   /* (1) */
-        memcpy( w + 4, infohash, SHA_DIGEST_LENGTH );                /* (2) */
-        tr_sha1( x, w, sizeof( w ), NULL );                          /* (3) */
+        uint32_t ui32 = ntohl( htonl( addr->addr.addr4.s_addr ) & 0xffffff00 );   /* (1) */
+        memcpy( w, &ui32, sizeof( uint32_t ) );
+        walk += sizeof( uint32_t );
+        memcpy( walk, infohash, SHA_DIGEST_LENGTH );                 /* (2) */
+        walk += SHA_DIGEST_LENGTH;
+        tr_sha1( x, w, walk-w, NULL );                               /* (3) */
+        assert( sizeof( w ) == walk-w );
 
         while( setSize<desiredSetSize )
         {
@@ -977,7 +998,7 @@ sendLtepHandshake( tr_peermsgs * msgs )
     m  = tr_bencDictAddDict( &val, "m", 1 );
     if( pex )
         tr_bencDictAddInt( m, "ut_pex", TR_LTEP_PEX );
-    buf = tr_bencSave( &val, &len );
+    buf = tr_bencToStr( &val, TR_FMT_BENC, &len );
 
     tr_peerIoWriteUint32( msgs->peer->io, out, 2 * sizeof( uint8_t ) + len );
     tr_peerIoWriteUint8 ( msgs->peer->io, out, BT_LTEP );
@@ -1428,7 +1449,9 @@ readBtMessage( tr_peermsgs * msgs, struct evbuffer * inbuf, size_t inlen )
 
         case BT_PORT:
             dbgmsg( msgs, "Got a BT_PORT" );
-            tr_peerIoReadUint16( msgs->peer->io, inbuf, &msgs->peer->port );
+            tr_peerIoReadUint16( msgs->peer->io, inbuf, &msgs->peer->dht_port );
+            if( msgs->peer->dht_port > 0 )
+                tr_dhtAddNode( msgs->session, &msgs->peer->addr, msgs->peer->dht_port, 0 );
             break;
 
         case BT_FEXT_SUGGEST:
@@ -1702,27 +1725,43 @@ fillOutputBuffer( tr_peermsgs * msgs, time_t now )
         if( requestIsValid( msgs, &req )
             && tr_cpPieceIsComplete( &msgs->torrent->completion, req.index ) )
         {
+            /* FIXME(libevent2) use evbuffer_reserve_space() + evbuffer_commit_space() */
             int err;
-            static uint8_t buf[MAX_BLOCK_SIZE];
+            const uint32_t msglen = 4 + 1 + 4 + 4 + req.length;
+            struct evbuffer * out;
+            tr_peerIo * io = msgs->peer->io;
 
-            /* send a block */
-            if(( err = tr_ioRead( msgs->torrent, req.index, req.offset, req.length, buf ))) {
-                fireError( msgs, err );
-                bytesWritten = 0;
-                msgs = NULL;
-            } else {
-                tr_peerIo * io = msgs->peer->io;
-                struct evbuffer * out = tr_getBuffer( );
-                dbgmsg( msgs, "sending block %u:%u->%u", req.index, req.offset, req.length );
-                tr_peerIoWriteUint32( io, out, sizeof( uint8_t ) + 2 * sizeof( uint32_t ) + req.length );
-                tr_peerIoWriteUint8 ( io, out, BT_PIECE );
-                tr_peerIoWriteUint32( io, out, req.index );
-                tr_peerIoWriteUint32( io, out, req.offset );
-                tr_peerIoWriteBytes ( io, out, buf, req.length );
+            out = evbuffer_new( );
+            evbuffer_expand( out, msglen );
+
+            tr_peerIoWriteUint32( io, out, sizeof( uint8_t ) + 2 * sizeof( uint32_t ) + req.length );
+            tr_peerIoWriteUint8 ( io, out, BT_PIECE );
+            tr_peerIoWriteUint32( io, out, req.index );
+            tr_peerIoWriteUint32( io, out, req.offset );
+
+            err = tr_ioRead( msgs->torrent, req.index, req.offset, req.length, EVBUFFER_DATA(out)+EVBUFFER_LENGTH(out) );
+            if( err )
+            {
+                if( fext )
+                    protocolSendReject( msgs, &req );
+            }
+            else
+            {
+                dbgmsg( msgs, "sending block %u:%u->%u", req.index, req.offset, req.length ); 
+                EVBUFFER_LENGTH(out) += req.length;
+                assert( EVBUFFER_LENGTH( out ) == msglen );
                 tr_peerIoWriteBuf( io, out, TRUE );
                 bytesWritten += EVBUFFER_LENGTH( out );
                 msgs->clientSentAnythingAt = now;
-                tr_releaseBuffer( out );
+            }
+
+            evbuffer_free( out );
+
+            if( err )
+            {
+                fireError( msgs, err );
+                bytesWritten = 0;
+                msgs = NULL;
             }
         }
         else if( fext ) /* peer needs a reject message */
@@ -1827,6 +1866,7 @@ sendBitfield( tr_peermsgs * msgs )
     tr_peerIoWriteUint32( msgs->peer->io, out,
                           sizeof( uint8_t ) + field->byteCount );
     tr_peerIoWriteUint8 ( msgs->peer->io, out, BT_BITFIELD );
+    /* FIXME(libevent2): use evbuffer_add_reference() */
     tr_peerIoWriteBytes ( msgs->peer->io, out, field->bits, field->byteCount );
     dbgmsg( msgs, "sending bitfield... outMessage size is now %zu",
             EVBUFFER_LENGTH( out ) );
@@ -2048,7 +2088,7 @@ sendPex( tr_peermsgs * msgs )
             tr_free( tmp );
 
             /* write the pex message */
-            benc = tr_bencSave( &val, &bencLen );
+            benc = tr_bencToStr( &val, TR_FMT_BENC, &bencLen );
             tr_peerIoWriteUint32( io, out, 2 * sizeof( uint8_t ) + bencLen );
             tr_peerIoWriteUint8 ( io, out, BT_LTEP );
             tr_peerIoWriteUint8 ( io, out, msgs->ut_pex_id );
@@ -2069,15 +2109,18 @@ sendPex( tr_peermsgs * msgs )
         tr_free( diffs6.dropped );
         tr_free( newPex6 );
 
-        msgs->clientSentPexAt = time( NULL );
+        /*msgs->clientSentPexAt = time( NULL );*/
     }
 }
 
-static TR_INLINE int
-pexPulse( void * vpeer )
+static void
+pexPulse( int foo UNUSED, short bar UNUSED, void * vmsgs )
 {
-    sendPex( vpeer );
-    return TRUE;
+    struct tr_peermsgs * msgs = vmsgs;
+
+    sendPex( msgs );
+
+    tr_timerAdd( &msgs->pexTimer, PEX_INTERVAL_SECS, 0 );
 }
 
 /**
@@ -2107,7 +2150,6 @@ tr_peerMsgsNew( struct tr_torrent * torrent,
     m->peer->peerIsInterested = 0;
     m->peer->have = tr_bitfieldNew( torrent->info.pieceCount );
     m->state = AWAITING_BT_LENGTH;
-    m->pexTimer = tr_timerNew( m->session, pexPulse, m, PEX_INTERVAL );
     m->outMessages = evbuffer_new( );
     m->outMessagesBatchedAt = 0;
     m->outMessagesBatchPeriod = LOW_PRIORITY_INTERVAL_SECS;
@@ -2115,6 +2157,8 @@ tr_peerMsgsNew( struct tr_torrent * torrent,
     m->peerAskedFor = REQUEST_LIST_INIT;
     m->clientAskedFor = REQUEST_LIST_INIT;
     m->clientWillAskFor = REQUEST_LIST_INIT;
+    evtimer_set( &m->pexTimer, pexPulse, m );
+    tr_timerAdd( &m->pexTimer, PEX_INTERVAL_SECS, 0 );
     peer->msgs = m;
 
     *setme = tr_publisherSubscribe( &m->publisher, func, userData );
@@ -2122,6 +2166,9 @@ tr_peerMsgsNew( struct tr_torrent * torrent,
     if( tr_peerIoSupportsLTEP( peer->io ) )
         sendLtepHandshake( m );
 
+    if(tr_peerIoSupportsDHT(peer->io))
+        protocolSendPort(m, tr_dhtPort(torrent->session));
+
     tellPeerWhatWeHave( m );
 
     tr_peerIoSetIOFuncs( m->peer->io, canRead, didWrite, gotError, m );
@@ -2135,7 +2182,7 @@ tr_peerMsgsFree( tr_peermsgs* msgs )
 {
     if( msgs )
     {
-        tr_timerFree( &msgs->pexTimer );
+        evtimer_del( &msgs->pexTimer );
         tr_publisherDestruct( &msgs->publisher );
         reqListClear( &msgs->clientWillAskFor );
         reqListClear( &msgs->clientAskedFor );
