@@ -1,5 +1,5 @@
 /*
- * This file Copyright (C) 2008 Charles Kerr <charles@rebelbase.com>
+ * This file Copyright (C) 2008-2009 Charles Kerr <charles@transmissionbt.com>
  *
  * This file is licensed by the GPL version 2.  Works owned by the
  * Transmission project are granted a special exemption to clause 2(b)
@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: resume.c 7455 2008-12-22 00:51:14Z charles $
+ * $Id: resume.c 8588 2009-06-02 01:48:48Z charles $
  */
 
 #include <unistd.h> /* unlink */
@@ -15,35 +15,45 @@
 #include <string.h>
 
 #include "transmission.h"
+#include "session.h"
 #include "bencode.h"
 #include "completion.h"
 #include "fastresume.h"
-#include "net.h"
 #include "peer-mgr.h" /* pex */
 #include "platform.h" /* tr_getResumeDir */
 #include "resume.h"
 #include "torrent.h"
 #include "utils.h" /* tr_buildPath */
 
-#define KEY_ACTIVITY_DATE   "activity-date"
-#define KEY_ADDED_DATE      "added-date"
-#define KEY_CORRUPT         "corrupt"
-#define KEY_DONE_DATE       "done-date"
-#define KEY_DOWNLOAD_DIR    "destination"
-#define KEY_DND             "dnd"
-#define KEY_DOWNLOADED      "downloaded"
-#define KEY_MAX_PEERS       "max-peers"
-#define KEY_PAUSED          "paused"
-#define KEY_PEERS           "peers"
-#define KEY_PRIORITY        "priority"
-#define KEY_PROGRESS        "progress"
-#define KEY_SPEEDLIMIT      "speed-limit"
-#define KEY_UPLOADED        "uploaded"
+#define KEY_ACTIVITY_DATE       "activity-date"
+#define KEY_ADDED_DATE          "added-date"
+#define KEY_CORRUPT             "corrupt"
+#define KEY_DONE_DATE           "done-date"
+#define KEY_DOWNLOAD_DIR        "destination"
+#define KEY_DND                 "dnd"
+#define KEY_DOWNLOADED          "downloaded"
+#define KEY_MAX_PEERS           "max-peers"
+#define KEY_PAUSED              "paused"
+#define KEY_PEERS               "peers"
+#define KEY_PEERS6              "peers6"
+#define KEY_FILE_PRIORITIES     "priority"
+#define KEY_BANDWIDTH_PRIORITY  "bandwidth-priority"
+#define KEY_PROGRESS            "progress"
+#define KEY_SPEEDLIMIT_OLD      "speed-limit"
+#define KEY_SPEEDLIMIT_UP       "speed-limit-up"
+#define KEY_SPEEDLIMIT_DOWN     "speed-limit-down"
+#define KEY_RATIOLIMIT          "ratio-limit"
+#define KEY_UPLOADED            "uploaded"
 
-#define KEY_SPEEDLIMIT_DOWN_SPEED "down-speed"
-#define KEY_SPEEDLIMIT_DOWN_MODE  "down-mode"
-#define KEY_SPEEDLIMIT_UP_SPEED   "up-speed"
-#define KEY_SPEEDLIMIT_UP_MODE    "up-mode"
+#define KEY_SPEED                  "speed"
+#define KEY_USE_GLOBAL_SPEED_LIMIT "use-global-speed-limit"
+#define KEY_USE_SPEED_LIMIT        "use-speed-limit"
+#define KEY_SPEEDLIMIT_DOWN_SPEED  "down-speed"
+#define KEY_SPEEDLIMIT_DOWN_MODE   "down-mode"
+#define KEY_SPEEDLIMIT_UP_SPEED    "up-speed"
+#define KEY_SPEEDLIMIT_UP_MODE     "up-mode"
+#define KEY_RATIOLIMIT_RATIO       "ratio-limit"
+#define KEY_RATIOLIMIT_MODE        "ratio-mode"
 
 #define KEY_PROGRESS_MTIMES   "mtimes"
 #define KEY_PROGRESS_BITFIELD "bitfield"
@@ -66,12 +76,19 @@ static void
 savePeers( tr_benc *          dict,
            const tr_torrent * tor )
 {
-    tr_pex *  pex = NULL;
-    const int count = tr_peerMgrGetPeers( tor->session->peerMgr,
-                                          tor->info.hash, &pex );
+    tr_pex * pex = NULL;
+    int count = tr_peerMgrGetPeers( (tr_torrent*) tor, &pex, TR_AF_INET );
 
     if( count > 0 )
         tr_bencDictAddRaw( dict, KEY_PEERS, pex, sizeof( tr_pex ) * count );
+
+    tr_free( pex );
+    pex = NULL;
+    
+    count = tr_peerMgrGetPeers( (tr_torrent*) tor, &pex, TR_AF_INET6 );
+    if( count > 0 )
+        tr_bencDictAddRaw( dict, KEY_PEERS6, pex, sizeof( tr_pex ) * count );
+    
     tr_free( pex );
 }
 
@@ -91,10 +108,23 @@ loadPeers( tr_benc *    dict,
         {
             tr_pex pex;
             memcpy( &pex, str + ( i * sizeof( tr_pex ) ), sizeof( tr_pex ) );
-            tr_peerMgrAddPex( tor->session->peerMgr,
-                              tor->info.hash, TR_PEER_FROM_CACHE, &pex );
+            tr_peerMgrAddPex( tor, TR_PEER_FROM_CACHE, &pex );
         }
-        tr_tordbg( tor, "Loaded %d peers from resume file", count );
+        tr_tordbg( tor, "Loaded %d IPv4 peers from resume file", count );
+        ret = TR_FR_PEERS;
+    }
+    
+    if( tr_bencDictFindRaw( dict, KEY_PEERS6, &str, &len ) )
+    {
+        int       i;
+        const int count = len / sizeof( tr_pex );
+        for( i = 0; i < count; ++i )
+        {
+            tr_pex pex;
+            memcpy( &pex, str + ( i * sizeof( tr_pex ) ), sizeof( tr_pex ) );
+            tr_peerMgrAddPex( tor, TR_PEER_FROM_CACHE, &pex );
+        }
+        tr_tordbg( tor, "Loaded %d IPv6 peers from resume file", count );
         ret = TR_FR_PEERS;
     }
 
@@ -160,7 +190,7 @@ loadDND( tr_benc *    dict,
 
         tr_free( dnd );
         tr_free( dl );
-        ret = TR_FR_PRIORITY;
+        ret = TR_FR_DND;
     }
     else
     {
@@ -178,37 +208,35 @@ loadDND( tr_benc *    dict,
 ***/
 
 static void
-savePriorities( tr_benc *          dict,
-                const tr_torrent * tor )
+saveFilePriorities( tr_benc * dict, const tr_torrent * tor )
 {
     const tr_info *       inf = tr_torrentInfo( tor );
     const tr_file_index_t n = inf->fileCount;
     tr_file_index_t       i;
     tr_benc *             list;
 
-    list = tr_bencDictAddList( dict, KEY_PRIORITY, n );
+    list = tr_bencDictAddList( dict, KEY_FILE_PRIORITIES, n );
     for( i = 0; i < n; ++i )
         tr_bencListAddInt( list, inf->files[i].priority );
 }
 
 static uint64_t
-loadPriorities( tr_benc *    dict,
-                tr_torrent * tor )
+loadFilePriorities( tr_benc * dict, tr_torrent * tor )
 {
     uint64_t              ret = 0;
     tr_info *             inf = &tor->info;
     const tr_file_index_t n = inf->fileCount;
     tr_benc *             list;
 
-    if( tr_bencDictFindList( dict, KEY_PRIORITY, &list )
+    if( tr_bencDictFindList( dict, KEY_FILE_PRIORITIES, &list )
       && ( tr_bencListSize( list ) == n ) )
     {
-        int64_t         tmp;
+        int64_t priority;
         tr_file_index_t i;
         for( i = 0; i < n; ++i )
-            if( tr_bencGetInt( tr_bencListChild( list, i ), &tmp ) )
-                inf->files[i].priority = tmp;
-        ret = TR_FR_PRIORITY;
+            if( tr_bencGetInt( tr_bencListChild( list, i ), &priority ) )
+                tr_torrentInitFilePriority( tor, i, priority );
+        ret = TR_FR_FILE_PRIORITIES;
     }
 
     return ret;
@@ -219,45 +247,111 @@ loadPriorities( tr_benc *    dict,
 ***/
 
 static void
-saveSpeedLimits( tr_benc *          dict,
-                 const tr_torrent * tor )
+saveSingleSpeedLimit( tr_benc * d, const tr_torrent * tor, tr_direction dir )
 {
-    tr_benc * d = tr_bencDictAddDict( dict, KEY_SPEEDLIMIT, 4 );
-
-    tr_bencDictAddInt( d, KEY_SPEEDLIMIT_DOWN_SPEED,
-                      tr_torrentGetSpeedLimit( tor, TR_DOWN ) );
-    tr_bencDictAddInt( d, KEY_SPEEDLIMIT_DOWN_MODE,
-                      tr_torrentGetSpeedMode( tor, TR_DOWN ) );
-    tr_bencDictAddInt( d, KEY_SPEEDLIMIT_UP_SPEED,
-                      tr_torrentGetSpeedLimit( tor, TR_UP ) );
-    tr_bencDictAddInt( d, KEY_SPEEDLIMIT_UP_MODE,
-                      tr_torrentGetSpeedMode( tor, TR_UP ) );
+    tr_bencDictReserve( d, 3 );
+    tr_bencDictAddInt( d, KEY_SPEED, tr_torrentGetSpeedLimit( tor, dir ) );
+    tr_bencDictAddBool( d, KEY_USE_GLOBAL_SPEED_LIMIT, tr_torrentUsesSessionLimits( tor ) );
+    tr_bencDictAddBool( d, KEY_USE_SPEED_LIMIT, tr_torrentUsesSpeedLimit( tor, dir ) );
 }
 
+static void
+saveSpeedLimits( tr_benc * dict, const tr_torrent * tor )
+{
+    saveSingleSpeedLimit( tr_bencDictAddDict( dict, KEY_SPEEDLIMIT_DOWN, 0 ), tor, TR_DOWN );
+    saveSingleSpeedLimit( tr_bencDictAddDict( dict, KEY_SPEEDLIMIT_UP, 0 ), tor, TR_UP );
+}
+
+static void
+saveRatioLimits( tr_benc * dict, const tr_torrent * tor )
+{
+    tr_benc * d = tr_bencDictAddDict( dict, KEY_RATIOLIMIT, 2 );
+    tr_bencDictAddReal( d, KEY_RATIOLIMIT_RATIO, tr_torrentGetRatioLimit( tor ) );
+    tr_bencDictAddInt( d, KEY_RATIOLIMIT_MODE, tr_torrentGetRatioMode( tor ) );
+}
+
+static void
+loadSingleSpeedLimit( tr_benc * d, tr_direction dir, tr_torrent * tor )
+{
+    int64_t i;
+    tr_bool boolVal;
+
+    if( tr_bencDictFindInt( d, KEY_SPEED, &i ) )
+        tr_torrentSetSpeedLimit( tor, dir, i );
+
+    if( tr_bencDictFindBool( d, KEY_USE_SPEED_LIMIT, &boolVal ) )
+        tr_torrentUseSpeedLimit( tor, dir, boolVal );
+
+    if( tr_bencDictFindBool( d, KEY_USE_GLOBAL_SPEED_LIMIT, &boolVal ) )
+        tr_torrentUseSessionLimits( tor, boolVal );
+}
+
+enum old_speed_modes
+{
+    TR_SPEEDLIMIT_GLOBAL,   /* only follow the overall speed limit */
+    TR_SPEEDLIMIT_SINGLE    /* only follow the per-torrent limit */
+};
+
 static uint64_t
-loadSpeedLimits( tr_benc *    dict,
-                 tr_torrent * tor )
+loadSpeedLimits( tr_benc * dict, tr_torrent * tor )
 {
     uint64_t  ret = 0;
     tr_benc * d;
 
-    if( tr_bencDictFindDict( dict, KEY_SPEEDLIMIT, &d ) )
+    if( tr_bencDictFindDict( dict, KEY_SPEEDLIMIT_UP, &d ) )
     {
+        loadSingleSpeedLimit( d, TR_UP, tor );
+        ret = TR_FR_SPEEDLIMIT;
+    }
+    if( tr_bencDictFindDict( dict, KEY_SPEEDLIMIT_DOWN, &d ) )
+    {
+        loadSingleSpeedLimit( d, TR_DOWN, tor );
+        ret = TR_FR_SPEEDLIMIT;
+    }
+
+    /* older speedlimit structure */
+    if( !ret && tr_bencDictFindDict( dict, KEY_SPEEDLIMIT_OLD, &d ) )
+    {
+
         int64_t i;
         if( tr_bencDictFindInt( d, KEY_SPEEDLIMIT_DOWN_SPEED, &i ) )
             tr_torrentSetSpeedLimit( tor, TR_DOWN, i );
-        if( tr_bencDictFindInt( d, KEY_SPEEDLIMIT_DOWN_MODE, &i ) )
-            tr_torrentSetSpeedMode( tor, TR_DOWN, i );
+        if( tr_bencDictFindInt( d, KEY_SPEEDLIMIT_DOWN_MODE, &i ) ) {
+            tr_torrentUseSpeedLimit( tor, TR_DOWN, i==TR_SPEEDLIMIT_SINGLE );
+            tr_torrentUseSessionLimits( tor, i==TR_SPEEDLIMIT_GLOBAL );
+         }
         if( tr_bencDictFindInt( d, KEY_SPEEDLIMIT_UP_SPEED, &i ) )
             tr_torrentSetSpeedLimit( tor, TR_UP, i );
-        if( tr_bencDictFindInt( d, KEY_SPEEDLIMIT_UP_MODE, &i ) )
-            tr_torrentSetSpeedMode( tor, TR_UP, i );
+        if( tr_bencDictFindInt( d, KEY_SPEEDLIMIT_UP_MODE, &i ) ) {
+            tr_torrentUseSpeedLimit( tor, TR_UP, i==TR_SPEEDLIMIT_SINGLE );
+            tr_torrentUseSessionLimits( tor, i==TR_SPEEDLIMIT_GLOBAL );
+        }
         ret = TR_FR_SPEEDLIMIT;
     }
 
     return ret;
 }
 
+static uint64_t
+loadRatioLimits( tr_benc *    dict,
+                 tr_torrent * tor )
+{
+    uint64_t  ret = 0;
+    tr_benc * d;
+
+    if( tr_bencDictFindDict( dict, KEY_RATIOLIMIT, &d ) )
+    {
+        int64_t i;
+        double dratio;
+        if( tr_bencDictFindReal( d, KEY_RATIOLIMIT_RATIO, &dratio ) )
+            tr_torrentSetRatioLimit( tor, dratio );
+        if( tr_bencDictFindInt( d, KEY_RATIOLIMIT_MODE, &i ) )
+            tr_torrentSetRatioMode( tor, i );
+      ret = TR_FR_RATIOLIMIT;
+    }
+
+    return ret;
+}
 /***
 ****
 ***/
@@ -286,7 +380,7 @@ saveProgress( tr_benc *          dict,
     }
 
     /* add the bitfield */
-    bitfield = tr_cpBlockBitfield( tor->completion );
+    bitfield = tr_cpBlockBitfield( &tor->completion );
     tr_bencDictAddRaw( p, KEY_PROGRESS_BITFIELD,
                        bitfield->bits, bitfield->byteCount );
 
@@ -354,7 +448,7 @@ loadProgress( tr_benc *    dict,
             tmp.byteCount = rawlen;
             tmp.bitCount = tmp.byteCount * 8;
             tmp.bits = (uint8_t*) raw;
-            if( !tr_cpBlockBitfieldSet( tor->completion, &tmp ) )
+            if( !tr_cpBlockBitfieldSet( &tor->completion, &tmp ) )
             {
                 tr_torrentUncheck( tor );
                 tr_tordbg(
@@ -407,16 +501,18 @@ tr_torrentSaveResume( const tr_torrent * tor )
                        tor->uploadedPrev + tor->uploadedCur );
     tr_bencDictAddInt( &top, KEY_MAX_PEERS,
                        tor->maxConnectedPeers );
-    tr_bencDictAddInt( &top, KEY_PAUSED,
-                       tor->isRunning ? 0 : 1 );
+    tr_bencDictAddInt( &top, KEY_BANDWIDTH_PRIORITY,
+                       tr_torrentGetPriority( tor ) );
+    tr_bencDictAddBool( &top, KEY_PAUSED, !tor->isRunning );
     savePeers( &top, tor );
-    savePriorities( &top, tor );
+    saveFilePriorities( &top, tor );
     saveDND( &top, tor );
     saveProgress( &top, tor );
     saveSpeedLimits( &top, tor );
+    saveRatioLimits( &top, tor );
 
     filename = getResumeFilename( tor );
-    tr_bencSaveFile( filename, &top );
+    tr_bencToFile( &top, TR_FMT_BENC, filename );
     tr_free( filename );
 
     tr_bencFree( &top );
@@ -431,10 +527,11 @@ loadFromFile( tr_torrent * tor,
     uint64_t     fieldsLoaded = 0;
     char *       filename;
     tr_benc      top;
+    tr_bool      boolVal;
 
     filename = getResumeFilename( tor );
 
-    if( tr_bencLoadFile( filename, &top ) )
+    if( tr_bencLoadFile( &top, TR_FMT_BENC, filename ) )
     {
         tr_tordbg( tor, "Couldn't read \"%s\"; trying old format.",
                    filename );
@@ -461,7 +558,8 @@ loadFromFile( tr_torrent * tor,
     }
 
     if( ( fieldsToLoad & ( TR_FR_PROGRESS | TR_FR_DOWNLOAD_DIR ) )
-      && tr_bencDictFindStr( &top, KEY_DOWNLOAD_DIR, &str ) )
+      && ( tr_bencDictFindStr( &top, KEY_DOWNLOAD_DIR, &str ) )
+      && ( str && *str ) )
     {
         tr_free( tor->downloadDir );
         tor->downloadDir = tr_strdup( str );
@@ -490,9 +588,9 @@ loadFromFile( tr_torrent * tor,
     }
 
     if( ( fieldsToLoad & TR_FR_RUN )
-      && tr_bencDictFindInt( &top, KEY_PAUSED, &i ) )
+      && tr_bencDictFindBool( &top, KEY_PAUSED, &boolVal ) )
     {
-        tor->isRunning = i ? 0 : 1;
+        tor->isRunning = !boolVal;
         fieldsLoaded |= TR_FR_RUN;
     }
 
@@ -513,15 +611,23 @@ loadFromFile( tr_torrent * tor,
     if( ( fieldsToLoad & TR_FR_ACTIVITY_DATE )
       && tr_bencDictFindInt( &top, KEY_ACTIVITY_DATE, &i ) )
     {
-        tor->activityDate = i;
+        tr_torrentSetActivityDate( tor, i );
         fieldsLoaded |= TR_FR_ACTIVITY_DATE;
     }
 
+    if( ( fieldsToLoad & TR_FR_BANDWIDTH_PRIORITY )
+      && tr_bencDictFindInt( &top, KEY_BANDWIDTH_PRIORITY, &i )
+      && tr_isPriority( i ) )
+    {
+        tr_torrentSetPriority( tor, i );
+        fieldsLoaded |= TR_FR_BANDWIDTH_PRIORITY;
+    }
+
     if( fieldsToLoad & TR_FR_PEERS )
         fieldsLoaded |= loadPeers( &top, tor );
 
-    if( fieldsToLoad & TR_FR_PRIORITY )
-        fieldsLoaded |= loadPriorities( &top, tor );
+    if( fieldsToLoad & TR_FR_FILE_PRIORITIES )
+        fieldsLoaded |= loadFilePriorities( &top, tor );
 
     if( fieldsToLoad & TR_FR_PROGRESS )
         fieldsLoaded |= loadProgress( &top, tor );
@@ -531,6 +637,9 @@ loadFromFile( tr_torrent * tor,
 
     if( fieldsToLoad & TR_FR_SPEEDLIMIT )
         fieldsLoaded |= loadSpeedLimits( &top, tor );
+    
+    if( fieldsToLoad & TR_FR_RATIOLIMIT )
+        fieldsLoaded |= loadRatioLimits( &top, tor );
 
     tr_bencFree( &top );
     tr_free( filename );
@@ -561,12 +670,12 @@ setFromCtor( tr_torrent *    tor,
 
     if( fields & TR_FR_DOWNLOAD_DIR )
     {
-        const char * downloadDir;
-        if( !tr_ctorGetDownloadDir( ctor, mode, &downloadDir ) )
+        const char * path;
+        if( !tr_ctorGetDownloadDir( ctor, mode, &path ) && path && *path )
         {
             ret |= TR_FR_DOWNLOAD_DIR;
             tr_free( tor->downloadDir );
-            tor->downloadDir = tr_strdup( downloadDir );
+            tor->downloadDir = tr_strdup( path );
         }
     }
 
