@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: rpc-server.c 8204 2009-04-10 17:34:25Z charles $
+ * $Id: rpc-server.c 8685 2009-06-14 01:00:36Z charles $
  */
 
 #include <assert.h>
@@ -24,18 +24,28 @@
  #include <zlib.h>
 #endif
 
-#include <libevent/event.h>
-#include <libevent/evhttp.h>
+#include <event.h>
+#include <evhttp.h>
 
 #include "transmission.h"
 #include "bencode.h"
+#include "crypto.h"
 #include "list.h"
 #include "platform.h"
+#include "ptrarray.h"
 #include "rpcimpl.h"
 #include "rpc-server.h"
 #include "trevent.h"
 #include "utils.h"
 #include "web.h"
+#include "net.h"
+
+/* session-id is used to make cross-site request forgery attacks difficult.
+ * Don't disable this feature unless you really know what you're doing!
+ * http://en.wikipedia.org/wiki/Cross-site_request_forgery
+ * http://shiflett.org/articles/cross-site-request-forgeries
+ * http://www.webappsec.org/lists/websecurity/archive/2008-04/msg00037.html */
+#define REQUIRE_SESSION_ID
 
 #define MY_NAME "RPC Server"
 #define MY_REALM "Transmission"
@@ -51,6 +61,7 @@ struct tr_rpc_server
     tr_bool            isPasswordEnabled;
     tr_bool            isWhitelistEnabled;
     tr_port            port;
+    struct in_addr     bindAddress;
     struct evhttp *    httpd;
     tr_session *       session;
     char *             username;
@@ -58,6 +69,9 @@ struct tr_rpc_server
     char *             whitelistStr;
     tr_list *          whitelist;
 
+    char *             sessionId;
+    time_t             sessionIdExpiresAt;
+
 #ifdef HAVE_ZLIB
     z_stream           stream;
 #endif
@@ -70,6 +84,36 @@ struct tr_rpc_server
     } while( 0 )
 
 
+/***
+****
+***/
+
+static char*
+get_current_session_id( struct tr_rpc_server * server )
+{
+    const time_t now = time( NULL );
+
+    if( !server->sessionId || ( now >= server->sessionIdExpiresAt ) )
+    {
+        int i;
+        const int n = 48;
+        const char * pool = "0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ";
+        const size_t pool_size = strlen( pool );
+        char * buf = tr_new( char, n+1 );
+
+        for( i=0; i<n; ++i )
+            buf[i] = pool[ tr_cryptoRandInt( pool_size ) ];
+        buf[n] = '\0';
+
+        tr_free( server->sessionId );
+        server->sessionId = buf;
+        server->sessionIdExpiresAt = now + (60*60); /* expire in an hour */
+    }
+
+    return server->sessionId;
+}
+
+
 /**
 ***
 **/
@@ -80,32 +124,75 @@ send_simple_response( struct evhttp_request * req,
                       const char *            text )
 {
     const char *      code_text = tr_webGetResponseStr( code );
-    struct evbuffer * body = tr_getBuffer( );
+    struct evbuffer * body = evbuffer_new( );
 
     evbuffer_add_printf( body, "<h1>%d: %s</h1>", code, code_text );
     if( text )
         evbuffer_add_printf( body, "%s", text );
     evhttp_send_reply( req, code, code_text, body );
 
-    tr_releaseBuffer( body );
+    evbuffer_free( body );
 }
 
-static const char*
-tr_memmem( const char * s1,
-           size_t       l1,
-           const char * s2,
-           size_t       l2 )
+struct tr_mimepart
 {
-    if( !l2 ) return s1;
-    while( l1 >= l2 )
+    char * headers;
+    int headers_len;
+    char * body;
+    int body_len;
+};
+
+static void
+tr_mimepart_free( struct tr_mimepart * p )
+{
+    tr_free( p->body );
+    tr_free( p->headers );
+    tr_free( p );
+}
+
+static void
+extract_parts_from_multipart( const struct evkeyvalq * headers,
+                              const struct evbuffer * body,
+                              tr_ptrArray * setme_parts )
+{
+    const char * content_type = evhttp_find_header( headers, "Content-Type" );
+    const char * in = (const char*) EVBUFFER_DATA( body );
+    size_t inlen = EVBUFFER_LENGTH( body );
+
+    const char * boundary_key = "boundary=";
+    const char * boundary_key_begin = strstr( content_type, boundary_key );
+    const char * boundary_val = boundary_key_begin ? boundary_key_begin + strlen( boundary_key ) : "arglebargle";
+    char * boundary = tr_strdup_printf( "--%s", boundary_val );
+    const size_t boundary_len = strlen( boundary );
+
+    const char * delim = tr_memmem( in, inlen, boundary, boundary_len );
+    while( delim )
     {
-        l1--;
-        if( !memcmp( s1, s2, l2 ) )
-            return s1;
-        s1++;
+        size_t part_len;
+        const char * part = delim + boundary_len;
+
+        inlen -= ( part - in );
+        in = part;
+
+        delim = tr_memmem( in, inlen, boundary, boundary_len );
+        part_len = delim ? (size_t)( delim - part ) : inlen;
+
+        if( part_len )
+        {
+            const char * rnrn = tr_memmem( part, part_len, "\r\n\r\n", 4 );
+            if( rnrn )
+            {
+                struct tr_mimepart * p = tr_new( struct tr_mimepart, 1 );
+                p->headers_len = rnrn - part;
+                p->headers = tr_strndup( part, p->headers_len );
+                p->body_len = (part+part_len) - (rnrn + 4);
+                p->body = tr_strndup( rnrn+4, p->body_len );
+                tr_ptrArrayAppend( setme_parts, p );
+            }
+        }
     }
 
-    return NULL;
+    tr_free( boundary );
 }
 
 static void
@@ -118,76 +205,67 @@ handle_upload( struct evhttp_request * req,
     }
     else
     {
-        const char * content_type = evhttp_find_header( req->input_headers,
-                                                        "Content-Type" );
+        int i;
+        int n;
+        tr_bool hasSessionId = FALSE;
+        tr_ptrArray parts = TR_PTR_ARRAY_INIT;
 
         const char * query = strchr( req->uri, '?' );
-        const int    paused = query && strstr( query + 1, "paused=true" );
+        const tr_bool paused = query && strstr( query + 1, "paused=true" );
 
-        const char * in = (const char *) EVBUFFER_DATA( req->input_buffer );
-        size_t       inlen = EVBUFFER_LENGTH( req->input_buffer );
+        extract_parts_from_multipart( req->input_headers, req->input_buffer, &parts );
+        n = tr_ptrArraySize( &parts );
 
-        const char * boundary_key = "boundary=";
-        const char * boundary_key_begin = strstr( content_type,
-                                                  boundary_key );
-        const char * boundary_val =
-            boundary_key_begin ? boundary_key_begin +
-            strlen( boundary_key ) : "arglebargle";
+        /* first look for the session id */
+        for( i=0; i<n; ++i ) {
+            struct tr_mimepart * p = tr_ptrArrayNth( &parts, i );
+            if( tr_memmem( p->headers, p->headers_len, TR_RPC_SESSION_ID_HEADER, strlen( TR_RPC_SESSION_ID_HEADER ) ) )
+                break;
+        }
+        if( i<n ) {
+            const struct tr_mimepart * p = tr_ptrArrayNth( &parts, i );
+            const char * ours = get_current_session_id( server );
+            const int ourlen = strlen( ours );
+            hasSessionId = ourlen<=p->body_len && !memcmp( p->body, ours, ourlen );
+        }
 
-        char *       boundary = tr_strdup_printf( "--%s", boundary_val );
-        const size_t boundary_len = strlen( boundary );
-
-        const char * delim = tr_memmem( in, inlen, boundary, boundary_len );
-        while( delim )
+        if( !hasSessionId )
         {
-            size_t       part_len;
-            const char * part = delim + boundary_len;
-            inlen -= ( part - in );
-            in = part;
-            delim = tr_memmem( in, inlen, boundary, boundary_len );
-            part_len = delim ? (size_t)( delim - part ) : inlen;
-
-            if( part_len )
+            send_simple_response( req, 409, NULL );
+        }
+        else for( i=0; i<n; ++i )
+        {
+            struct tr_mimepart * p = tr_ptrArrayNth( &parts, i );
+            if( strstr( p->headers, "filename=\"" ) )
             {
-                char * text = tr_strndup( part, part_len );
-                if( strstr( text, "filename=\"" ) )
-                {
-                    const char * body = strstr( text, "\r\n\r\n" );
-                    if( body )
-                    {
-                        char * b64;
-                        size_t  body_len;
-                        tr_benc top, *args;
-                        struct evbuffer * json = tr_getBuffer( );
+                char * b64;
+                int body_len = p->body_len;
+                tr_benc top, *args;
+                const char * body = p->body;
+                struct evbuffer * json = evbuffer_new( );
 
-                        body += 4; /* walk past the \r\n\r\n */
-                        body_len = part_len - ( body - text );
-                        if( body_len >= 2
-                          && !memcmp( &body[body_len - 2], "\r\n", 2 ) )
-                            body_len -= 2;
+                if( body_len >= 2 && !memcmp( &body[body_len - 2], "\r\n", 2 ) )
+                    body_len -= 2;
 
-                        tr_bencInitDict( &top, 2 );
-                        args = tr_bencDictAddDict( &top, "arguments", 2 );
-                        tr_bencDictAddStr( &top, "method", "torrent-add" );
-                        b64 = tr_base64_encode( body, body_len, NULL );
-                        tr_bencDictAddStr( args, "metainfo", b64 );
-                        tr_bencDictAddInt( args, "paused", paused );
-                        tr_bencSaveAsJSON( &top, json );
-                        tr_rpc_request_exec_json( server->session,
-                                                  EVBUFFER_DATA( json ),
-                                                  EVBUFFER_LENGTH( json ),
-                                                  NULL, NULL );
+                tr_bencInitDict( &top, 2 );
+                args = tr_bencDictAddDict( &top, "arguments", 2 );
+                tr_bencDictAddStr( &top, "method", "torrent-add" );
+                b64 = tr_base64_encode( body, body_len, NULL );
+                tr_bencDictAddStr( args, "metainfo", b64 );
+                tr_bencDictAddBool( args, "paused", paused );
+                tr_bencToBuf( &top, TR_FMT_JSON_LEAN, json );
+                tr_rpc_request_exec_json( server->session,
+                                          EVBUFFER_DATA( json ),
+                                          EVBUFFER_LENGTH( json ),
+                                          NULL, NULL );
 
-                        tr_releaseBuffer( json );
-                        tr_free( b64 );
-                        tr_bencFree( &top );
-                    }
-                }
-                tr_free( text );
+                evbuffer_free( json );
+                tr_free( b64 );
+                tr_bencFree( &top );
             }
         }
 
-        tr_free( boundary );
+        tr_ptrArrayDestruct( &parts, (PtrArrayForeachFunc)tr_mimepart_free );
 
         /* use xml here because json responses to file uploads is trouble.
          * see http://www.malsup.com/jquery/form/#sample7 for details */
@@ -246,6 +324,9 @@ add_response( struct evhttp_request * req,
     {
         int state;
 
+        /* FIXME(libevent2): this won't compile under libevent2.
+           but we can use evbuffer_reserve_space() + evbuffer_commit_space() instead */
+
         server->stream.next_in = (Bytef*) content;
         server->stream.avail_in = content_len;
 
@@ -291,6 +372,17 @@ add_response( struct evhttp_request * req,
 #endif
 }
 
+static void
+add_time_header( struct evkeyvalq * headers, const char * key, time_t value )
+{
+    /* According to RFC 2616 this must follow RFC 1123's date format,
+       so use gmtime instead of localtime... */
+    char buf[128];
+    struct tm tm = *gmtime( &value );
+    strftime( buf, sizeof( buf ), "%a, %d %b %Y %H:%M:%S GMT", &tm );
+    evhttp_add_header( headers, key, buf );
+}
+
 static void
 serve_file( struct evhttp_request * req,
             struct tr_rpc_server *  server,
@@ -313,20 +405,24 @@ serve_file( struct evhttp_request * req,
 
         if( errno )
         {
-            send_simple_response( req, HTTP_NOTFOUND, filename );
+            char * tmp = tr_strdup_printf( "%s (%s)", filename, tr_strerror( errno ) );
+            send_simple_response( req, HTTP_NOTFOUND, tmp );
+            tr_free( tmp );
         }
         else
         {
             struct evbuffer * out;
+            const time_t now = time( NULL );
 
             errno = error;
-            out = tr_getBuffer( );
-            evhttp_add_header( req->output_headers, "Content-Type",
-                               mimetype_guess( filename ) );
+            out = evbuffer_new( );
+            evhttp_add_header( req->output_headers, "Content-Type", mimetype_guess( filename ) );
+            add_time_header( req->output_headers, "Date", now );
+            add_time_header( req->output_headers, "Expires", now+(24*60*60) );
             add_response( req, server, out, content, content_len );
             evhttp_send_reply( req, HTTP_OK, "OK", out );
 
-            tr_releaseBuffer( out );
+            evbuffer_free( out );
             tr_free( content );
         }
     }
@@ -345,7 +441,7 @@ handle_clutch( struct evhttp_request * req,
         send_simple_response( req, HTTP_NOTFOUND,
             "<p>Couldn't find Transmission's web interface files!</p>"
             "<p>Users: to tell Transmission where to look, "
-            "set the TRANSMISSION_WEB_HOME environmental "
+            "set the TRANSMISSION_WEB_HOME environment "
             "variable to the folder where the web interface's "
             "index.html is located.</p>"
             "<p>Package Builders: to set a custom default at compile time, "
@@ -385,6 +481,7 @@ struct rpc_response_data
     struct tr_rpc_server  * server;
 };
 
+/* FIXME(libevent2): make "response" an evbuffer and remove response_len */
 static void
 rpc_response_func( tr_session      * session UNUSED,
                    const char      * response,
@@ -392,14 +489,14 @@ rpc_response_func( tr_session      * session UNUSED,
                    void            * user_data )
 {
     struct rpc_response_data * data = user_data;
-    struct evbuffer * buf = tr_getBuffer( );
+    struct evbuffer * buf = evbuffer_new( );
 
     add_response( data->req, data->server, buf, response, response_len );
     evhttp_add_header( data->req->output_headers,
                            "Content-Type", "application/json; charset=UTF-8" );
     evhttp_send_reply( data->req, HTTP_OK, "OK", buf );
 
-    tr_releaseBuffer( buf );
+    evbuffer_free( buf );
     tr_free( data );
 }
 
@@ -445,22 +542,29 @@ isAddressAllowed( const tr_rpc_server * server,
     return FALSE;
 }
 
+static tr_bool
+test_session_id( struct tr_rpc_server * server, struct evhttp_request * req )
+{
+    const char * ours = get_current_session_id( server );
+    const char * theirs = evhttp_find_header( req->input_headers, TR_RPC_SESSION_ID_HEADER );
+    const tr_bool success =  theirs && !strcmp( theirs, ours );
+    return success;
+}
+
 static void
-handle_request( struct evhttp_request * req,
-                void *                  arg )
+handle_request( struct evhttp_request * req, void * arg )
 {
     struct tr_rpc_server * server = arg;
 
     if( req && req->evcon )
     {
         const char * auth;
-        char *       user = NULL;
-        char *       pass = NULL;
+        char * user = NULL;
+        char * pass = NULL;
 
         evhttp_add_header( req->output_headers, "Server", MY_REALM );
 
         auth = evhttp_find_header( req->input_headers, "Authorization" );
-
         if( auth && !strncasecmp( auth, "basic ", 6 ) )
         {
             int    plen;
@@ -474,7 +578,7 @@ handle_request( struct evhttp_request * req,
 
         if( !isAddressAllowed( server, req->remote_host ) )
         {
-            send_simple_response( req, 401,
+            send_simple_response( req, 403,
                 "<p>Unauthorized IP Address.</p>"
                 "<p>Either disable the IP address whitelist or add your address to it.</p>"
                 "<p>If you're editing settings.json, see the 'rpc-whitelist' and 'rpc-whitelist-enabled' entries.</p>"
@@ -482,7 +586,8 @@ handle_request( struct evhttp_request * req,
         }
         else if( server->isPasswordEnabled
                  && ( !pass || !user || strcmp( server->username, user )
-                                     || strcmp( server->password, pass ) ) )
+                                     || !tr_ssha1_matches( server->password,
+                                                           pass ) ) )
         {
             evhttp_add_header( req->output_headers,
                                "WWW-Authenticate",
@@ -505,14 +610,36 @@ handle_request( struct evhttp_request * req,
         {
             handle_clutch( req, server );
         }
-        else if( !strncmp( req->uri, "/transmission/rpc", 17 ) )
-        {
-            handle_rpc( req, server );
-        }
         else if( !strncmp( req->uri, "/transmission/upload", 20 ) )
         {
             handle_upload( req, server );
         }
+#ifdef REQUIRE_SESSION_ID
+        else if( !test_session_id( server, req ) )
+        {
+            const char * sessionId = get_current_session_id( server );
+            char * tmp = tr_strdup_printf(
+                "<p>Please add this header to your HTTP requests:</p>"
+                "<p style=\"padding-left: 20pt;\"><code>%s: %s</code></p>"
+                "<p><b>RPC Application Developers:</b></p>"
+                "<p style=\"padding-left: 20pt;\">As of Transmission 1.53 and 1.61, RPC clients "
+                "need to look for this 409 response containing the phrase \"invalid session-id\".  "
+                "It occurs when the request's "TR_RPC_SESSION_ID_HEADER" header was missing "
+                "(such as during bootstrapping) or expired. "
+                "Either way, you can parse this response's headers for the new session-id.</p>"
+                "<p style=\"padding-left: 20pt;\">This requirement has been added to make "
+                "<a href=\"http://en.wikipedia.org/wiki/Cross-site_request_forgery\">CSRF</a>"
+                " attacks more difficult.</p>",
+                TR_RPC_SESSION_ID_HEADER, sessionId );
+            evhttp_add_header( req->output_headers, TR_RPC_SESSION_ID_HEADER, sessionId );
+            send_simple_response( req, 409, tmp );
+            tr_free( tmp );
+        }
+#endif
+        else if( !strncmp( req->uri, "/transmission/rpc", 17 ) )
+        {
+            handle_rpc( req, server );
+        }
         else
         {
             send_simple_response( req, HTTP_NOTFOUND, req->uri );
@@ -526,12 +653,17 @@ static void
 startServer( void * vserver )
 {
     tr_rpc_server * server  = vserver;
+    tr_address addr;
 
     if( !server->httpd )
     {
+        addr.type = TR_AF_INET;
+        addr.addr.addr4 = server->bindAddress;
         server->httpd = evhttp_new( tr_eventGetBase( server->session ) );
-        evhttp_bind_socket( server->httpd, "0.0.0.0", server->port );
+        evhttp_bind_socket( server->httpd, tr_ntop_non_ts( &addr ),
+                            server->port );
         evhttp_set_gencb( server->httpd, handle_request, server );
+
     }
 }
 
@@ -634,10 +766,10 @@ tr_rpcSetWhitelist( tr_rpc_server * server,
     }
 }
 
-char*
+const char*
 tr_rpcGetWhitelist( const tr_rpc_server * server )
 {
-    return tr_strdup( server->whitelistStr ? server->whitelistStr : "" );
+    return server->whitelistStr ? server->whitelistStr : "";
 }
 
 void
@@ -666,10 +798,10 @@ tr_rpcSetUsername( tr_rpc_server * server,
     dbgmsg( "setting our Username to [%s]", server->username );
 }
 
-char*
+const char*
 tr_rpcGetUsername( const tr_rpc_server * server )
 {
-    return tr_strdup( server->username ? server->username : "" );
+    return server->username ? server->username : "";
 }
 
 void
@@ -677,14 +809,17 @@ tr_rpcSetPassword( tr_rpc_server * server,
                    const char *    password )
 {
     tr_free( server->password );
-    server->password = tr_strdup( password );
+    if( *password != '{' )
+        server->password = tr_ssha1( password );
+    else
+        server->password = strdup( password );
     dbgmsg( "setting our Password to [%s]", server->password );
 }
 
-char*
+const char*
 tr_rpcGetPassword( const tr_rpc_server * server )
 {
-    return tr_strdup( server->password ? server->password : "" );
+    return server->password ? server->password : "" ;
 }
 
 void
@@ -701,6 +836,15 @@ tr_rpcIsPasswordEnabled( const tr_rpc_server * server )
     return server->isPasswordEnabled;
 }
 
+const char *
+tr_rpcGetBindAddress( const tr_rpc_server * server )
+{
+    tr_address addr;
+    addr.type = TR_AF_INET;
+    addr.addr.addr4 = server->bindAddress;
+    return tr_ntop_non_ts( &addr );
+}
+
 /****
 *****  LIFE CYCLE
 ****/
@@ -717,6 +861,7 @@ closeServer( void * vserver )
 #ifdef HAVE_ZLIB
     deflateEnd( &s->stream );
 #endif
+    tr_free( s->sessionId );
     tr_free( s->whitelistStr );
     tr_free( s->username );
     tr_free( s->password );
@@ -736,27 +881,29 @@ tr_rpcInit( tr_session  * session,
 {
     tr_rpc_server * s;
     tr_bool found;
+    tr_bool boolVal;
     int64_t i;
     const char *str;
+    tr_address address;
 
     s = tr_new0( tr_rpc_server, 1 );
     s->session = session;
 
-    found = tr_bencDictFindInt( settings, TR_PREFS_KEY_RPC_ENABLED, &i );
+    found = tr_bencDictFindBool( settings, TR_PREFS_KEY_RPC_ENABLED, &boolVal );
     assert( found );
-    s->isEnabled = i != 0;
+    s->isEnabled = boolVal;
 
     found = tr_bencDictFindInt( settings, TR_PREFS_KEY_RPC_PORT, &i );
     assert( found );
     s->port = i;
 
-    found = tr_bencDictFindInt( settings, TR_PREFS_KEY_RPC_WHITELIST_ENABLED, &i );
+    found = tr_bencDictFindBool( settings, TR_PREFS_KEY_RPC_WHITELIST_ENABLED, &boolVal );
     assert( found );
-    s->isWhitelistEnabled = i != 0;
+    s->isWhitelistEnabled = boolVal;
 
-    found = tr_bencDictFindInt( settings, TR_PREFS_KEY_RPC_AUTH_REQUIRED, &i );
+    found = tr_bencDictFindBool( settings, TR_PREFS_KEY_RPC_AUTH_REQUIRED, &boolVal );
     assert( found );
-    s->isPasswordEnabled = i != 0;
+    s->isPasswordEnabled = boolVal;
 
     found = tr_bencDictFindStr( settings, TR_PREFS_KEY_RPC_WHITELIST, &str );
     assert( found );
@@ -768,7 +915,22 @@ tr_rpcInit( tr_session  * session,
 
     found = tr_bencDictFindStr( settings, TR_PREFS_KEY_RPC_PASSWORD, &str );
     assert( found );
-    s->password = tr_strdup( str );
+    if( *str != '{' )
+        s->password = tr_ssha1( str );
+    else
+        s->password = strdup( str );
+
+    found = tr_bencDictFindStr( settings, TR_PREFS_KEY_RPC_BIND_ADDRESS, &str );
+    assert( found );
+    if( tr_pton( str, &address ) == NULL ) {
+        tr_err( _( "%s is not a valid address" ), str );
+        address = tr_inaddr_any;
+    } else if( address.type != TR_AF_INET ) {
+        tr_err( _( "%s is not an IPv4 address. RPC listeners must be IPv4" ),
+                   str );
+        address = tr_inaddr_any;
+    }
+    s->bindAddress = address.addr.addr4;
 
 #ifdef HAVE_ZLIB
     s->stream.zalloc = (alloc_func) Z_NULL;
