@@ -1,29 +1,17 @@
-/******************************************************************************
- * $Id: session.h 7813 2009-01-29 18:18:24Z charles $
+/*
+ * This file Copyright (C) 2008-2009 Charles Kerr <charles@transmissionbt.com>
  *
- * Copyright (c) 2005-2008 Transmission authors and contributors
+ * This file is licensed by the GPL version 2.  Works owned by the
+ * Transmission project are granted a special exemption to clause 2(b)
+ * so that the bulk of its code can remain under the MIT license.
+ * This exemption does not extend to derived works not owned by
+ * the Transmission project.
  *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software"),
- * to deal in the Software without restriction, including without limitation
- * the rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice shall be included in
- * all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
- * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
- * DEALINGS IN THE SOFTWARE.
- *****************************************************************************/
+ * $Id: session.h 8661 2009-06-09 21:59:25Z charles $
+ */
 
 #ifndef __TRANSMISSION__
-#error only libtransmission should #include this header.
+ #error only libtransmission should #include this header.
 #endif
 
 #ifndef TR_INTERNAL_H
@@ -39,6 +27,7 @@
  #endif
 #endif
 
+#include "bencode.h"
 
 typedef enum { TR_NET_OK, TR_NET_ERROR, TR_NET_WAIT } tr_tristate_t;
 
@@ -46,29 +35,41 @@ uint8_t*       tr_peerIdNew( void );
 
 const uint8_t* tr_getPeerId( void );
 
-struct tr_metainfo_lookup
-{
-    char    hashString[2 * SHA_DIGEST_LENGTH + 1];
-    char *  filename;
-};
-
 struct tr_address;
 struct tr_bandwidth;
+struct tr_bindsockets;
 
 struct tr_session
 {
-    tr_bool                      isPortSet;
     tr_bool                      isPortRandom;
     tr_bool                      isPexEnabled;
+    tr_bool                      isDHTEnabled;
     tr_bool                      isBlocklistEnabled;
     tr_bool                      isProxyEnabled;
     tr_bool                      isProxyAuthEnabled;
     tr_bool                      isClosed;
     tr_bool                      isWaiting;
     tr_bool                      useLazyBitfield;
+    tr_bool                      isRatioLimited;
 
-    tr_bool                      isSpeedLimited[2];
+    tr_benc                      removedTorrents;
+    
     int                          speedLimit[2];
+    tr_bool                      speedLimitEnabled[2];
+
+    int                          altSpeed[2];
+    tr_bool                      altSpeedEnabled;
+
+    int                          altSpeedTimeBegin;
+    int                          altSpeedTimeEnd;
+    tr_sched_day                 altSpeedTimeDay;
+    tr_bool                      altSpeedTimeEnabled;
+    tr_bool                      altSpeedChangedByUser;
+
+    tr_altSpeedFunc            * altCallback;
+    void                       * altCallbackUserData;
+
+
     int                          magicNumber;
 
     tr_encryption_mode           encryptionMode;
@@ -115,11 +116,12 @@ struct tr_session
     tr_rpc_func                  rpc_func;
     void *                       rpc_func_user_data;
 
-    struct tr_stats_handle *     sessionStats;
-    struct tr_tracker_handle *   tracker;
+    struct tr_stats_handle     * sessionStats;
+    struct tr_tracker_handle   * tracker;
 
-    struct tr_metainfo_lookup *  metainfoLookup;
-    int                          metainfoLookupCount;
+    tr_benc                    * metainfoLookup;
+
+    struct event               * altTimer;
 
     /* the size of the output buffer for peer connections */
     int so_sndbuf;
@@ -128,9 +130,18 @@ struct tr_session
     int so_rcvbuf;
 
     /* monitors the "global pool" speeds */
-    struct tr_bandwidth       * bandwidth;
+    struct tr_bandwidth        * bandwidth;
+
+    double                       desiredRatio;
+
+    struct tr_bindinfo         * public_ipv4;
+    struct tr_bindinfo         * public_ipv6;
 };
 
+tr_bool      tr_sessionGetActiveSpeedLimit( const tr_session  * session,
+                                            tr_direction        dir,
+                                            int               * setme );
+
 const char * tr_sessionFindTorrentFile( const tr_session * session,
                                         const char *       hashString );
 
@@ -141,21 +152,45 @@ void         tr_sessionSetTorrentFile( tr_session * session,
 tr_bool      tr_sessionIsAddressBlocked( const tr_session        * session,
                                          const struct tr_address * addr );
 
-
 void         tr_globalLock( tr_session * );
 
 void         tr_globalUnlock( tr_session * );
 
 tr_bool      tr_globalIsLocked( const tr_session * );
 
+const struct tr_address*  tr_sessionGetPublicAddress( const tr_session *, int tr_af_type );
+
+struct tr_bindsockets * tr_sessionGetBindSockets( tr_session * );
+
 enum
 {
     SESSION_MAGIC_NUMBER = 3845
 };
 
-static inline tr_bool tr_isSession( const tr_session * session )
+static TR_INLINE tr_bool tr_isSession( const tr_session * session )
 {
     return ( session != NULL ) && ( session->magicNumber == SESSION_MAGIC_NUMBER );
 }
 
+static TR_INLINE tr_bool tr_isPreallocationMode( tr_preallocation_mode m  )
+{
+    return ( m == TR_PREALLOCATE_NONE )
+        || ( m == TR_PREALLOCATE_SPARSE )
+        || ( m == TR_PREALLOCATE_FULL );
+}
+
+static TR_INLINE tr_bool tr_isEncryptionMode( tr_encryption_mode m )
+{
+    return ( m == TR_CLEAR_PREFERRED )
+        || ( m == TR_ENCRYPTION_PREFERRED )
+        || ( m == TR_ENCRYPTION_REQUIRED );
+}
+
+static TR_INLINE tr_bool tr_isPriority( tr_priority_t p )
+{
+    return ( p == TR_PRI_LOW )
+        || ( p == TR_PRI_NORMAL )
+        || ( p == TR_PRI_HIGH );
+}
+
 #endif
