@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: torrent.c 8219 2009-04-12 17:29:35Z charles $
+ * $Id: torrent.c 8709 2009-06-16 19:31:17Z charles $
  */
 
 #include <sys/types.h> /* stat */
@@ -71,7 +71,7 @@ tr_torrentFindFromHashString( tr_session *  session, const char * str )
     tr_torrent * tor = NULL;
 
     while(( tor = tr_torrentNext( session, tor )))
-        if( !strcmp( str, tor->info.hashString ))
+        if( !strcmp( str, tor->info.hashString ) )
             return tor;
 
     return NULL;
@@ -109,72 +109,142 @@ tr_torrentFindFromObfuscatedHash( tr_session * session,
 ***/
 
 void
-tr_torrentSetSpeedMode( tr_torrent *  tor,
-                        tr_direction  dir,
-                        tr_speedlimit mode )
-{
-    assert( tr_isTorrent( tor ) );
-    assert( tr_isDirection( dir ) );
-    assert( mode==TR_SPEEDLIMIT_GLOBAL || mode==TR_SPEEDLIMIT_SINGLE || mode==TR_SPEEDLIMIT_UNLIMITED  );
-
-    tor->speedLimitMode[dir] = mode;
-
-    tr_bandwidthSetLimited( tor->bandwidth, dir, mode==TR_SPEEDLIMIT_SINGLE );
-    tr_bandwidthHonorParentLimits( tor->bandwidth, dir, mode==TR_SPEEDLIMIT_GLOBAL );
-}
-
-tr_speedlimit
-tr_torrentGetSpeedMode( const tr_torrent * tor,
-                        tr_direction       dir )
+tr_torrentSetSpeedLimit( tr_torrent * tor, tr_direction dir, int KiB_sec )
 {
     assert( tr_isTorrent( tor ) );
     assert( tr_isDirection( dir ) );
 
-    return tor->speedLimitMode[dir];
-}
-
-void
-tr_torrentSetSpeedLimit( tr_torrent * tor,
-                         tr_direction dir,
-                         int          desiredSpeed )
-{
-    tr_bandwidthSetDesiredSpeed( tor->bandwidth, dir, desiredSpeed );
+    tr_bandwidthSetDesiredSpeed( tor->bandwidth, dir, KiB_sec );
 }
 
 int
-tr_torrentGetSpeedLimit( const tr_torrent * tor,
-                         tr_direction       dir )
+tr_torrentGetSpeedLimit( const tr_torrent * tor, tr_direction dir )
 {
+    assert( tr_isTorrent( tor ) );
+    assert( tr_isDirection( dir ) );
+
     return tr_bandwidthGetDesiredSpeed( tor->bandwidth, dir );
 }
 
+void
+tr_torrentUseSpeedLimit( tr_torrent * tor, tr_direction dir, tr_bool do_use )
+{
+    assert( tr_isTorrent( tor ) );
+    assert( tr_isDirection( dir ) );
+
+    tr_bandwidthSetLimited( tor->bandwidth, dir, do_use );
+}
+
+tr_bool
+tr_torrentUsesSpeedLimit( const tr_torrent * tor, tr_direction dir )
+{
+    assert( tr_isTorrent( tor ) );
+    assert( tr_isDirection( dir ) );
+
+    return tr_bandwidthIsLimited( tor->bandwidth, dir );
+}
+
+void
+tr_torrentUseSessionLimits( tr_torrent * tor, tr_bool doUse )
+{
+    assert( tr_isTorrent( tor ) );
+
+    tr_bandwidthHonorParentLimits( tor->bandwidth, TR_UP, doUse );
+    tr_bandwidthHonorParentLimits( tor->bandwidth, TR_DOWN, doUse );
+}
+
+tr_bool
+tr_torrentUsesSessionLimits( const tr_torrent * tor )
+{
+    assert( tr_isTorrent( tor ) );
+
+    return tr_bandwidthAreParentLimitsHonored( tor->bandwidth, TR_UP );
+}
+
+/***
+****
+***/
+
+void
+tr_torrentSetRatioMode( tr_torrent *  tor, tr_ratiolimit mode )
+{
+    assert( tr_isTorrent( tor ) );
+    assert( mode==TR_RATIOLIMIT_GLOBAL || mode==TR_RATIOLIMIT_SINGLE || mode==TR_RATIOLIMIT_UNLIMITED  );
+
+    tor->ratioLimitMode = mode;
+    tor->needsSeedRatioCheck = TRUE;
+}
+
+tr_ratiolimit
+tr_torrentGetRatioMode( const tr_torrent * tor )
+{
+    assert( tr_isTorrent( tor ) );
+
+    return tor->ratioLimitMode;
+}
+
+void
+tr_torrentSetRatioLimit( tr_torrent * tor, double desiredRatio )
+{
+    assert( tr_isTorrent( tor ) );
+
+    tor->desiredRatio = desiredRatio;
+
+    tor->needsSeedRatioCheck = TRUE;
+}
+
+double
+tr_torrentGetRatioLimit( const tr_torrent * tor )
+{
+    assert( tr_isTorrent( tor ) );
+
+    return tor->desiredRatio;
+}
+
 tr_bool
 tr_torrentIsPieceTransferAllowed( const tr_torrent  * tor,
                                   tr_direction        direction )
 {
-    tr_bool isEnabled = FALSE;
+    int limit;
+    tr_bool allowed = TRUE;
 
-    switch( tr_torrentGetSpeedMode( tor, direction ) )
+    if( tr_torrentUsesSpeedLimit( tor, direction ) )
+        if( tr_torrentGetSpeedLimit( tor, direction ) <= 0 )
+            allowed = FALSE;
+
+    if( tr_torrentUsesSessionLimits( tor ) )
+        if( tr_sessionGetActiveSpeedLimit( tor->session, direction, &limit ) )
+            if( limit <= 0 )
+                allowed = FALSE;
+
+    return allowed;
+}
+
+tr_bool
+tr_torrentGetSeedRatio( const tr_torrent * tor, double * ratio )
+{
+    tr_bool isLimited;
+
+    switch( tr_torrentGetRatioMode( tor ) )
     {
-        case TR_SPEEDLIMIT_GLOBAL:
-            isEnabled = !tr_sessionIsSpeedLimitEnabled( tor->session, direction )
-                      || tr_sessionGetSpeedLimit( tor->session, direction ) > 0;
+        case TR_RATIOLIMIT_SINGLE:
+            isLimited = TRUE;
+            if( ratio )
+                *ratio = tr_torrentGetRatioLimit( tor );
             break;
 
-        case TR_SPEEDLIMIT_SINGLE:
-            isEnabled = tr_torrentGetSpeedLimit( tor, direction ) > 0;
+        case TR_RATIOLIMIT_GLOBAL:
+            isLimited = tr_sessionIsRatioLimited( tor->session );
+            if( isLimited && ratio )
+                *ratio = tr_sessionGetRatioLimit( tor->session );
             break;
 
-        case TR_SPEEDLIMIT_UNLIMITED:
-            isEnabled = TRUE;
-            break;
-
-        default:
-            assert( 0 && "unhandled speed mode" );
+        default: /* TR_RATIOLIMIT_UNLIMITED */
+            isLimited = FALSE;
             break;
     }
 
-    return isEnabled;
+    return isLimited;
 }
 
 /***
@@ -282,14 +352,11 @@ calculatePiecePriority( const tr_torrent * tor,
     int             priority = TR_PRI_LOW;
 
     /* find the first file that has data in this piece */
-    if( fileHint >= 0 )
-    {
+    if( fileHint >= 0 ) {
         i = fileHint;
         while( i > 0 && pieceHasFile( piece, &tor->info.files[i - 1] ) )
             --i;
-    }
-    else
-    {
+    } else {
         for( i = 0; i < tor->info.fileCount; ++i )
             if( pieceHasFile( piece, &tor->info.files[i] ) )
                 break;
@@ -455,9 +522,7 @@ getBlockSize( uint32_t pieceSize )
 }
 
 static void
-torrentRealInit( tr_session      * session,
-                 tr_torrent      * tor,
-                 const tr_ctor   * ctor )
+torrentRealInit( tr_torrent * tor, const tr_ctor * ctor )
 {
     int          doStart;
     uint64_t     loaded;
@@ -465,6 +530,9 @@ torrentRealInit( tr_session      * session,
     const char * dir;
     static int   nextUniqueId = 1;
     tr_info    * info = &tor->info;
+    tr_session * session = tr_ctorGetSession( ctor );
+
+    assert( session != NULL );
 
     tr_globalLock( session );
 
@@ -528,17 +596,21 @@ torrentRealInit( tr_session      * session,
 
     tr_peerMgrAddTorrent( session->peerMgr, tor );
 
-    assert( session->isPortSet );
     assert( !tor->downloadedCur );
     assert( !tor->uploadedCur );
 
+    tr_ctorInitTorrentPriorities( ctor, tor );
+
+    tr_ctorInitTorrentWanted( ctor, tor );
+
     tor->error   = 0;
 
     tr_bitfieldConstruct( &tor->checkedPieces, tor->info.pieceCount );
     tr_torrentUncheck( tor );
 
-    tor->addedDate = time( NULL ); /* this is a default value to be
-                                      overwritten by the resume file */
+    tr_torrentSetAddedDate( tor, time( NULL ) ); /* this is a default value to be
+                                                    overwritten by the resume file */
+
     loaded = tr_torrentLoadResume( tor, ~0, ctor );
 
     doStart = tor->isRunning;
@@ -546,11 +618,17 @@ torrentRealInit( tr_session      * session,
 
     if( !( loaded & TR_FR_SPEEDLIMIT ) )
     {
-        tr_torrentSetSpeedLimit( tor, TR_UP,
-                                tr_sessionGetSpeedLimit( tor->session, TR_UP ) );
-        tr_torrentSetSpeedLimit( tor, TR_DOWN,
-                                tr_sessionGetSpeedLimit( tor->session,
-                                                         TR_DOWN ) );
+        tr_torrentUseSpeedLimit( tor, TR_UP, FALSE );
+        tr_torrentSetSpeedLimit( tor, TR_UP, tr_sessionGetSpeedLimit( tor->session, TR_UP ) );
+        tr_torrentUseSpeedLimit( tor, TR_DOWN, FALSE );
+        tr_torrentSetSpeedLimit( tor, TR_DOWN, tr_sessionGetSpeedLimit( tor->session, TR_DOWN ) );
+        tr_torrentUseSessionLimits( tor, TRUE );
+    }
+
+    if( !( loaded & TR_FR_RATIOLIMIT ) )
+    {
+        tr_torrentSetRatioMode( tor, TR_RATIOLIMIT_GLOBAL );
+        tr_torrentSetRatioLimit( tor, tr_sessionGetRatioLimit( tor->session ) );
     }
 
     tor->completeness = tr_cpGetStatus( &tor->completion );
@@ -582,9 +660,8 @@ torrentRealInit( tr_session      * session,
         if( !tr_ctorGetMetainfo( ctor, &val ) )
         {
             const char * filename = tor->info.torrent;
-            tr_bencSaveFile( filename, val );
-            tr_sessionSetTorrentFile( tor->session, tor->info.hashString,
-                                      filename );
+            tr_bencToFile( val, TR_FMT_BENC, filename );
+            tr_sessionSetTorrentFile( tor->session, tor->info.hashString, filename );
         }
     }
 
@@ -595,14 +672,14 @@ torrentRealInit( tr_session      * session,
 }
 
 int
-tr_torrentParse( const tr_session  * session,
-                 const tr_ctor     * ctor,
+tr_torrentParse( const tr_ctor     * ctor,
                  tr_info           * setmeInfo )
 {
     int             err = 0;
     int             doFree;
     tr_info         tmp;
     const tr_benc * metainfo;
+    tr_session    * session = tr_ctorGetSession( ctor );
 
     if( setmeInfo == NULL )
         setmeInfo = &tmp;
@@ -617,7 +694,7 @@ tr_torrentParse( const tr_session  * session,
     if( !err && !getBlockSize( setmeInfo->pieceSize ) )
         err = TR_EINVALID;
 
-    if( !err && tr_torrentExists( session, setmeInfo->hash ) )
+    if( !err && session && tr_torrentExists( session, setmeInfo->hash ) )
         err = TR_EDUPLICATE;
 
     if( doFree )
@@ -627,20 +704,22 @@ tr_torrentParse( const tr_session  * session,
 }
 
 tr_torrent *
-tr_torrentNew( tr_session     * session,
-               const tr_ctor  * ctor,
+tr_torrentNew( const tr_ctor  * ctor,
                int            * setmeError )
 {
     int          err;
     tr_info      tmpInfo;
     tr_torrent * tor = NULL;
 
-    err = tr_torrentParse( session, ctor, &tmpInfo );
+    assert( ctor != NULL );
+    assert( tr_isSession( tr_ctorGetSession( ctor ) ) );
+
+    err = tr_torrentParse( ctor, &tmpInfo );
     if( !err )
     {
         tor = tr_new0( tr_torrent, 1 );
         tor->info = tmpInfo;
-        torrentRealInit( session, tor, ctor );
+        torrentRealInit( tor, ctor );
     }
     else if( setmeError )
     {
@@ -655,8 +734,7 @@ tr_torrentNew( tr_session     * session,
 **/
 
 void
-tr_torrentSetDownloadDir( tr_torrent * tor,
-                          const char * path )
+tr_torrentSetDownloadDir( tr_torrent * tor, const char * path )
 {
     assert( tr_isTorrent( tor  ) );
 
@@ -704,7 +782,7 @@ tr_torrentManualUpdate( tr_torrent * tor )
     tr_runInEventThread( tor->session, tr_torrentManualUpdateImpl, tor );
 }
 
-int
+tr_bool
 tr_torrentCanManualUpdate( const tr_torrent * tor )
 {
     return ( tr_isTorrent( tor  ) )
@@ -728,6 +806,16 @@ tr_torrentStatCached( tr_torrent * tor )
          : tr_torrentStat( tor );
 }
 
+void
+tr_torrentSetVerifyState( tr_torrent * tor, tr_verify_state state )
+{
+    assert( tr_isTorrent( tor ) );
+    assert( state==TR_VERIFY_NONE || state==TR_VERIFY_WAIT || state==TR_VERIFY_NOW );
+
+    tor->verifyState = state;
+    tor->anyDate = time( NULL );
+}
+
 tr_torrent_activity
 tr_torrentGetActivity( tr_torrent * tor )
 {
@@ -755,6 +843,9 @@ tr_torrentStat( tr_torrent * tor )
     const tr_tracker_info * ti;
     int                     usableSeeds = 0;
     uint64_t                now;
+    double                  downloadedForRatio, seedRatio=0;
+    double                  d;
+    tr_bool                 checkSeedRatio;
 
     if( !tor )
         return NULL;
@@ -792,11 +883,12 @@ tr_torrentStat( tr_torrent * tor )
                             s->peersFrom );
 
     now = tr_date( );
+    d = tr_peerMgrGetWebseedSpeed( tor, now );
     s->swarmSpeed         = tr_rcRate( &tor->swarmSpeed, now );
     s->rawUploadSpeed     = tr_bandwidthGetRawSpeed  ( tor->bandwidth, now, TR_UP );
-    s->rawDownloadSpeed   = tr_bandwidthGetRawSpeed  ( tor->bandwidth, now, TR_DOWN );
     s->pieceUploadSpeed   = tr_bandwidthGetPieceSpeed( tor->bandwidth, now, TR_UP );
-    s->pieceDownloadSpeed = tr_bandwidthGetPieceSpeed( tor->bandwidth, now, TR_DOWN );
+    s->rawDownloadSpeed   = d + tr_bandwidthGetRawSpeed  ( tor->bandwidth, now, TR_DOWN );
+    s->pieceDownloadSpeed = d + tr_bandwidthGetPieceSpeed( tor->bandwidth, now, TR_DOWN );
 
     usableSeeds += tor->info.webseedCount;
 
@@ -812,7 +904,6 @@ tr_torrentStat( tr_torrent * tor )
                            (double) tor->info.pieceCount )
                        : 0.0;
 
-
     s->activityDate = tor->activityDate;
     s->addedDate    = tor->addedDate;
     s->doneDate     = tor->doneDate;
@@ -824,7 +915,6 @@ tr_torrentStat( tr_torrent * tor )
     s->haveValid       = tr_cpHaveValid( &tor->completion );
     s->haveUnchecked   = tr_cpHaveTotal( &tor->completion ) - s->haveValid;
 
-
     if( usableSeeds > 0 )
     {
         s->desiredAvailable = s->leftUntilDone;
@@ -845,17 +935,45 @@ tr_torrentStat( tr_torrent * tor )
         tr_bitfieldFree( peerPieces );
     }
 
-    if( s->leftUntilDone > s->desiredAvailable )
-        s->eta = TR_ETA_NOT_AVAIL;
-    else if( s->pieceDownloadSpeed < 0.1 )
-        s->eta = TR_ETA_UNKNOWN;
-    else
-        s->eta = s->leftUntilDone / s->pieceDownloadSpeed / 1024.0;
+    downloadedForRatio = s->downloadedEver ? s->downloadedEver : s->haveValid;
+    s->ratio = tr_getRatio( s->uploadedEver, downloadedForRatio );
 
-    s->ratio = tr_getRatio(
-        s->uploadedEver,
-        s->downloadedEver ? s->downloadedEver : s->
-        haveValid );
+    checkSeedRatio = tr_torrentGetSeedRatio( tor, &seedRatio );
+
+    switch( s->activity )
+    {
+        case TR_STATUS_DOWNLOAD:
+            if( s->leftUntilDone > s->desiredAvailable )
+                s->eta = TR_ETA_NOT_AVAIL;
+            else if( s->pieceDownloadSpeed < 0.1 )
+                s->eta = TR_ETA_UNKNOWN;
+            else
+                s->eta = s->leftUntilDone / s->pieceDownloadSpeed / 1024.0;
+            break;
+
+        case TR_STATUS_SEED:
+            if( checkSeedRatio )
+            {
+                if( s->pieceUploadSpeed < 0.1 )
+                    s->eta = TR_ETA_UNKNOWN;
+                else
+                    s->eta = (downloadedForRatio * (seedRatio - s->ratio)) / s->pieceUploadSpeed / 1024.0;
+            }
+            else
+                s->eta = TR_ETA_NOT_AVAIL;
+            break;
+
+        default:
+            s->eta = TR_ETA_NOT_AVAIL;
+            break;
+    }
+
+    if( !checkSeedRatio || s->ratio >= seedRatio || s->ratio == TR_RATIO_INF )
+        s->percentRatio = 1.0;
+    else if( s->ratio == TR_RATIO_NA )
+        s->percentRatio = 0.0;
+    else 
+        s->percentRatio = s->ratio / seedRatio;
 
     tr_torrentUnlock( tor );
 
@@ -867,54 +985,72 @@ tr_torrentStat( tr_torrent * tor )
 ***/
 
 static uint64_t
-fileBytesCompleted( const tr_torrent * tor,
-                    tr_file_index_t    fileIndex )
+fileBytesCompleted( const tr_torrent * tor, tr_file_index_t index )
 {
-    const tr_file *        file     =  &tor->info.files[fileIndex];
-    const tr_block_index_t firstBlock       =  file->offset /
-                                              tor->blockSize;
-    const uint64_t         firstBlockOffset =  file->offset %
-                                              tor->blockSize;
-    const uint64_t         lastOffset       =
-        file->length ? ( file->length - 1 ) : 0;
-    const tr_block_index_t lastBlock        =
-        ( file->offset + lastOffset ) / tor->blockSize;
-    const uint64_t         lastBlockOffset  =
-        ( file->offset + lastOffset ) % tor->blockSize;
-    uint64_t               haveBytes = 0;
+    uint64_t total = 0;
+    const tr_file * f = &tor->info.files[index];
 
-    assert( tr_isTorrent( tor ) );
-    assert( fileIndex < tor->info.fileCount );
-    assert( file->offset + file->length <= tor->info.totalSize );
-    assert( ( firstBlock < tor->blockCount )
-          || ( !file->length && file->offset == tor->info.totalSize ) );
-    assert( ( lastBlock < tor->blockCount )
-          || ( !file->length && file->offset == tor->info.totalSize ) );
-    assert( firstBlock <= lastBlock );
-    assert( tr_torBlockPiece( tor, firstBlock ) == file->firstPiece );
-    assert( tr_torBlockPiece( tor, lastBlock ) == file->lastPiece );
-
-    if( firstBlock == lastBlock )
+    if( f->length )
     {
-        if( tr_cpBlockIsCompleteFast( &tor->completion, firstBlock ) )
-            haveBytes += lastBlockOffset + 1 - firstBlockOffset;
-    }
-    else
-    {
-        tr_block_index_t i;
+        const tr_block_index_t firstBlock = f->offset / tor->blockSize;
+        const uint64_t lastByte = f->offset + f->length - 1;
+        const tr_block_index_t lastBlock = lastByte / tor->blockSize;
 
-        if( tr_cpBlockIsCompleteFast( &tor->completion, firstBlock ) )
-            haveBytes += tor->blockSize - firstBlockOffset;
+        if( firstBlock == lastBlock )
+        {
+            if( tr_cpBlockIsCompleteFast( &tor->completion, firstBlock ) )
+                total = f->length;
+        }
+        else
+        {
+            uint32_t i;
 
-        for( i = firstBlock + 1; i < lastBlock; ++i )
-            if( tr_cpBlockIsCompleteFast( &tor->completion, i ) )
-                haveBytes += tor->blockSize;
+            /* the first block */
+            if( tr_cpBlockIsCompleteFast( &tor->completion, firstBlock ) )
+                total += tor->blockSize - ( f->offset % tor->blockSize );
 
-        if( tr_cpBlockIsCompleteFast( &tor->completion, lastBlock ) )
-            haveBytes += lastBlockOffset + 1;
+            /* the middle blocks */
+            if( f->firstPiece == f->lastPiece )
+            {
+                for( i=firstBlock+1; i<lastBlock; ++i )
+                    if( tr_cpBlockIsCompleteFast( &tor->completion, i ) )
+                        total += tor->blockSize;
+            }
+            else
+            {
+                int64_t b = 0;
+                const tr_block_index_t firstBlockOfLastPiece
+                           = tr_torPieceFirstBlock( tor, f->lastPiece );
+                const tr_block_index_t lastBlockOfFirstPiece
+                           = tr_torPieceFirstBlock( tor, f->firstPiece )
+                             + tr_torPieceCountBlocks( tor, f->firstPiece ) - 1;
+
+                /* the rest of the first piece */
+                for( i=firstBlock+1; i<lastBlock && i<=lastBlockOfFirstPiece; ++i )
+                    if( tr_cpBlockIsCompleteFast( &tor->completion, i ) )
+                        ++b;
+
+                /* the middle pieces */
+                if( f->firstPiece + 1 < f->lastPiece )
+                    for( i=f->firstPiece+1; i<f->lastPiece; ++i )
+                        b += tor->blockCountInPiece - tr_cpMissingBlocksInPiece( &tor->completion, i );
+
+                /* the rest of the last piece */
+                for( i=firstBlockOfLastPiece; i<lastBlock; ++i )
+                    if( tr_cpBlockIsCompleteFast( &tor->completion, i ) )
+                        ++b;
+
+                b *= tor->blockSize;
+                total += b;
+            }
+
+            /* the last block */
+            if( tr_cpBlockIsCompleteFast( &tor->completion, lastBlock ) )
+                total += ( f->offset + f->length ) - ( tor->blockSize * lastBlock );
+        }
     }
 
-    return haveBytes;
+    return total;
 }
 
 tr_file_stat *
@@ -1089,19 +1225,23 @@ freeTorrent( tr_torrent * tor )
 static void
 checkAndStartImpl( void * vtor )
 {
+    time_t now;
     tr_torrent * tor = vtor;
 
     assert( tr_isTorrent( tor ) );
 
     tr_globalLock( tor->session );
 
+    now = time( NULL );
     tor->isRunning = TRUE;
+    tor->needsSeedRatioCheck = TRUE;
     *tor->errorString = '\0';
     tr_torrentResetTransferStats( tor );
     tor->completeness = tr_cpGetStatus( &tor->completion );
     tr_torrentSaveResume( tor );
-    tor->startDate = time( NULL );
+    tor->startDate = tor->anyDate = now;
     tr_trackerStart( tor->tracker );
+    tor->dhtAnnounceAt = now + tr_cryptoWeakRandInt( 20 );
     tr_peerMgrStartTorrent( tor );
 
     tr_globalUnlock( tor->session );
@@ -1117,8 +1257,7 @@ checkAndStartCB( tr_torrent * tor )
 }
 
 static void
-torrentStart( tr_torrent * tor,
-              int          reloadProgress )
+torrentStart( tr_torrent * tor, int reloadProgress )
 {
     assert( tr_isTorrent( tor ) );
 
@@ -1126,15 +1265,13 @@ torrentStart( tr_torrent * tor,
 
     if( !tor->isRunning )
     {
-        const int isVerifying = tr_verifyInProgress( tor );
+        tr_verifyRemove( tor );
 
-        if( !isVerifying && reloadProgress )
+        if( reloadProgress )
             tr_torrentLoadResume( tor, TR_FR_PROGRESS, NULL );
 
         tor->isRunning = 1;
-
-        if( !isVerifying )
-            tr_verifyAdd( tor, checkAndStartCB );
+        tr_verifyAdd( tor, checkAndStartCB );
     }
 
     tr_globalUnlock( tor->session );
@@ -1154,6 +1291,12 @@ torrentRecheckDoneImpl( void * vtor )
 
     assert( tr_isTorrent( tor ) );
     tr_torrentRecheckCompleteness( tor );
+
+    if( tor->startAfterVerify )
+    {
+        tor->startAfterVerify = FALSE;
+        tr_torrentStart( tor );
+    }
 }
 
 static void
@@ -1168,11 +1311,18 @@ void
 tr_torrentVerify( tr_torrent * tor )
 {
     assert( tr_isTorrent( tor ) );
-
-    tr_verifyRemove( tor );
-
     tr_globalLock( tor->session );
 
+    /* if the torrent's already being verified, stop it */
+    tr_verifyRemove( tor );
+
+    /* if the torrent's running, stop it & set the restart-after-verify flag */
+    if( tor->isRunning ) {
+        tr_torrentStop( tor );
+        tor->startAfterVerify = TRUE;
+    }
+
+    /* add the torrent to the recheck queue */
     tr_torrentUncheck( tor );
     tr_verifyAdd( tor, torrentRecheckDoneCB );
 
@@ -1187,6 +1337,11 @@ tr_torrentCloseLocalFiles( const tr_torrent * tor )
 
     assert( tr_isTorrent( tor ) );
 
+    /* FIXME(libevent2) we're just using the evbuffer to build a key here anyway.
+       so we do (tor->info.fileCount * fd.openFileLimit) strcmps for these keys. :/
+       it would be more efficient to remove this code altogether and
+       add "int torrentId;" to "struct tr_openfile", and a new function
+       tr_fdCloseTorrentFiles( tr_session*, int torrentId ) */
     for( i=0; i<tor->info.fileCount; ++i )
     {
         const tr_file * file = &tor->info.files[i];
@@ -1198,7 +1353,6 @@ tr_torrentCloseLocalFiles( const tr_torrent * tor )
     evbuffer_free( buf );
 }
 
-
 static void
 stopTorrent( void * vtor )
 {
@@ -1216,6 +1370,8 @@ stopTorrent( void * vtor )
 void
 tr_torrentStop( tr_torrent * tor )
 {
+    assert( tr_isTorrent( tor ) );
+
     if( tr_isTorrent( tor ) )
     {
         tr_globalLock( tor->session );
@@ -1232,10 +1388,15 @@ tr_torrentStop( tr_torrent * tor )
 static void
 closeTorrent( void * vtor )
 {
+    tr_benc * d;
     tr_torrent * tor = vtor;
 
     assert( tr_isTorrent( tor ) );
 
+    d = tr_bencListAddDict( &tor->session->removedTorrents, 2 );
+    tr_bencDictAddInt( d, "id", tor->uniqueId );
+    tr_bencDictAddInt( d, "date", time( NULL ) );
+
     tr_torrentSaveResume( tor );
     tor->isRunning = 0;
     stopTorrent( tor );
@@ -1321,12 +1482,29 @@ tr_torrentSetCompletenessCallback( tr_torrent                    * tor,
     tor->completeness_func_user_data = user_data;
 }
 
+void
+tr_torrentSetRatioLimitHitCallback( tr_torrent                     * tor,
+                                    tr_torrent_ratio_limit_hit_func  func,
+                                    void                           * user_data )
+{
+    assert( tr_isTorrent( tor ) );
+
+    tor->ratio_limit_hit_func = func;
+    tor->ratio_limit_hit_func_user_data = user_data;
+}
+
 void
 tr_torrentClearCompletenessCallback( tr_torrent * torrent )
 {
     tr_torrentSetCompletenessCallback( torrent, NULL, NULL );
 }
 
+void
+tr_torrentClearRatioLimitHitCallback( tr_torrent * torrent )
+{
+    tr_torrentSetRatioLimitHitCallback( torrent, NULL, NULL );
+}
+
 void
 tr_torrentRecheckCompleteness( tr_torrent * tor )
 {
@@ -1350,6 +1528,7 @@ tr_torrentRecheckCompleteness( tr_torrent * tor )
         }
 
         tor->completeness = completeness;
+        tor->needsSeedRatioCheck = TRUE;
         tr_torrentCloseLocalFiles( tor );
         fireCompletenessChange( tor, completeness );
 
@@ -1357,7 +1536,7 @@ tr_torrentRecheckCompleteness( tr_torrent * tor )
         {
             tr_trackerCompleted( tor->tracker );
 
-            tor->doneDate = time( NULL );
+            tor->doneDate = tor->anyDate = time( NULL );
         }
 
         tr_torrentSaveResume( tor );
@@ -1380,13 +1559,12 @@ tr_torrentInitFilePriority( tr_torrent *    tor,
 
     assert( tr_isTorrent( tor ) );
     assert( fileIndex < tor->info.fileCount );
-    assert( priority == TR_PRI_LOW || priority == TR_PRI_NORMAL || priority == TR_PRI_HIGH );
+    assert( tr_isPriority( priority ) );
 
     file = &tor->info.files[fileIndex];
     file->priority = priority;
     for( i = file->firstPiece; i <= file->lastPiece; ++i )
-        tor->info.pieces[i].priority = calculatePiecePriority( tor, i,
-                                                               fileIndex );
+        tor->info.pieces[i].priority = calculatePiecePriority( tor, i, fileIndex );
 }
 
 void
@@ -1532,7 +1710,8 @@ tr_torrentInitFileDLs( tr_torrent      * tor,
 
     for( i=0; i<fileCount; ++i )
         setFileDND( tor, files[i], doDownload );
-    tr_cpInvalidateDND ( &tor->completion );
+    tr_cpInvalidateDND( &tor->completion );
+    tor->needsSeedRatioCheck = TRUE;
 
     tr_torrentUnlock( tor );
 }
@@ -1555,6 +1734,27 @@ tr_torrentSetFileDLs( tr_torrent *      tor,
 ****
 ***/
 
+tr_priority_t
+tr_torrentGetPriority( const tr_torrent * tor )
+{
+    assert( tr_isTorrent( tor ) );
+
+    return tor->bandwidth->priority;
+}
+
+void
+tr_torrentSetPriority( tr_torrent * tor, tr_priority_t priority )
+{
+    assert( tr_isTorrent( tor ) );
+    assert( tr_isPriority( priority ) );
+
+    tor->bandwidth->priority = priority;
+}
+
+/***
+****
+***/
+
 void
 tr_torrentSetPeerLimit( tr_torrent * tor,
                         uint16_t     maxConnectedPeers )
@@ -1695,7 +1895,7 @@ tr_torrentUncheck( tr_torrent * tor )
 {
     assert( tr_isTorrent( tor ) );
 
-    tr_bitfieldRemRange ( &tor->checkedPieces, 0, tor->info.pieceCount );
+    tr_bitfieldRemRange( &tor->checkedPieces, 0, tor->info.pieceCount );
 }
 
 int
@@ -1749,7 +1949,7 @@ tr_torrentSetAnnounceList( tr_torrent *            tor,
     assert( tr_isTorrent( tor ) );
 
     /* save to the .torrent file */
-    if( !tr_bencLoadFile( tor->info.torrent, &metainfo ) )
+    if( !tr_bencLoadFile( &metainfo, TR_FMT_BENC, tor->info.torrent ) )
     {
         int       i;
         int       prevTier = -1;
@@ -1787,7 +1987,7 @@ tr_torrentSetAnnounceList( tr_torrent *            tor,
             tmpInfo.trackerCount = swap.trackerCount;
 
             tr_metainfoFree( &tmpInfo );
-            tr_bencSaveFile( tor->info.torrent, &metainfo );
+            tr_bencToFile( &metainfo, TR_FMT_BENC, tor->info.torrent );
         }
 
         /* cleanup */
@@ -1806,6 +2006,7 @@ tr_torrentSetAddedDate( tr_torrent * tor,
     assert( tr_isTorrent( tor ) );
 
     tor->addedDate = t;
+    tor->anyDate = MAX( tor->anyDate, tor->addedDate );
 }
 
 void
@@ -1815,6 +2016,7 @@ tr_torrentSetActivityDate( tr_torrent * tor,
     assert( tr_isTorrent( tor ) );
 
     tor->activityDate = t;
+    tor->anyDate = MAX( tor->anyDate, tor->activityDate );
 }
 
 void
@@ -1824,6 +2026,7 @@ tr_torrentSetDoneDate( tr_torrent * tor,
     assert( tr_isTorrent( tor ) );
 
     tor->doneDate = t;
+    tor->anyDate = MAX( tor->anyDate, tor->doneDate );
 }
 
 /**
@@ -2026,3 +2229,161 @@ tr_torrentDeleteLocalData( tr_torrent * tor, tr_fileFunc fileFunc )
         tr_free( path );
     }
 }
+
+/***
+****
+***/
+
+struct LocationData
+{
+    tr_bool move_from_old_location;
+    int * setme_state;
+    double * setme_progress;
+    char * location;
+    tr_torrent * tor;
+};
+
+static void
+setLocation( void * vdata )
+{
+    int err = 0;
+    struct LocationData * data = vdata;
+    tr_torrent * tor = data->tor;
+    const tr_bool do_move = data->move_from_old_location;
+    const char * location = data->location;
+    double bytesHandled = 0;
+
+    assert( tr_isTorrent( tor ) );
+
+    if( strcmp( location, tor->downloadDir ) )
+    {
+        tr_file_index_t i;
+
+        /* bad idea to move files while they're being verified... */
+        tr_verifyRemove( tor );
+
+        /* if the torrent is running, stop it and set a flag to
+         * restart after we're done */
+        if( tor->isRunning )
+        {
+            tr_torrentStop( tor );
+            tor->startAfterVerify = TRUE;
+        }
+
+        /* try to move the files.
+         * FIXME: there are still all kinds of nasty cases, like what
+         * if the target directory runs out of space halfway through... */
+        for( i=0; !err && i<tor->info.fileCount; ++i )
+        {
+            struct stat sb;
+            const tr_file * f = &tor->info.files[i];
+            char * oldpath = tr_buildPath( tor->downloadDir, f->name, NULL );
+            char * newpath = tr_buildPath( location, f->name, NULL );
+            
+            if( do_move )
+            {
+                errno = 0;
+                tr_torinf( tor, "moving \"%s\" to \"%s\"", oldpath, newpath );
+                if( tr_moveFile( oldpath, newpath ) ) {
+                    err = 1;
+                    tr_torerr( tor, "error moving \"%s\" to \"%s\": %s",
+                                    oldpath, newpath, tr_strerror( errno ) );
+                }
+            }
+            else if( !stat( newpath, &sb ) )
+            {
+                tr_torinf( tor, "found \"%s\"", newpath );
+            }
+
+            if( data->setme_progress )
+            {
+                bytesHandled += f->length;
+                *data->setme_progress = bytesHandled / tor->info.totalSize;
+            }
+
+            tr_free( newpath );
+            tr_free( oldpath );
+        }
+
+        if( !err )
+        {
+            /* blow away the leftover subdirectories in the old location */
+            tr_torrentDeleteLocalData( tor, unlink );
+
+            /* set the new location and reverify */
+            tr_torrentSetDownloadDir( tor, location );
+            tr_torrentVerify( tor );
+        }
+    }
+
+    if( data->setme_state )
+        *data->setme_state = err ? TR_LOC_ERROR : TR_LOC_DONE;
+
+    /* cleanup */
+    tr_free( data->location );
+    tr_free( data );
+}
+
+void
+tr_torrentSetLocation( tr_torrent  * tor,
+                       const char  * location,
+                       tr_bool       move_from_old_location,
+                       double      * setme_progress,
+                       int         * setme_state )
+{
+    struct LocationData * data;
+
+    assert( tr_isTorrent( tor ) );
+
+    if( setme_state )
+        *setme_state = TR_LOC_MOVING;
+    if( setme_progress )
+        *setme_progress = 0;
+
+    /* run this in the libtransmission thread */
+    data = tr_new( struct LocationData, 1 );
+    data->tor = tor;
+    data->location = tr_strdup( location );
+    data->move_from_old_location = move_from_old_location;
+    data->setme_state = setme_state;
+    data->setme_progress = setme_progress;
+    tr_runInEventThread( tor->session, setLocation, data );
+}
+
+/***
+****
+***/
+
+void
+tr_torrentCheckSeedRatio( tr_torrent * tor )
+{
+    double seedRatio;
+
+    assert( tr_isTorrent( tor ) );
+
+    /* if we're seeding and we've reached our seed ratio limit, stop the torrent */
+    if( tor->isRunning && tr_torrentIsSeed( tor ) && tr_torrentGetSeedRatio( tor, &seedRatio ) )
+    {
+        const uint64_t up = tor->uploadedCur + tor->uploadedPrev;
+        uint64_t down = tor->downloadedCur + tor->downloadedPrev;
+        double ratio;
+
+        /* maybe we're the initial seeder and never downloaded anything... */
+        if( down == 0 )
+            down = tr_cpHaveValid( &tor->completion );
+
+        ratio = tr_getRatio( up, down );
+
+        if( ratio >= seedRatio || ratio == TR_RATIO_INF )
+        {
+            tr_torrentStop( tor );
+
+            /* set to no ratio limit to allow easy restarting */
+            tr_torrentSetRatioMode( tor, TR_RATIOLIMIT_UNLIMITED );
+
+            /* maybe notify the client */
+            if( tor->ratio_limit_hit_func != NULL )
+                tor->ratio_limit_hit_func( tor, tor->ratio_limit_hit_func_user_data );
+        }
+    }
+}
