@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: torrent.h 8218 2009-04-12 15:01:29Z charles $
+ * $Id: torrent.h 8709 2009-06-16 19:31:17Z charles $
  */
 
 #ifndef __TRANSMISSION__
@@ -35,6 +35,10 @@ void        tr_ctorSetSave( tr_ctor * ctor,
 
 int         tr_ctorGetSave( const tr_ctor * ctor );
 
+void        tr_ctorInitTorrentPriorities( const tr_ctor * ctor, tr_torrent * tor );
+
+void        tr_ctorInitTorrentWanted( const tr_ctor * ctor, tr_torrent * tor );
+
 /**
 ***
 **/
@@ -55,9 +59,6 @@ void        tr_torrentSetHasPiece( tr_torrent *     tor,
 
 void        tr_torrentChangeMyPort( tr_torrent * session );
 
-tr_torrent* tr_torrentFindFromId( tr_session * session,
-                                  int          id );
-
 tr_torrent* tr_torrentFindFromHash( tr_session *    session,
                                     const uint8_t * hash );
 
@@ -71,6 +72,7 @@ tr_bool     tr_torrentIsPieceTransferAllowed( const tr_torrent * torrent,
                                               tr_direction       direction );
 
 
+
 #define tr_block( a, b ) _tr_block( tor, a, b )
 tr_block_index_t _tr_block( const tr_torrent * tor,
                             tr_piece_index_t   index,
@@ -90,7 +92,6 @@ void             tr_torrentInitFilePriority( tr_torrent       * tor,
                                              tr_file_index_t    fileIndex,
                                              tr_priority_t      priority );
 
-
 int              tr_torrentCountUncheckedPieces( const tr_torrent * );
 
 tr_bool          tr_torrentIsFileChecked( const tr_torrent  * tor,
@@ -115,6 +116,9 @@ time_t*          tr_torrentGetMTimes( const tr_torrent  * tor,
 tr_torrent*      tr_torrentNext( tr_session  * session,
                                  tr_torrent  * current );
 
+void             tr_torrentCheckSeedRatio( tr_torrent * tor );
+
+
 
 typedef enum
 {
@@ -124,13 +128,14 @@ typedef enum
 }
 tr_verify_state;
 
+void             tr_torrentSetVerifyState( tr_torrent      * tor,
+                                           tr_verify_state   state );
+
 struct tr_torrent
 {
     tr_session *             session;
     tr_info                  info;
 
-    tr_speedlimit            speedLimitMode[2];
-
     struct tr_ratecontrol    swarmSpeed;
 
     int                      magicNumber;
@@ -170,6 +175,9 @@ struct tr_torrent
     struct tr_tracker *        tracker;
     struct tr_publisher_tag *  trackerSubscription;
 
+    time_t                     dhtAnnounceAt;
+    tr_bool                    dhtAnnounceInProgress;
+
     uint64_t                   downloadedCur;
     uint64_t                   downloadedPrev;
     uint64_t                   uploadedCur;
@@ -181,12 +189,18 @@ struct tr_torrent
     time_t                     activityDate;
     time_t                     doneDate;
     time_t                     startDate;
+    time_t                     anyDate;
 
     tr_torrent_completeness_func *   completeness_func;
     void *                     completeness_func_user_data;
 
+    tr_torrent_ratio_limit_hit_func * ratio_limit_hit_func;
+    void *                     ratio_limit_hit_func_user_data;
+
     tr_bool                    isRunning;
     tr_bool                    isDeleting;
+    tr_bool                    needsSeedRatioCheck;
+    tr_bool                    startAfterVerify;
 
     uint16_t                   maxConnectedPeers;
 
@@ -202,6 +216,9 @@ struct tr_torrent
     struct tr_bandwidth      * bandwidth;
 
     struct tr_torrent_peers  * torrentPeers;
+
+    double                     desiredRatio;
+    tr_ratiolimit              ratioLimitMode;
 };
 
 /* get the index of this piece's first block */
@@ -274,6 +291,11 @@ static TR_INLINE tr_bool tr_torrentAllowsPex( const tr_torrent * tor )
     return ( tor != NULL  ) && tor->session->isPexEnabled && !tr_torrentIsPrivate( tor );
 }
 
+static TR_INLINE tr_bool tr_torrentAllowsDHT( const tr_torrent * tor )
+{
+    return ( tor != NULL  ) && tor->session->isDHTEnabled && !tr_torrentIsPrivate( tor );
+}
+
 static TR_INLINE tr_bool tr_torrentIsPieceChecked( const tr_torrent  * tor, tr_piece_index_t i )
 {
     return tr_bitfieldHasFast( &tor->checkedPieces, i );
