@@ -1,5 +1,5 @@
 /*
- * This file Copyright (C) 2007-2008 Charles Kerr <charles@rebelbase.com>
+ * This file Copyright (C) 2007-2009 Charles Kerr <charles@transmissionbt.com>
  *
  * This file is licensed by the GPL version 2.  Works owned by the
  * Transmission project are granted a special exemption to clause 2(b)
@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: upnp.c 6849 2008-10-04 15:23:47Z charles $
+ * $Id: upnp.c 8271 2009-04-22 16:00:45Z charles $
  */
 
 #include <assert.h>
@@ -37,12 +37,12 @@ tr_upnp_state;
 
 struct tr_upnp
 {
+    tr_bool            hasDiscovered;
     struct UPNPUrls    urls;
     struct IGDdatas    data;
     int                port;
     char               lanaddr[16];
     unsigned int       isMapped;
-    unsigned int       hasDiscovered : 1;
     tr_upnp_state      state;
 };
 
@@ -80,7 +80,8 @@ tr_upnpClose( tr_upnp * handle )
 int
 tr_upnpPulse( tr_upnp * handle,
               int       port,
-              int       isEnabled )
+              int       isEnabled,
+              int       doPortCheck )
 {
     int ret;
 
@@ -127,13 +128,31 @@ tr_upnpPulse( tr_upnp * handle,
             handle->state = TR_UPNP_UNMAP;
     }
 
+    if( isEnabled && handle->isMapped && doPortCheck )
+    {
+        char portStr[8];
+        char intPort[8];
+        char intClient[16];
+        int i;
+
+        tr_snprintf( portStr, sizeof( portStr ), "%d", handle->port );
+        i = UPNP_GetSpecificPortMappingEntry( handle->urls.controlURL,
+                                              handle->data.servicetype, portStr,
+                                              "TCP", intClient, intPort );
+        if( i != UPNPCOMMAND_SUCCESS )
+        {
+            tr_ninf( getKey( ), _( "Port %d isn't forwarded" ), handle->port );
+            handle->isMapped = FALSE;
+        }
+    }
+
     if( handle->state == TR_UPNP_UNMAP )
     {
         char portStr[16];
         tr_snprintf( portStr, sizeof( portStr ), "%d", handle->port );
         UPNP_DeletePortMapping( handle->urls.controlURL,
                                 handle->data.servicetype,
-                                portStr, "TCP" );
+                                portStr, "TCP", NULL );
         tr_ninf( getKey( ),
                  _(
                      "Stopping port forwarding through \"%s\", service \"%s\"" ),
@@ -152,18 +171,20 @@ tr_upnpPulse( tr_upnp * handle,
     if( handle->state == TR_UPNP_MAP )
     {
         int  err = -1;
-        char portStr[16];
-        tr_snprintf( portStr, sizeof( portStr ), "%d", port );
         errno = 0;
 
         if( !handle->urls.controlURL || !handle->data.servicetype )
             handle->isMapped = 0;
         else
         {
+            char portStr[16];
+            char desc[64];
+            tr_snprintf( portStr, sizeof( portStr ), "%d", port );
+            tr_snprintf( desc, sizeof( desc ), "%s at %d", TR_NAME, port );
             err = UPNP_AddPortMapping( handle->urls.controlURL,
                                        handle->data.servicetype,
                                        portStr, portStr, handle->lanaddr,
-                                       "Transmission", "TCP" );
+                                       desc, "TCP", NULL );
             handle->isMapped = !err;
         }
         tr_ninf( getKey( ),
