@@ -7,16 +7,20 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: utils.c 7900 2009-02-18 05:09:25Z charles $
+ * $Id: utils.c 8698 2009-06-15 03:24:40Z charles $
  */
 
+#ifdef HAVE_MEMMEM
+ #define _GNU_SOURCE /* glibc's string.h needs this to pick up memmem */
+#endif
+
 #include <assert.h>
 #include <ctype.h> /* isalpha, tolower */
 #include <errno.h>
 #include <stdarg.h>
 #include <stdio.h>
 #include <stdlib.h>
-#include <string.h> /* strerror, memset */
+#include <string.h> /* strerror, memset, memmem */
 
 #include <libgen.h> /* basename */
 #include <sys/time.h>
@@ -32,6 +36,7 @@
 #endif
 
 #include "transmission.h"
+#include "fdlimit.h"
 #include "ConvertUTF.h"
 #include "list.h"
 #include "utils.h"
@@ -178,8 +183,8 @@ tr_freeMessageList( tr_msg_list * list )
 ***
 **/
 
-static struct tm *
-tr_localtime_r( time_t *_clock, struct tm *_result )
+struct tm *
+tr_localtime_r( const time_t *_clock, struct tm *_result )
 {
 #ifdef HAVE_LOCALTIME_R
     return localtime_r( _clock, _result );
@@ -254,7 +259,7 @@ tr_deepLog( const char  * file,
     {
         va_list           args;
         char              timestr[64];
-        struct evbuffer * buf = tr_getBuffer( );
+        struct evbuffer * buf = evbuffer_new( );
         char *            base = tr_basename( file );
 
         evbuffer_add_printf( buf, "[%s] ",
@@ -265,12 +270,13 @@ tr_deepLog( const char  * file,
         evbuffer_add_vprintf( buf, fmt, args );
         va_end( args );
         evbuffer_add_printf( buf, " (%s:%d)\n", base, line );
+        /* FIXME(libevent2) ifdef this out for nonwindows platforms */
         OutputDebugString( EVBUFFER_DATA( buf ) );
-        if(fp)
+        if(fp) /* FIXME(libevent2) tr_getLog() should return an fd, then use evbuffer_write() here ) */
             (void) fwrite( EVBUFFER_DATA( buf ), 1, EVBUFFER_LENGTH( buf ), fp );
 
         tr_free( base );
-        tr_releaseBuffer( buf );
+        evbuffer_free( buf );
     }
 }
 
@@ -288,13 +294,11 @@ tr_msgLoggingIsActive( int level )
 }
 
 void
-tr_msg( const char * file,
-        int          line,
-        int          level,
-        const char * name,
-        const char * fmt,
-        ... )
+tr_msg( const char * file, int line,
+        int level, const char * name,
+        const char * fmt, ... )
 {
+    const int err = errno; /* message logging shouldn't affect errno */
     FILE * fp;
     tr_msgInit( );
     tr_lockLock( messageLock );
@@ -349,6 +353,7 @@ tr_msg( const char * file,
     }
 
     tr_lockUnlock( messageLock );
+    errno = err;
 }
 
 /***
@@ -452,6 +457,14 @@ tr_strip_positional_args( const char* str )
 ***
 **/
 
+tr_bool
+tr_isTimeval( const struct timeval * tv )
+{
+    return tv && ( tv->tv_sec >= 0 )
+              && ( tv->tv_usec >= 0 )
+              && ( tv->tv_usec < 1000000 );
+}
+
 void
 tr_timevalMsec( uint64_t milliseconds, struct timeval * setme )
 {
@@ -459,15 +472,37 @@ tr_timevalMsec( uint64_t milliseconds, struct timeval * setme )
     assert( setme != NULL );
     setme->tv_sec  = microseconds / 1000000;
     setme->tv_usec = microseconds % 1000000;
+    assert( tr_isTimeval( setme ) );
 }
 
+void
+tr_timevalSet( struct timeval * setme, int seconds, int microseconds )
+{
+    setme->tv_sec = seconds;
+    setme->tv_usec = microseconds;
+    assert( tr_isTimeval( setme ) );
+}
+
+void
+tr_timerAdd( struct event * timer, int seconds, int milliseconds )
+{
+    struct timeval tv;
+    tr_timevalSet( &tv, seconds, milliseconds );
+    event_add( timer, &tv );
+}
+
+/**
+***
+**/
+
 uint8_t *
 tr_loadFile( const char * path,
              size_t *     size )
 {
-    uint8_t *    buf;
+    uint8_t * buf;
     struct stat  sb;
-    FILE *       file;
+    int fd;
+    ssize_t n;
     const char * err_fmt = _( "Couldn't read \"%1$s\": %2$s" );
 
     /* try to stat the file */
@@ -488,34 +523,36 @@ tr_loadFile( const char * path,
     }
 
     /* Load the torrent file into our buffer */
-    file = fopen( path, "rb" );
-    if( !file )
+    fd = tr_open_file_for_scanning( path );
+    if( fd < 0 )
     {
         const int err = errno;
         tr_err( err_fmt, path, tr_strerror( errno ) );
         errno = err;
         return NULL;
     }
-    buf = malloc( sb.st_size );
+    buf = malloc( sb.st_size + 1 );
     if( !buf )
     {
         const int err = errno;
         tr_err( err_fmt, path, _( "Memory allocation failed" ) );
-        fclose( file );
+        tr_close_file( fd );
         errno = err;
         return NULL;
     }
-    if( fread( buf, sb.st_size, 1, file ) != 1 )
+    n = read( fd, buf, sb.st_size );
+    if( n == -1 )
     {
         const int err = errno;
         tr_err( err_fmt, path, tr_strerror( errno ) );
-        fclose( file );
+        tr_close_file( fd );
         free( buf );
         errno = err;
         return NULL;
     }
 
-    fclose( file );
+    tr_close_file( fd );
+    buf[ sb.st_size ] = '\0';
     *size = sb.st_size;
     return buf;
 }
@@ -680,21 +717,45 @@ tr_strndup( const void * in, int len )
     return out;
 }
 
+const char*
+tr_memmem( const char * haystack, size_t haystacklen,
+           const char * needle, size_t needlelen )
+{
+#ifdef HAVE_MEMMEM
+    return memmem( haystack, haystacklen, needle, needlelen );
+#else
+    size_t i;
+    if( !needlelen )
+        return haystack;
+    if( needlelen > haystacklen || !haystack || !needle )
+        return NULL;
+    for( i=0; i<=haystacklen-needlelen; ++i )
+        if( !memcmp( haystack+i, needle, needlelen ) )
+            return haystack+i;
+    return NULL;
+#endif
+}
+
 char*
 tr_strdup_printf( const char * fmt, ... )
 {
-    char *            ret = NULL;
-    struct evbuffer * buf;
-    va_list           ap;
+    va_list ap;
+    char * ret;
+    size_t len;
+    char statbuf[2048];
 
-    buf = tr_getBuffer( );
     va_start( ap, fmt );
-
-    if( evbuffer_add_vprintf( buf, fmt, ap ) != -1 )
-        ret = tr_strdup( EVBUFFER_DATA( buf ) );
-
+    len = evutil_vsnprintf( statbuf, sizeof( statbuf ), fmt, ap );
     va_end( ap );
-    tr_releaseBuffer( buf );
+    if( len < sizeof( statbuf ) )
+        ret = tr_strndup( statbuf, len );
+    else {
+        ret = tr_new( char, len + 1 );
+        va_start( ap, fmt );
+        evutil_vsnprintf( ret, len + 1, fmt, ap );
+        va_end( ap );
+    }
+
     return ret;
 }
 
@@ -740,232 +801,6 @@ tr_strstrip( char * str )
 *****
 ****/
 
-tr_bitfield*
-tr_bitfieldConstruct( tr_bitfield * b, size_t bitCount )
-{
-    b->bitCount = bitCount;
-    b->byteCount = ( bitCount + 7u ) / 8u;
-    b->bits = tr_new0( uint8_t, b->byteCount );
-    return b;
-}
-
-tr_bitfield*
-tr_bitfieldDestruct( tr_bitfield * b )
-{
-    if( b )
-        tr_free( b->bits );
-    return b;
-}
-
-tr_bitfield*
-tr_bitfieldDup( const tr_bitfield * in )
-{
-    tr_bitfield * ret = tr_new0( tr_bitfield, 1 );
-
-    ret->bitCount = in->bitCount;
-    ret->byteCount = in->byteCount;
-    ret->bits = tr_memdup( in->bits, in->byteCount );
-    return ret;
-}
-
-void
-tr_bitfieldClear( tr_bitfield * bitfield )
-{
-    memset( bitfield->bits, 0, bitfield->byteCount );
-}
-
-int
-tr_bitfieldIsEmpty( const tr_bitfield * bitfield )
-{
-    size_t i;
-
-    for( i = 0; i < bitfield->byteCount; ++i )
-        if( bitfield->bits[i] )
-            return 0;
-
-    return 1;
-}
-
-int
-tr_bitfieldAdd( tr_bitfield * bitfield,
-                size_t        nth )
-{
-    assert( bitfield );
-    assert( bitfield->bits );
-
-    if( nth >= bitfield->bitCount )
-        return -1;
-
-    bitfield->bits[nth >> 3u] |= ( 0x80 >> ( nth & 7u ) );
-    return 0;
-}
-
-/* Sets bit range [begin, end) to 1 */
-int
-tr_bitfieldAddRange( tr_bitfield * b,
-                     size_t        begin,
-                     size_t        end )
-{
-    size_t        sb, eb;
-    unsigned char sm, em;
-
-    end--;
-
-    if( ( end >= b->bitCount ) || ( begin > end ) )
-        return -1;
-
-    sb = begin >> 3;
-    sm = ~( 0xff << ( 8 - ( begin & 7 ) ) );
-    eb = end >> 3;
-    em = 0xff << ( 7 - ( end & 7 ) );
-
-    if( sb == eb )
-    {
-        b->bits[sb] |= ( sm & em );
-    }
-    else
-    {
-        b->bits[sb] |= sm;
-        b->bits[eb] |= em;
-        if( ++sb < eb )
-            memset ( b->bits + sb, 0xff, eb - sb );
-    }
-
-    return 0;
-}
-
-int
-tr_bitfieldRem( tr_bitfield * bitfield,
-                size_t        nth )
-{
-    assert( bitfield );
-    assert( bitfield->bits );
-
-    if( nth >= bitfield->bitCount )
-        return -1;
-
-    bitfield->bits[nth >> 3u] &= ( 0xff7f >> ( nth & 7u ) );
-    return 0;
-}
-
-/* Clears bit range [begin, end) to 0 */
-int
-tr_bitfieldRemRange( tr_bitfield * b,
-                     size_t        begin,
-                     size_t        end )
-{
-    size_t        sb, eb;
-    unsigned char sm, em;
-
-    end--;
-
-    if( ( end >= b->bitCount ) || ( begin > end ) )
-        return -1;
-
-    sb = begin >> 3;
-    sm = 0xff << ( 8 - ( begin & 7 ) );
-    eb = end >> 3;
-    em = ~( 0xff << ( 7 - ( end & 7 ) ) );
-
-    if( sb == eb )
-    {
-        b->bits[sb] &= ( sm | em );
-    }
-    else
-    {
-        b->bits[sb] &= sm;
-        b->bits[eb] &= em;
-        if( ++sb < eb )
-            memset ( b->bits + sb, 0, eb - sb );
-    }
-
-    return 0;
-}
-
-tr_bitfield*
-tr_bitfieldOr( tr_bitfield *       a,
-               const tr_bitfield * b )
-{
-    uint8_t *      ait;
-    const uint8_t *aend, *bit;
-
-    assert( a->bitCount == b->bitCount );
-
-    for( ait = a->bits, bit = b->bits, aend = ait + a->byteCount;
-         ait != aend; )
-        *ait++ |= *bit++;
-
-    return a;
-}
-
-/* set 'a' to all the flags that were in 'a' but not 'b' */
-void
-tr_bitfieldDifference( tr_bitfield *       a,
-                       const tr_bitfield * b )
-{
-    uint8_t *      ait;
-    const uint8_t *aend, *bit;
-
-    assert( a->bitCount == b->bitCount );
-
-    for( ait = a->bits, bit = b->bits, aend = ait + a->byteCount;
-         ait != aend; )
-        *ait++ &= ~( *bit++ );
-}
-
-size_t
-tr_bitfieldCountTrueBits( const tr_bitfield* b )
-{
-    size_t           ret = 0;
-    const uint8_t *  it, *end;
-    static const int trueBitCount[512] = {
-        0, 1, 1, 2, 1, 2, 2, 3, 1, 2, 2, 3, 2, 3, 3, 4, 1, 2, 2, 3, 2, 3, 3,
-        4, 2, 3, 3, 4, 3, 4, 4, 5,
-        1, 2, 2, 3, 2, 3, 3, 4, 2, 3, 3, 4, 3, 4, 4, 5, 2, 3, 3, 4, 3, 4, 4,
-        5, 3, 4, 4, 5, 4, 5, 5, 6,
-        1, 2, 2, 3, 2, 3, 3, 4, 2, 3, 3, 4, 3, 4, 4, 5, 2, 3, 3, 4, 3, 4, 4,
-        5, 3, 4, 4, 5, 4, 5, 5, 6,
-        2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 5, 5,
-        6, 4, 5, 5, 6, 5, 6, 6, 7,
-        1, 2, 2, 3, 2, 3, 3, 4, 2, 3, 3, 4, 3, 4, 4, 5, 2, 3, 3, 4, 3, 4, 4,
-        5, 3, 4, 4, 5, 4, 5, 5, 6,
-        2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 5, 5,
-        6, 4, 5, 5, 6, 5, 6, 6, 7,
-        2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 5, 5,
-        6, 4, 5, 5, 6, 5, 6, 6, 7,
-        3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6,
-        7, 5, 6, 6, 7, 6, 7, 7, 8,
-        1, 2, 2, 3, 2, 3, 3, 4, 2, 3, 3, 4, 3, 4, 4, 5, 2, 3, 3, 4, 3, 4, 4,
-        5, 3, 4, 4, 5, 4, 5, 5, 6,
-        2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 5, 5,
-        6, 4, 5, 5, 6, 5, 6, 6, 7,
-        2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 5, 5,
-        6, 4, 5, 5, 6, 5, 6, 6, 7,
-        3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6,
-        7, 5, 6, 6, 7, 6, 7, 7, 8,
-        2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6, 3, 4, 4, 5, 4, 5, 5,
-        6, 4, 5, 5, 6, 5, 6, 6, 7,
-        3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6,
-        7, 5, 6, 6, 7, 6, 7, 7, 8,
-        3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7, 4, 5, 5, 6, 5, 6, 6,
-        7, 5, 6, 6, 7, 6, 7, 7, 8,
-        4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8, 5, 6, 6, 7, 6, 7, 7,
-        8, 6, 7, 7, 8, 7, 8, 8, 9
-    };
-
-    if( !b )
-        return 0;
-
-    for( it = b->bits, end = it + b->byteCount; it != end; ++it )
-        ret += trueBitCount[*it];
-
-    return ret;
-}
-
-/***
-****
-***/
-
 uint64_t
 tr_date( void )
 {
@@ -1262,49 +1097,6 @@ tr_int2ptr( int i )
 ****
 ***/
 
-static tr_list * _bufferList = NULL;
-
-static tr_lock *
-getBufferLock( void )
-{
-    static tr_lock * lock = NULL;
-    if( lock == NULL )
-        lock = tr_lockNew( );
-    return lock;
-}
-
-struct evbuffer*
-tr_getBuffer( void )
-{
-    struct evbuffer * buf;
-    tr_lock * l = getBufferLock( );
-    tr_lockLock( l );
-
-    buf = tr_list_pop_front( &_bufferList );
-    if( buf == NULL )
-        buf = evbuffer_new( );
-
-    tr_lockUnlock( l );
-    return buf;
-}
-
-void
-tr_releaseBuffer( struct evbuffer * buf )
-{
-    tr_lock * l = getBufferLock( );
-    tr_lockLock( l );
-
-    evbuffer_drain( buf, EVBUFFER_LENGTH( buf ) );
-    assert( EVBUFFER_LENGTH( buf ) == 0 );
-    tr_list_prepend( &_bufferList, buf );
-
-    tr_lockUnlock( l );
-}
-
-/***
-****
-***/
-
 int
 tr_lowerBound( const void * key,
                const void * base,
@@ -1352,35 +1144,44 @@ tr_lowerBound( const void * key,
 char*
 tr_utf8clean( const char * str, int max_len, tr_bool * err )
 {
-    const char zero = '\0';
     char * ret;
-    struct evbuffer * buf = evbuffer_new( );
     const char * end;
 
-    if( err != NULL )
-        *err = FALSE;
-
     if( max_len < 0 )
         max_len = (int) strlen( str );
 
-    while( !tr_utf8_validate ( str, max_len, &end ) )
+    if( err != NULL )
+        *err = FALSE;
+
+    if( tr_utf8_validate( str, max_len, &end  ) )
     {
-        const int good_len = end - str;
+        ret = tr_strndup( str, max_len );
+    }
+    else
+    {
+        const char zero = '\0';
+        struct evbuffer * buf = evbuffer_new( );
 
-        evbuffer_add( buf, str, good_len );
-        max_len -= ( good_len + 1 );
-        str += ( good_len + 1 );
-        evbuffer_add( buf, "?", 1 );
+        while( !tr_utf8_validate ( str, max_len, &end ) )
+        {
+            const int good_len = end - str;
 
-        if( err != NULL )
-            *err = TRUE;
+            evbuffer_add( buf, str, good_len );
+            max_len -= ( good_len + 1 );
+            str += ( good_len + 1 );
+            evbuffer_add( buf, "?", 1 );
+
+            if( err != NULL )
+                *err = TRUE;
+        }
+
+        evbuffer_add( buf, str, max_len );
+        evbuffer_add( buf, &zero, 1 );
+        ret = tr_memdup( EVBUFFER_DATA( buf ), EVBUFFER_LENGTH( buf ) );
+        evbuffer_free( buf );
     }
 
-    evbuffer_add( buf, str, max_len );
-    evbuffer_add( buf, &zero, 1 );
-    ret = tr_memdup( EVBUFFER_DATA( buf ), EVBUFFER_LENGTH( buf ) );
     assert( tr_utf8_validate( ret, -1, NULL ) );
-    evbuffer_free( buf );
     return ret;
 }
 
@@ -1518,3 +1319,108 @@ tr_parseNumberRange( const char * str_in, int len, int * setmeCount )
     *setmeCount = n;
     return uniq;
 }
+
+/***
+****
+***/
+
+static void
+printf_double_without_rounding( char * buf, int buflen, double d, int places )
+{
+    char * pch;
+    char tmp[128];
+    int len;
+    tr_snprintf( tmp, sizeof( tmp ), "%'.64f", d );
+    pch = tmp;
+    while( isdigit( *pch ) ) ++pch; /* walk to the decimal point */
+    ++pch; /* walk over the decimal point */
+    pch += places;
+    len = MIN( buflen - 1, pch - tmp );
+    memcpy( buf, tmp, len );
+    buf[len] = '\0';
+}
+
+char*
+tr_strratio( char * buf, size_t buflen, double ratio, const char * infinity )
+{
+    if( (int)ratio == TR_RATIO_NA )
+        tr_strlcpy( buf, _( "None" ), buflen );
+    else if( (int)ratio == TR_RATIO_INF )
+        tr_strlcpy( buf, infinity, buflen );
+    else if( ratio < 10.0 )
+        printf_double_without_rounding( buf, buflen, ratio, 2 );
+    else if( ratio < 100.0 )
+        printf_double_without_rounding( buf, buflen, ratio, 1 );
+    else
+        tr_snprintf( buf, buflen, "%'.0f", ratio );
+    return buf;
+}
+
+/***
+****
+***/
+
+int
+tr_moveFile( const char * oldpath, const char * newpath )
+{
+    int in;
+    int out;
+    char * buf;
+    struct stat st;
+    size_t bytesLeft;
+    size_t buflen;
+
+    /* make sure the old file exists */
+    if( stat( oldpath, &st ) ) {
+        const int err = errno;
+        errno = err;
+        return -1;
+    }
+    if( !S_ISREG( st.st_mode ) ) {
+        errno = ENOENT;
+        return -1;
+    }
+    bytesLeft = st.st_size;
+
+    /* make sure the target directory exists */
+    {
+        char * newdir = tr_dirname( newpath );
+        int i = tr_mkdirp( newdir, 0777 );
+        tr_free( newdir );
+        if( i )
+            return i;
+    }
+
+    /* they  might be on the same filesystem... */
+    if( !rename( oldpath, newpath ) )
+        return 0;
+
+    /* copy the file */
+    in = tr_open_file_for_scanning( oldpath );
+    tr_preallocate_file( newpath, bytesLeft );
+    out = tr_open_file_for_writing( newpath );
+    buflen = stat( newpath, &st ) ? 4096 : st.st_blksize;
+    buf = tr_new( char, buflen );
+    while( bytesLeft > 0 )
+    {
+        ssize_t bytesWritten;
+        const size_t bytesThisPass = MIN( bytesLeft, buflen );
+        const int numRead = read( in, buf, bytesThisPass );
+        if( numRead < 0 )
+            break;
+        bytesWritten = write( out, buf, numRead );
+        if( bytesWritten < 0 )
+            break;
+        bytesLeft -= bytesWritten;
+    }
+
+    /* cleanup */
+    tr_free( buf );
+    tr_close_file( out );
+    tr_close_file( in );
+    if( bytesLeft != 0 )
+        return -1;
+
+    unlink( oldpath );
+    return 0;
+}
