@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: web.c 8204 2009-04-10 17:34:25Z charles $
+ * $Id: web.c 8391 2009-05-13 19:04:14Z charles $
  */
 
 #include <assert.h>
@@ -24,6 +24,7 @@
 #include "net.h" /* socklen_t */
 #include "trevent.h"
 #include "utils.h"
+#include "version.h"
 #include "web.h"
 
 /* Use curl_multi_socket_action() instead of curl_multi_perform()
@@ -83,7 +84,7 @@ struct tr_web
 static struct tr_web_sockinfo *
 getSockinfo( tr_web * web, int fd, tr_bool createIfMissing )
 {
-    tr_list * l = web->fds;
+    tr_list * l;
 
     for( l=web->fds; l!=NULL; l=l->next ) {
         struct tr_web_sockinfo * s =  l->data;
