@@ -1,7 +1,7 @@
 /******************************************************************************
- * $Id: Controller.h 7666 2009-01-11 18:04:45Z livings124 $
+ * $Id: Controller.h 8386 2009-05-12 00:58:22Z livings124 $
  *
- * Copyright (c) 2005-2008 Transmission authors and contributors
+ * Copyright (c) 2005-2009 Transmission authors and contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -36,7 +36,6 @@
 @class TorrentTableView;
 @class StatusBarView;
 @class FilterButton;
-@class MenuLabel;
 
 typedef enum
 {
@@ -67,15 +66,11 @@ typedef enum
     NSTimer                         * fTimer;
     
     IBOutlet NSMenuItem             * fOpenIgnoreDownloadFolder;
-    
-    IBOutlet NSBox                  * fBottomTigerLine;
     IBOutlet NSButton               * fActionButton, * fSpeedLimitButton;
     IBOutlet NSTextField            * fTotalTorrentsField;
     
     IBOutlet StatusBarView          * fStatusBar;
     IBOutlet NSButton               * fStatusButton;
-    IBOutlet MenuLabel              * fStatusTigerField;
-    IBOutlet NSImageView            * fStatusTigerImageView;
     IBOutlet NSTextField            * fTotalDLField, * fTotalULField;
     IBOutlet NSImageView            * fTotalDLImageView;
     
@@ -102,12 +97,11 @@ typedef enum
     IBOutlet NSButton               * fURLSheetOpenButton;
     
     BOOL                            fUpdateInProgress;
+    BOOL                            fPauseOnLaunch;
     
     Badger                          * fBadger;
     IBOutlet NSMenu                 * fDockMenu;
     
-    NSTimer                         * fSpeedLimitTimer;
-    
     NSMutableArray                  * fAutoImportedNames;
     NSTimer                         * fAutoImportTimer;
     
@@ -186,6 +180,8 @@ typedef enum
 - (void) resizeStatusButton;
 - (void) setBottomCountText: (BOOL) filtering;
 
+- (void) updateSpeedFieldsToolTips;
+
 - (void) updateTorrentsInQueue;
 - (NSInteger) numToStartFromQueue: (BOOL) downloadQueue;
 
@@ -209,16 +205,14 @@ typedef enum
 
 - (void) setStatusLabel: (id) sender;
 
-- (void) showGroups: (id) sender;
 - (void) setGroup: (id) sender; //used by delegate-generated menu items
 - (void) setGroupFilter: (id) sender;
 - (void) updateGroupsFilterButton;
 - (void) updateGroupsFilters: (NSNotification *) notification;
 
 - (void) toggleSpeedLimit: (id) sender;
-- (void) autoSpeedLimitChange: (NSNotification *) notification;
-- (void) autoSpeedLimit: (NSTimer *) timer;
-- (void) setAutoSpeedLimitTimer: (BOOL) nextIsLimit;
+- (void) speedLimitChanged: (id) sender;
+- (void) altSpeedToggledCallbackIsLimited: (NSDictionary *) dict;
 
 - (void) setLimitGlobalEnabled: (id) sender;
 - (void) setQuickLimitGlobal: (id) sender;
