@@ -1,7 +1,7 @@
 /******************************************************************************
- * $Id: CreatorWindowController.m 7032 2008-11-04 01:31:24Z livings124 $
+ * $Id: CreatorWindowController.m 8675 2009-06-11 23:02:54Z livings124 $
  *
- * Copyright (c) 2007-2008 Transmission authors and contributors
+ * Copyright (c) 2007-2009 Transmission authors and contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -23,7 +23,6 @@
  *****************************************************************************/
 
 #import "CreatorWindowController.h"
-#import "NSApplicationAdditions.h"
 #import "NSStringAdditions.h"
 #import "utils.h" //tr_httpIsValidURL
 
@@ -45,7 +44,7 @@
 
 @implementation CreatorWindowController
 
-+ (void) createTorrentFile: (tr_handle *) handle
++ (void) createTorrentFile: (tr_session *) handle
 {
     //get file/folder for torrent
     NSString * path;
@@ -56,20 +55,20 @@
     [creator showWindow: nil];
 }
 
-+ (void) createTorrentFile: (tr_handle *) handle forFile: (NSString *) file
++ (void) createTorrentFile: (tr_session *) handle forFile: (NSString *) file
 {
     CreatorWindowController * creator = [[self alloc] initWithHandle: handle path: file];
     [creator showWindow: nil];
 }
 
-- (id) initWithHandle: (tr_handle *) handle path: (NSString *) path
+- (id) initWithHandle: (tr_session *) handle path: (NSString *) path
 {
     if ((self = [super initWithWindowNibName: @"Creator"]))
     {
         fStarted = NO;
         
         fPath = [path retain];
-        fInfo = tr_metaInfoBuilderCreate(handle, [fPath UTF8String]);
+        fInfo = tr_metaInfoBuilderCreate([fPath UTF8String]);
         
         if (fInfo->fileCount == 0)
         {
@@ -184,13 +183,6 @@
     
     fOpenTorrent = [fDefaults boolForKey: @"CreatorOpen"];
     [self updateEnableOpenCheckForTrackers];
-    
-    if (![NSApp isOnLeopardOrBetter])
-    {
-        [fTrackerAddRemoveControl sizeToFit];
-        [fTrackerAddRemoveControl setLabel: @"+" forSegment: TRACKER_ADD_TAG];
-        [fTrackerAddRemoveControl setLabel: @"-" forSegment: TRACKER_REMOVE_TAG];
-    }
 }
 
 - (void) dealloc
@@ -221,7 +213,7 @@
     [panel setMessage: NSLocalizedString(@"Select the name and location for the torrent file.",
                                         "Create torrent -> location sheet -> message")]; 
     
-    [panel setRequiredFileType: @"torrent"];
+    [panel setAllowedFileTypes: [NSArray arrayWithObjects: @"org.bittorrent.torrent", @"torrent", nil]];
     [panel setCanSelectHiddenExtension: YES];
     
     [panel beginSheetForDirectory: [fLocation stringByDeletingLastPathComponent] file: [fLocation lastPathComponent]
@@ -240,11 +232,7 @@
             " that will add the address for you.", "Create torrent -> blank address -> message")];
         [alert addButtonWithTitle: NSLocalizedString(@"Create", "Create torrent -> blank address -> button")];
         [alert addButtonWithTitle: NSLocalizedString(@"Cancel", "Create torrent -> blank address -> button")];
-        
-        if ([NSApp isOnLeopardOrBetter])
-            [alert setShowsSuppressionButton: YES];
-        else
-            [alert addButtonWithTitle: NSLocalizedString(@"Don't Alert Again", "Create torrent -> blank address -> button")];
+        [alert setShowsSuppressionButton: YES];
 
         [alert beginSheetModalForWindow: [self window] modalDelegate: self
             didEndSelector: @selector(createBlankAddressAlertDidEnd:returnCode:contextInfo:) contextInfo: nil];
@@ -377,7 +365,7 @@
 
 - (void) createBlankAddressAlertDidEnd: (NSAlert *) alert returnCode: (NSInteger) returnCode contextInfo: (void *) contextInfo
 {
-    if (([NSApp isOnLeopardOrBetter] ? [[alert suppressionButton] state] == NSOnState : returnCode == NSAlertThirdButtonReturn))
+    if ([[alert suppressionButton] state] == NSOnState)
         [[NSUserDefaults standardUserDefaults] setBool: NO forKey: @"WarningCreatorBlankAddress"];
     
     [alert release];
@@ -468,10 +456,12 @@
                 else if (fInfo->result == TR_MAKEMETA_IO_WRITE)
                     [alert setInformativeText: [NSString stringWithFormat: NSLocalizedString(@"Could not write \"%s\": %s.",
                         "Create torrent -> failed -> warning"), fInfo->errfile, strerror(fInfo->my_errno)]];
-                else; //invalid url should have been caught before creating
+                else //invalid url should have been caught before creating
+                    [alert setInformativeText: [NSString stringWithFormat: @"%@ (%d)",
+                        NSLocalizedString(@"An unknown error has occurred.", "Create torrent -> failed -> warning"), fInfo->result]];
                 
                 [alert beginSheetModalForWindow: [self window] modalDelegate: self
-                        didEndSelector: @selector(failureSheetClosed:returnCode:contextInfo:) contextInfo: nil];
+                    didEndSelector: @selector(failureSheetClosed:returnCode:contextInfo:) contextInfo: nil];
         }
     }
     else
