@@ -1,7 +1,7 @@
 /******************************************************************************
- * $Id: InfoWindowController.h 7016 2008-11-02 01:07:01Z livings124 $
+ * $Id: InfoWindowController.h 8261 2009-04-20 03:36:35Z livings124 $
  *
- * Copyright (c) 2006-2008 Transmission authors and contributors
+ * Copyright (c) 2006-2009 Transmission authors and contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -68,10 +68,13 @@
     NSViewAnimation * fWebSeedTableAnimation;
     
     IBOutlet FileOutlineController * fFileController;
+    IBOutlet NSSearchField * fFileFilterField;
     
-    IBOutlet NSPopUpButton * fRatioPopUp, * fUploadLimitPopUp, * fDownloadLimitPopUp;
-    IBOutlet NSTextField * fUploadLimitField, * fDownloadLimitField, * fRatioLimitField, * fPeersConnectField,
-                        * fUploadLimitLabel, * fDownloadLimitLabel, * fPeersConnectLabel;
+    IBOutlet NSPopUpButton * fPriorityPopUp, * fRatioPopUp;
+    IBOutlet NSButton * fUploadLimitCheck, * fDownloadLimitCheck, * fGlobalLimitCheck;
+    IBOutlet NSTextField * fUploadLimitField, * fDownloadLimitField, * fRatioLimitField,
+                        * fUploadLimitLabel, * fDownloadLimitLabel, * fPeersConnectLabel,
+                        * fPeersConnectField;
     
     NSString * fInitialString;
 }
@@ -98,12 +101,17 @@
 - (void) revealTorrentFile: (id) sender;
 - (void) revealDataFile: (id) sender;
 
-- (void) setSpeedMode: (id) sender;
+- (void) setFileFilterText: (id) sender;
+
+- (void) setUseSpeedLimit: (id) sender;
 - (void) setSpeedLimit: (id) sender;
+- (void) setUseGlobalSpeedLimit: (id) sender;
 
 - (void) setRatioSetting: (id) sender;
 - (void) setRatioLimit: (id) sender;
 
+- (void) setPriority: (id) sender;
+
 - (void) setPeersConnectLimit: (id) sender;
 
 @end
