@@ -1,7 +1,7 @@
 /******************************************************************************
- * $Id: PrefsController.m 7081 2008-11-09 12:33:09Z livings124 $
+ * $Id: PrefsController.m 8461 2009-05-21 04:35:40Z livings124 $
  *
- * Copyright (c) 2005-2008 Transmission authors and contributors
+ * Copyright (c) 2005-2009 Transmission authors and contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -26,11 +26,13 @@
 #import "BlocklistDownloaderViewController.h"
 #import "BlocklistScheduler.h"
 #import "PortChecker.h"
-#import "NSApplicationAdditions.h"
+#import "BonjourController.h"
 #import "NSStringAdditions.h"
 #import "UKKQueue.h"
 #import "utils.h"
 
+#import <Sparkle/Sparkle.h>
+
 #define DOWNLOAD_FOLDER     0
 #define DOWNLOAD_TORRENT    2
 
@@ -43,6 +45,7 @@
 
 #define TOOLBAR_GENERAL     @"TOOLBAR_GENERAL"
 #define TOOLBAR_TRANSFERS   @"TOOLBAR_TRANSFERS"
+#define TOOLBAR_GROUPS      @"TOOLBAR_GROUPS"
 #define TOOLBAR_BANDWIDTH   @"TOOLBAR_BANDWIDTH"
 #define TOOLBAR_PEERS       @"TOOLBAR_PEERS"
 #define TOOLBAR_NETWORK     @"TOOLBAR_NETWORK"
@@ -70,13 +73,13 @@
 
 @implementation PrefsController
 
-tr_handle * fHandle;
-+ (void) setHandle: (tr_handle *) handle
+tr_session * fHandle;
++ (void) setHandle: (tr_session *) handle
 {
     fHandle = handle;
 }
 
-+ (tr_handle *) handle
++ (tr_session *) handle
 {
     return fHandle;
 }
@@ -87,18 +90,6 @@ tr_handle * fHandle;
     {
         fDefaults = [NSUserDefaults standardUserDefaults];
         
-        //checks for old version speeds of -1
-        if ([fDefaults integerForKey: @"UploadLimit"] < 0)
-        {
-            [fDefaults removeObjectForKey: @"UploadLimit"];
-            [fDefaults setBool: NO forKey: @"CheckUpload"];
-        }
-        if ([fDefaults integerForKey: @"DownloadLimit"] < 0)
-        {
-            [fDefaults removeObjectForKey: @"DownloadLimit"];
-            [fDefaults setBool: NO forKey: @"CheckDownload"];
-        }
-        
         //check for old version download location (before 1.1)
         NSString * choice;
         if ((choice = [fDefaults stringForKey: @"DownloadChoice"]))
@@ -109,6 +100,10 @@ tr_handle * fHandle;
             [fDefaults removeObjectForKey: @"DownloadChoice"];
         }
         
+        //save a new random port
+        if ([fDefaults boolForKey: @"RandomPort"])
+            [fDefaults setInteger: tr_sessionGetPeerPort(fHandle) forKey: @"BindPort"];
+        
         //set auto import
         NSString * autoPath;
         if ([fDefaults boolForKey: @"AutoImport"] && (autoPath = [fDefaults stringForKey: @"AutoImportDirectory"]))
@@ -120,9 +115,6 @@ tr_handle * fHandle;
         //set encryption
         [self setEncryptionMode: nil];
         
-        //actually set bandwidth limits
-        [self applySpeedSettings: nil];
-        
         //set proxy type
         [self updateProxyType];
         [self updateProxyPassword];
@@ -134,6 +126,16 @@ tr_handle * fHandle;
         if (!fRPCWhitelistArray)
             fRPCWhitelistArray = [[NSMutableArray arrayWithObject: @"127.0.0.1"] retain];
         [self updateRPCWhitelist];
+        
+        //reset old Sparkle settings from previous versions
+        [fDefaults removeObjectForKey: @"SUScheduledCheckInterval"];
+        if ([fDefaults objectForKey: @"CheckForUpdates"])
+        {
+            [[SUUpdater sharedUpdater] setAutomaticallyChecksForUpdates: [fDefaults boolForKey: @"CheckForUpdates"]];
+            [fDefaults removeObjectForKey: @"CheckForUpdates"];
+        }
+        
+        [self setAutoUpdateToBeta: nil];
     }
     
     return self;
@@ -152,6 +154,8 @@ tr_handle * fHandle;
     
     [fRPCWhitelistArray release];
     
+    [fRPCPassword release];
+    
     [super dealloc];
 }
 
@@ -170,18 +174,11 @@ tr_handle * fHandle;
     
     [self setPrefView: nil];
     
-    if (![NSApp isOnLeopardOrBetter])
-    {
-        [fRPCAddRemoveControl sizeToFit];
-        [fRPCAddRemoveControl setLabel: @"+" forSegment: RPC_IP_ADD_TAG];
-        [fRPCAddRemoveControl setLabel: @"-" forSegment: RPC_IP_REMOVE_TAG];
-    }
-    
     //set download folder
     [fFolderPopUp selectItemAtIndex: [fDefaults boolForKey: @"DownloadLocationConstant"] ? DOWNLOAD_FOLDER : DOWNLOAD_TORRENT];
     
     //set stop ratio
-    [self updateRatioStopField];
+    [fRatioStopField setFloatValue: [fDefaults floatForKey: @"RatioLimit"]];
     
     //set limits
     [self updateLimitFields];
@@ -210,7 +207,7 @@ tr_handle * fHandle;
     //set proxy type
     [fProxyAddressField setStringValue: [fDefaults stringForKey: @"ProxyAddress"]];
     int proxyType;
-    switch(tr_sessionGetProxyType(fHandle))
+    switch (tr_sessionGetProxyType(fHandle))
     {
         case TR_PROXY_SOCKS4:
             proxyType = PROXY_SOCKS4;
@@ -237,10 +234,9 @@ tr_handle * fHandle;
     //set rpc port
     [fRPCPortField setIntValue: [fDefaults integerForKey: @"RPCPort"]];
     
-    //set rpc password - has to be released
-    char * rpcPassword = tr_sessionGetRPCPassword(fHandle);
-    [fRPCPasswordField setStringValue: [NSString stringWithUTF8String: rpcPassword]];
-    tr_free(rpcPassword);
+    //set rpc password
+    if (fRPCPassword)
+        [fRPCPasswordField setStringValue: fRPCPassword];
 }
 
 - (NSToolbarItem *) toolbar: (NSToolbar *) toolbar itemForItemIdentifier: (NSString *) ident willBeInsertedIntoToolbar: (BOOL) flag
@@ -250,7 +246,7 @@ tr_handle * fHandle;
     if ([ident isEqualToString: TOOLBAR_GENERAL])
     {
         [item setLabel: NSLocalizedString(@"General", "Preferences -> toolbar item title")];
-        [item setImage: [NSImage imageNamed: [NSApp isOnLeopardOrBetter] ? NSImageNamePreferencesGeneral : @"Preferences.png"]];
+        [item setImage: [NSImage imageNamed: NSImageNamePreferencesGeneral]];
         [item setTarget: self];
         [item setAction: @selector(setPrefView:)];
         [item setAutovalidates: NO];
@@ -263,6 +259,14 @@ tr_handle * fHandle;
         [item setAction: @selector(setPrefView:)];
         [item setAutovalidates: NO];
     }
+    else if ([ident isEqualToString: TOOLBAR_GROUPS])
+    {
+        [item setLabel: NSLocalizedString(@"Groups", "Preferences -> toolbar item title")];
+        [item setImage: [NSImage imageNamed: @"Groups.png"]];
+        [item setTarget: self];
+        [item setAction: @selector(setPrefView:)];
+        [item setAutovalidates: NO];
+    }
     else if ([ident isEqualToString: TOOLBAR_BANDWIDTH])
     {
         [item setLabel: NSLocalizedString(@"Bandwidth", "Preferences -> toolbar item title")];
@@ -274,7 +278,7 @@ tr_handle * fHandle;
     else if ([ident isEqualToString: TOOLBAR_PEERS])
     {
         [item setLabel: NSLocalizedString(@"Peers", "Preferences -> toolbar item title")];
-        [item setImage: [NSImage imageNamed: [NSApp isOnLeopardOrBetter] ? NSImageNameUserGroup : @"Peers.png"]];
+        [item setImage: [NSImage imageNamed: NSImageNameUserGroup]];
         [item setTarget: self];
         [item setAction: @selector(setPrefView:)];
         [item setAutovalidates: NO];
@@ -282,7 +286,7 @@ tr_handle * fHandle;
     else if ([ident isEqualToString: TOOLBAR_NETWORK])
     {
         [item setLabel: NSLocalizedString(@"Network", "Preferences -> toolbar item title")];
-        [item setImage: [NSImage imageNamed: [NSApp isOnLeopardOrBetter] ? NSImageNameNetwork : @"Network.png"]];
+        [item setImage: [NSImage imageNamed: NSImageNameNetwork]];
         [item setTarget: self];
         [item setAction: @selector(setPrefView:)];
         [item setAutovalidates: NO];
@@ -316,13 +320,24 @@ tr_handle * fHandle;
 
 - (NSArray *) toolbarAllowedItemIdentifiers: (NSToolbar *) toolbar
 {
-    return [NSArray arrayWithObjects: TOOLBAR_GENERAL, TOOLBAR_TRANSFERS, TOOLBAR_BANDWIDTH,
+    return [NSArray arrayWithObjects: TOOLBAR_GENERAL, TOOLBAR_TRANSFERS, TOOLBAR_GROUPS, TOOLBAR_BANDWIDTH,
                                         TOOLBAR_PEERS, TOOLBAR_NETWORK, TOOLBAR_REMOTE, nil];
 }
 
+//for a beta release, always use the beta appcast
+#if defined(TR_BETA_RELEASE)
+#define SPARKLE_TAG YES
+#else
+#define SPARKLE_TAG [fDefaults boolForKey: @"AutoUpdateBeta"]
+#endif
+- (void) setAutoUpdateToBeta: (id) sender
+{
+    [[SUUpdater sharedUpdater] setAllowedTags: SPARKLE_TAG ? [NSSet setWithObject: @"beta"] : nil];
+}
+
 - (void) setPort: (id) sender
 {
-    int port = [sender intValue];
+    const int port = [sender intValue];
     [fDefaults setInteger: port forKey: @"BindPort"];
     tr_sessionSetPeerPort(fHandle, port);
     
@@ -330,6 +345,19 @@ tr_handle * fHandle;
     [self updatePortStatus];
 }
 
+- (void) randomPort: (id) sender
+{
+    const tr_port port = tr_sessionSetPeerPortRandom(fHandle);
+    
+    [fPortField setIntValue: port];
+    [self setPort: fPortField];
+}
+
+- (void) setRandomPortOnStart: (id) sender
+{
+    tr_sessionSetPeerPortRandomOnStart(fHandle, [sender state] == NSOnState);
+}
+
 - (void) setNat: (id) sender
 {
     tr_sessionSetPortForwardingEnabled(fHandle, [fDefaults boolForKey: @"NatTraversal"]);
@@ -390,25 +418,22 @@ tr_handle * fHandle;
 {
     NSMutableArray * sounds = [NSMutableArray array];
     
-    NSMutableArray * directories = [NSMutableArray arrayWithObjects: @"/System/Library/Sounds", @"/Library/Sounds", nil];
-    if ([NSApp isOnLeopardOrBetter])
-        [directories addObject: [NSHomeDirectory() stringByAppendingPathComponent: @"Library/Sounds"]];
+    NSArray * directories = [NSArray arrayWithObjects: @"/System/Library/Sounds", @"/Library/Sounds", @"Library/Sounds", nil];
     
-    BOOL isDirectory;
-    NSString * directory;
-    NSEnumerator * enumerator = [directories objectEnumerator];
-    while ((directory = [enumerator nextObject]))
+    for (NSString * directory in directories)
+    {
+        BOOL isDirectory;
         if ([[NSFileManager defaultManager] fileExistsAtPath: directory isDirectory: &isDirectory] && isDirectory)
         {
-            NSString * sound;
-            NSEnumerator * soundEnumerator = [[[NSFileManager defaultManager] directoryContentsAtPath: directory] objectEnumerator];
-            while ((sound = [soundEnumerator nextObject]))
+            NSArray * directoryContents = [[NSFileManager defaultManager] directoryContentsAtPath: directory];
+            for (NSString * sound in directoryContents)
             {
                 sound = [sound stringByDeletingPathExtension];
                 if ([NSSound soundNamed: sound])
                     [sounds addObject: sound];
             }
         }
+    }
     
     return sounds;
 }
@@ -423,14 +448,14 @@ tr_handle * fHandle;
 
 - (void) setPeersGlobal: (id) sender
 {
-    int count = [sender intValue];
+    const int count = [sender intValue];
     [fDefaults setInteger: count forKey: @"PeersTotal"];
     tr_sessionSetPeerLimit(fHandle, count);
 }
 
 - (void) setPeersTorrent: (id) sender
 {
-    int count = [sender intValue];
+    const int count = [sender intValue];
     [fDefaults setInteger: count forKey: @"PeersTorrent"];
 }
 
@@ -439,15 +464,21 @@ tr_handle * fHandle;
     tr_sessionSetPexEnabled(fHandle, [fDefaults boolForKey: @"PEXGlobal"]);
 }
 
+- (void) setDHT: (id) sender
+{
+    tr_sessionSetDHTEnabled(fHandle, [fDefaults boolForKey: @"DHTGlobal"]);
+}
+
 - (void) setEncryptionMode: (id) sender
 {
-    tr_sessionSetEncryption(fHandle, [fDefaults boolForKey: @"EncryptionPrefer"] ? 
-        ([fDefaults boolForKey: @"EncryptionRequire"] ? TR_ENCRYPTION_REQUIRED : TR_ENCRYPTION_PREFERRED) : TR_CLEAR_PREFERRED);
+    const tr_encryption_mode mode = [fDefaults boolForKey: @"EncryptionPrefer"] ? 
+        ([fDefaults boolForKey: @"EncryptionRequire"] ? TR_ENCRYPTION_REQUIRED : TR_ENCRYPTION_PREFERRED) : TR_CLEAR_PREFERRED;
+    tr_sessionSetEncryption(fHandle, mode);
 }
 
 - (void) setBlocklistEnabled: (id) sender
 {
-    BOOL enable = [sender state] == NSOnState;
+    const BOOL enable = [sender state] == NSOnState;
     [fDefaults setBool: enable forKey: @"Blocklist"];
     tr_blocklistSetEnabled(fHandle, enable);
     
@@ -466,7 +497,7 @@ tr_handle * fHandle;
 
 - (void) updateBlocklistFields
 {
-    BOOL exists = tr_blocklistExists(fHandle);
+    const BOOL exists = tr_blocklistExists(fHandle);
     
     if (exists)
     {
@@ -511,35 +542,34 @@ tr_handle * fHandle;
 
 - (void) applySpeedSettings: (id) sender
 {
-    if ([fDefaults boolForKey: @"SpeedLimit"])
-    {
-        tr_sessionSetSpeedLimitEnabled(fHandle, TR_UP, 1);
-        tr_sessionSetSpeedLimit(fHandle, TR_UP, [fDefaults integerForKey: @"SpeedLimitUploadLimit"]);
+    tr_sessionLimitSpeed(fHandle, TR_UP, [fDefaults boolForKey: @"CheckUpload"]);
+    tr_sessionSetSpeedLimit(fHandle, TR_UP, [fDefaults integerForKey: @"UploadLimit"]);
+    
+    tr_sessionLimitSpeed(fHandle, TR_DOWN, [fDefaults boolForKey: @"CheckDownload"]);
+    tr_sessionSetSpeedLimit(fHandle, TR_DOWN, [fDefaults integerForKey: @"DownloadLimit"]);
+    
+    [[NSNotificationCenter defaultCenter] postNotificationName: @"SpeedLimitUpdate" object: nil];
+}
+
+- (void) applyAltSpeedSettings
+{
+    tr_sessionSetAltSpeed(fHandle, TR_UP, [fDefaults integerForKey: @"SpeedLimitUploadLimit"]);
+    tr_sessionSetAltSpeed(fHandle, TR_DOWN, [fDefaults integerForKey: @"SpeedLimitDownloadLimit"]);
         
-        tr_sessionSetSpeedLimitEnabled(fHandle, TR_DOWN, 1);
-        tr_sessionSetSpeedLimit(fHandle, TR_DOWN, [fDefaults integerForKey: @"SpeedLimitDownloadLimit"]);
-    }
-    else
-    {
-        tr_sessionSetSpeedLimitEnabled(fHandle, TR_UP, [fDefaults boolForKey: @"CheckUpload"]);
-        tr_sessionSetSpeedLimit(fHandle, TR_UP, [fDefaults integerForKey: @"UploadLimit"]);
-        
-        tr_sessionSetSpeedLimitEnabled(fHandle, TR_DOWN, [fDefaults boolForKey: @"CheckDownload"]);
-        tr_sessionSetSpeedLimit(fHandle, TR_DOWN, [fDefaults integerForKey: @"DownloadLimit"]);
-    }
+    [[NSNotificationCenter defaultCenter] postNotificationName: @"SpeedLimitUpdate" object: nil];
 }
 
 - (void) applyRatioSetting: (id) sender
 {
-    [[NSNotificationCenter defaultCenter] postNotificationName: @"UpdateUI" object: nil];
+    //[[NSNotificationCenter defaultCenter] postNotificationName: @"UpdateUI" object: nil];
+    tr_sessionSetRatioLimited(fHandle, [fDefaults boolForKey: @"RatioCheck"]);
+    tr_sessionSetRatioLimit(fHandle, [fDefaults floatForKey: @"RatioLimit"]);
 }
 
 - (void) updateRatioStopField
 {
-    if (!fHasLoaded)
-        return;
-    
-    [fRatioStopField setFloatValue: [fDefaults floatForKey: @"RatioLimit"]];
+    if (fHasLoaded)
+        [fRatioStopField setFloatValue: [fDefaults floatForKey: @"RatioLimit"]];
     
     [self applyRatioSetting: nil];
 }
@@ -547,6 +577,7 @@ tr_handle * fHandle;
 - (void) setRatioStop: (id) sender
 {
     [fDefaults setFloat: [sender floatValue] forKey: @"RatioLimit"];
+    
     [self applyRatioSetting: nil];
 }
 
@@ -569,12 +600,39 @@ tr_handle * fHandle;
 {
     [fDefaults setInteger: [sender intValue] forKey: sender == fSpeedLimitUploadField
                                                         ? @"SpeedLimitUploadLimit" : @"SpeedLimitDownloadLimit"];
-    [self applySpeedSettings: self];
+    [self applyAltSpeedSettings];
 }
 
 - (void) setAutoSpeedLimit: (id) sender
 {
-    [[NSNotificationCenter defaultCenter] postNotificationName: @"AutoSpeedLimitChange" object: self];
+    tr_sessionUseAltSpeedTime(fHandle, [fDefaults boolForKey: @"SpeedLimitAuto"]);
+}
+
+- (void) setAutoSpeedLimitTime: (id) sender
+{
+    tr_sessionSetAltSpeedBegin(fHandle, [PrefsController dateToTimeSum: [fDefaults objectForKey: @"SpeedLimitAutoOnDate"]]);
+    tr_sessionSetAltSpeedEnd(fHandle, [PrefsController dateToTimeSum: [fDefaults objectForKey: @"SpeedLimitAutoOffDate"]]);
+}
+
+- (void) setAutoSpeedLimitDay: (id) sender
+{
+    tr_sessionSetAltSpeedDay(fHandle, [[sender selectedItem] tag]);
+}
+
++ (NSInteger) dateToTimeSum: (NSDate *) date
+{
+    NSCalendar * calendar = [NSCalendar currentCalendar];
+    NSDateComponents * components = [calendar components: NSHourCalendarUnit | NSMinuteCalendarUnit fromDate: date];
+    return [components hour] * 60 + [components minute];
+}
+
++ (NSDate *) timeSumToDate: (NSInteger) sum
+{
+    NSDateComponents * comps = [[[NSDateComponents alloc] init] autorelease];
+    [comps setHour: sum / 60];
+    [comps setMinute: sum % 60];
+    
+    return [[NSCalendar currentCalendar] dateFromComponents: comps];
 }
 
 - (BOOL) control: (NSControl *) control textShouldBeginEditing: (NSText *) fieldEditor
@@ -807,7 +865,10 @@ tr_handle * fHandle;
 
 - (void) setRPCEnabled: (id) sender
 {
-    tr_sessionSetRPCEnabled(fHandle, [fDefaults boolForKey: @"RPC"]);
+    BOOL enable = [fDefaults boolForKey: @"RPC"];
+    tr_sessionSetRPCEnabled(fHandle, enable);
+    
+    [self setRPCWebUIDiscovery: nil];
 }
 
 - (void) linkWebUI: (id) sender
@@ -828,6 +889,9 @@ tr_handle * fHandle;
 
 - (void) setRPCPassword: (id) sender
 {
+    [fRPCPassword release];
+    fRPCPassword = [[sender stringValue] copy];
+    
     const char * password = [[sender stringValue] UTF8String];
     [self setKeychainPassword: password forService: RPC_KEYCHAIN_SERVICE username: RPC_KEYCHAIN_NAME];
     
@@ -841,6 +905,7 @@ tr_handle * fHandle;
     SecKeychainFindGenericPassword(NULL, strlen(RPC_KEYCHAIN_SERVICE), RPC_KEYCHAIN_SERVICE,
         strlen(RPC_KEYCHAIN_NAME), RPC_KEYCHAIN_NAME, &passwordLength, (void **)&password, NULL);
     
+    [fRPCPassword release];
     if (password != NULL)
     {
         char fullPassword[passwordLength+1];
@@ -849,8 +914,12 @@ tr_handle * fHandle;
         SecKeychainItemFreeContent(NULL, (void *)password);
         
         tr_sessionSetRPCPassword(fHandle, fullPassword);
-        [fRPCPasswordField setStringValue: [NSString stringWithUTF8String: fullPassword]];
+        
+        fRPCPassword = [[NSString alloc] initWithUTF8String: fullPassword];
+        [fRPCPasswordField setStringValue: fRPCPassword];
     }
+    else
+        fRPCPassword = nil;
 }
 
 - (void) setRPCPort: (id) sender
@@ -858,6 +927,8 @@ tr_handle * fHandle;
     int port = [sender intValue];
     [fDefaults setInteger: port forKey: @"RPCPort"];
     tr_sessionSetRPCPort(fHandle, port);
+    
+    [self setRPCWebUIDiscovery: nil];
 }
 
 - (void) setRPCUseWhitelist: (id) sender
@@ -865,6 +936,14 @@ tr_handle * fHandle;
     tr_sessionSetRPCWhitelistEnabled(fHandle, [fDefaults boolForKey: @"RPCUseWhitelist"]);
 }
 
+- (void) setRPCWebUIDiscovery: (id) sender
+{
+    if ([fDefaults boolForKey:@"RPC"] && [fDefaults boolForKey: @"RPCWebDiscovery"])
+        [[BonjourController defaultController] startWithPort: [fDefaults integerForKey: @"RPCPort"]];
+    else
+        [[BonjourController defaultController] stop];
+}
+
 - (void) updateRPCWhitelist
 {
     NSString * string = [fRPCWhitelistArray componentsJoinedByString: @","];
@@ -918,9 +997,7 @@ tr_handle * fHandle;
     if ([components count] == 4)
     {
         valid = true;
-        NSEnumerator * enumerator = [components objectEnumerator];
-        NSString * component;
-        while ((component = [enumerator nextObject]))
+        for (NSString * component in components)
         {
             if ([component isEqualToString: @"*"])
                 [newComponents addObject: component];
@@ -993,7 +1070,7 @@ tr_handle * fHandle;
 - (void) rpcUpdatePrefs
 {
     //encryption
-    tr_encryption_mode encryptionMode = tr_sessionGetEncryption(fHandle);
+    const tr_encryption_mode encryptionMode = tr_sessionGetEncryption(fHandle);
     [fDefaults setBool: encryptionMode != TR_CLEAR_PREFERRED forKey: @"EncryptionPrefer"];
     [fDefaults setBool: encryptionMode == TR_ENCRYPTION_REQUIRED forKey: @"EncryptionRequire"];
     
@@ -1002,38 +1079,84 @@ tr_handle * fHandle;
     [fDefaults setObject: downloadLocation forKey: @"DownloadFolder"];
     
     //peers
-    uint16_t peersTotal = tr_sessionGetPeerLimit(fHandle);
+    const uint16_t peersTotal = tr_sessionGetPeerLimit(fHandle);
     [fDefaults setInteger: peersTotal forKey: @"PeersTotal"];
     
+    const uint16_t peersTorrent = tr_sessionGetPeerLimitPerTorrent(fHandle);
+    [fDefaults setInteger: peersTotal forKey: @"PeersTorrent"];
+    
     //pex
-    BOOL pex = tr_sessionIsPexEnabled(fHandle);
+    const BOOL pex = tr_sessionIsPexEnabled(fHandle);
     [fDefaults setBool: pex forKey: @"PEXGlobal"];
     
+    //dht
+    const BOOL dht = tr_sessionIsDHTEnabled(fHandle);
+    [fDefaults setBool: dht forKey: @"DHTGlobal"];
+    
     //port
-    int port = tr_sessionGetPeerPort(fHandle);
+    const tr_port port = tr_sessionGetPeerPort(fHandle);
     [fDefaults setInteger: port forKey: @"BindPort"];
     
-    BOOL nat = tr_sessionIsPortForwardingEnabled(fHandle);
+    const BOOL nat = tr_sessionIsPortForwardingEnabled(fHandle);
     [fDefaults setBool: nat forKey: @"NatTraversal"];
     
     fPeerPort = -1;
     fNatStatus = -1;
     [self updatePortStatus];
     
+    const BOOL randomPort = tr_sessionGetPeerPortRandomOnStart(fHandle);
+    [fDefaults setBool: randomPort forKey: @"RandomPort"];
+    
     //speed limit - down
-    BOOL downLimitEnabled = tr_sessionIsSpeedLimitEnabled(fHandle, TR_DOWN);
+    const BOOL downLimitEnabled = tr_sessionIsSpeedLimited(fHandle, TR_DOWN);
     [fDefaults setBool: downLimitEnabled forKey: @"CheckDownload"];
     
-    int downLimit = tr_sessionGetSpeedLimit(fHandle, TR_DOWN);
+    const int downLimit = tr_sessionGetSpeedLimit(fHandle, TR_DOWN);
     [fDefaults setInteger: downLimit forKey: @"DownloadLimit"];
     
     //speed limit - up
-    BOOL upLimitEnabled = tr_sessionIsSpeedLimitEnabled(fHandle, TR_UP);
+    const BOOL upLimitEnabled = tr_sessionIsSpeedLimited(fHandle, TR_UP);
     [fDefaults setBool: upLimitEnabled forKey: @"CheckUpload"];
     
-    int upLimit = tr_sessionGetSpeedLimit(fHandle, TR_UP);
+    const int upLimit = tr_sessionGetSpeedLimit(fHandle, TR_UP);
     [fDefaults setInteger: upLimit forKey: @"UploadLimit"];
     
+    //alt speed limit enabled
+    const BOOL useAltSpeed = tr_sessionUsesAltSpeed(fHandle);
+    [fDefaults setBool: useAltSpeed forKey: @"SpeedLimit"];
+    
+    //alt speed limit - down
+    const int downLimitAlt = tr_sessionGetAltSpeed(fHandle, TR_DOWN);
+    [fDefaults setInteger: downLimit forKey: @"SpeedLimitDownloadLimit"];
+    
+    //alt speed limit - up
+    const int upLimitAlt = tr_sessionGetAltSpeed(fHandle, TR_UP);
+    [fDefaults setInteger: upLimitAlt forKey: @"SpeedLimitDownloadLimit"];
+    
+    //alt speed limit schedule
+    const BOOL useAltSpeedSched = tr_sessionUsesAltSpeedTime(fHandle);
+    [fDefaults setBool: useAltSpeedSched forKey: @"SpeedLimitAuto"];
+    
+    NSDate * limitStartDate = [PrefsController timeSumToDate: tr_sessionGetAltSpeedBegin(fHandle)];
+    [fDefaults setObject: limitStartDate forKey: @"SpeedLimitAutoOnDate"];
+    
+    NSDate * limitEndDate = [PrefsController timeSumToDate: tr_sessionGetAltSpeedEnd(fHandle)];
+    [fDefaults setObject: limitEndDate forKey: @"SpeedLimitAutoOffDate"];
+    
+    const int limitDay = tr_sessionGetAltSpeedDay(fHandle);
+    [fDefaults setInteger: limitDay forKey: @"SpeedLimitAutoDay"];
+    
+    //blocklist
+    const BOOL blocklist = tr_blocklistIsEnabled(fHandle);
+    [fDefaults setBool: blocklist forKey: @"Blocklist"];
+    
+    //seed ratio
+    const BOOL ratioLimited = tr_sessionIsRatioLimited(fHandle);
+    [fDefaults setBool: ratioLimited forKey: @"RatioCheck"];
+    
+    const float ratioLimit = tr_sessionGetRatioLimit(fHandle);
+    [fDefaults setFloat: ratioLimit forKey: @"RatioLimit"];
+    
     //update gui if loaded
     if (fHasLoaded)
     {
@@ -1042,18 +1165,37 @@ tr_handle * fHandle;
         //download directory handled by bindings
         
         [fPeersGlobalField setIntValue: peersTotal];
+        [fPeersTorrentField setIntValue: peersTorrent];
         
         //pex handled by bindings
         
+        //dht handled by bindings
+        
         [fPortField setIntValue: port];
         //port forwarding (nat) handled by bindings
+        //random port handled by bindings
         
         //limit check handled by bindings
         [fDownloadField setIntValue: downLimit];
         
         //limit check handled by bindings
         [fUploadField setIntValue: upLimit];
+        
+        [fSpeedLimitDownloadField setIntValue: downLimitAlt];
+        
+        [fSpeedLimitUploadField setIntValue: upLimitAlt];
+        
+        //speed limit schedule handled by bindings
+        
+        //speed limit schedule times and day handled by bindings
+        
+        [self updateBlocklistFields];
+        
+        //ratio limit enabled handled by bindings
+        [fRatioStopField setFloatValue: ratioLimit];
     }
+    
+    [[NSNotificationCenter defaultCenter] postNotificationName: @"SpeedLimitUpdate" object: nil];
 }
 
 @end
@@ -1062,22 +1204,35 @@ tr_handle * fHandle;
 
 - (void) setPrefView: (id) sender
 {
-    NSView * view = fGeneralView;
+    NSString * identifier;
     if (sender)
     {
-        NSString * identifier = [sender itemIdentifier];
-        if ([identifier isEqualToString: TOOLBAR_TRANSFERS])
-            view = fTransfersView;
-        else if ([identifier isEqualToString: TOOLBAR_BANDWIDTH])
-            view = fBandwidthView;
-        else if ([identifier isEqualToString: TOOLBAR_PEERS])
-            view = fPeersView;
-        else if ([identifier isEqualToString: TOOLBAR_NETWORK])
-            view = fNetworkView;
-        else if ([identifier isEqualToString: TOOLBAR_REMOTE])
-            view = fRemoteView;
-        else; //general view already selected
+        identifier = [sender itemIdentifier];
+        [[NSUserDefaults standardUserDefaults] setObject: identifier forKey: @"SelectedPrefView"];
     }
+    else
+        identifier = [[NSUserDefaults standardUserDefaults] stringForKey: @"SelectedPrefView"];
+    
+    NSView * view;
+    if ([identifier isEqualToString: TOOLBAR_TRANSFERS])
+        view = fTransfersView;
+    else if ([identifier isEqualToString: TOOLBAR_GROUPS])
+        view = fGroupsView;
+    else if ([identifier isEqualToString: TOOLBAR_BANDWIDTH])
+        view = fBandwidthView;
+    else if ([identifier isEqualToString: TOOLBAR_PEERS])
+        view = fPeersView;
+    else if ([identifier isEqualToString: TOOLBAR_NETWORK])
+        view = fNetworkView;
+    else if ([identifier isEqualToString: TOOLBAR_REMOTE])
+        view = fRemoteView;
+    else
+    {
+        identifier = TOOLBAR_GENERAL; //general view is the default selected
+        view = fGeneralView;
+    }
+    
+    [[[self window] toolbar] setSelectedItemIdentifier: identifier];
     
     NSWindow * window = [self window];
     if ([window contentView] == view)
@@ -1100,19 +1255,13 @@ tr_handle * fHandle;
     {
         NSToolbar * toolbar = [window toolbar];
         NSString * itemIdentifier = [toolbar selectedItemIdentifier];
-        NSEnumerator * enumerator = [[toolbar items] objectEnumerator];
-        NSToolbarItem * item;
-        while ((item = [enumerator nextObject]))
+        for (NSToolbarItem * item in [toolbar items])
             if ([[item itemIdentifier] isEqualToString: itemIdentifier])
             {
                 [window setTitle: [item label]];
                 break;
             }
     }
-    
-    //for network view make sure progress indicator hides itself (get around a Tiger bug)
-    if (![NSApp isOnLeopardOrBetter] && view == fNetworkView && [fPortStatusImage image])
-        [fPortStatusProgress setDisplayedWhenStopped: NO];
 }
 
 - (void) folderSheetClosed: (NSOpenPanel *) openPanel returnCode: (int) code contextInfo: (void *) info
@@ -1120,8 +1269,12 @@ tr_handle * fHandle;
     if (code == NSOKButton)
     {
         [fFolderPopUp selectItemAtIndex: DOWNLOAD_FOLDER];
-        [fDefaults setObject: [[openPanel filenames] objectAtIndex: 0] forKey: @"DownloadFolder"];
+        
+        NSString * folder = [[openPanel filenames] objectAtIndex: 0];
+        [fDefaults setObject: folder forKey: @"DownloadFolder"];
         [fDefaults setObject: @"Constant" forKey: @"DownloadChoice"];
+        
+        tr_sessionSetDownloadDir(fHandle, [folder UTF8String]);
     }
     else
     {
