@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: Torrent.m 7666 2009-01-11 18:04:45Z livings124 $
+ * $Id: Torrent.m 8667 2009-06-10 22:56:24Z livings124 $
  *
  * Copyright (c) 2006-2009 Transmission authors and contributors
  *
@@ -25,19 +25,17 @@
 #import "Torrent.h"
 #import "GroupsController.h"
 #import "FileListNode.h"
-#import "NSApplicationAdditions.h"
 #import "NSStringAdditions.h"
 #import "utils.h" //tr_httpIsValidURL
 
 @interface Torrent (Private)
 
-- (id) initWithHash: (NSString *) hashString path: (NSString *) path torrentStruct: (tr_torrent *) torrentStruct lib: (tr_session *) lib
-        publicTorrent: (NSNumber *) publicTorrent
+- (id) initWithPath: (NSString *) path hash: (NSString *) hashString torrentStruct: (tr_torrent *) torrentStruct lib: (tr_session *) lib
+        publicTorrent: (NSNumber *) publicTorrent publicTorrentLocation: (NSString *) publicTorrentLoc
         downloadFolder: (NSString *) downloadFolder
         useIncompleteFolder: (NSNumber *) useIncompleteFolder incompleteFolder: (NSString *) incompleteFolder
-        ratioSetting: (NSNumber *) ratioSetting ratioLimit: (NSNumber *) ratioLimit
         waitToStart: (NSNumber *) waitToStart
-        orderValue: (NSNumber *) orderValue groupValue: (NSNumber *) groupValue addedTrackers: (NSNumber *) addedTrackers;
+        groupValue: (NSNumber *) groupValue addedTrackers: (NSNumber *) addedTrackers;
 
 - (BOOL) shouldUseIncompleteFolderForName: (NSString *) name;
 - (void) updateDownloadFolder;
@@ -48,10 +46,12 @@
 
 - (void) completenessChange: (NSNumber *) status;
 
+- (void) ratioLimitHit;
+
 - (void) quickPause;
 - (void) endQuickPause;
 
-- (NSString *) etaString: (NSInteger) eta;
+- (NSString *) etaString;
 
 - (void) updateAllTrackers: (NSMutableArray *) trackers;
 
@@ -64,7 +64,12 @@
 void completenessChangeCallback(tr_torrent * torrent, tr_completeness status, void * torrentData)
 {
     [(Torrent *)torrentData performSelectorOnMainThread: @selector(completenessChange:)
-                withObject: [[NSNumber alloc] initWithInt: status] waitUntilDone: NO];
+        withObject: [[NSNumber alloc] initWithInt: status] waitUntilDone: NO];
+}
+
+void ratioLimitHitCallback(tr_torrent * torrent, void * torrentData)
+{
+    [(Torrent *)torrentData performSelectorOnMainThread: @selector(ratioLimitHit) withObject: nil waitUntilDone: NO];
 }
 
 int trashDataFile(const char * filename)
@@ -78,12 +83,12 @@ int trashDataFile(const char * filename)
 - (id) initWithPath: (NSString *) path location: (NSString *) location deleteTorrentFile: (torrentFileState) torrentDelete
         lib: (tr_session *) lib
 {
-    self = [self initWithHash: nil path: path torrentStruct: NULL lib: lib
+    self = [self initWithPath: path hash: nil torrentStruct: NULL lib: lib
             publicTorrent: torrentDelete != TORRENT_FILE_DEFAULT ? [NSNumber numberWithBool: torrentDelete == TORRENT_FILE_SAVE] : nil
+            publicTorrentLocation: path
             downloadFolder: location
             useIncompleteFolder: nil incompleteFolder: nil
-            ratioSetting: nil ratioLimit: nil
-            waitToStart: nil orderValue: nil groupValue: nil addedTrackers: nil];
+            waitToStart: nil groupValue: nil addedTrackers: nil];
     
     if (self)
     {
@@ -103,28 +108,26 @@ int trashDataFile(const char * filename)
 
 - (id) initWithTorrentStruct: (tr_torrent *) torrentStruct location: (NSString *) location lib: (tr_session *) lib
 {
-    self = [self initWithHash: nil path: nil torrentStruct: torrentStruct lib: lib
-            publicTorrent: [NSNumber numberWithBool: NO]
+    self = [self initWithPath: nil hash: nil torrentStruct: torrentStruct lib: lib
+            publicTorrent: [NSNumber numberWithBool: NO] publicTorrentLocation: nil
             downloadFolder: location
             useIncompleteFolder: nil incompleteFolder: nil
-            ratioSetting: nil ratioLimit: nil
-            waitToStart: nil orderValue: nil groupValue: nil addedTrackers: nil];
+            waitToStart: nil groupValue: nil addedTrackers: nil];
     
     return self;
 }
 
-- (id) initWithHistory: (NSDictionary *) history lib: (tr_session *) lib
+- (id) initWithHistory: (NSDictionary *) history lib: (tr_session *) lib forcePause: (BOOL) pause
 {
-    self = [self initWithHash: [history objectForKey: @"TorrentHash"]
-                path: [history objectForKey: @"TorrentPath"] torrentStruct: NULL lib: lib
+    self = [self initWithPath: [history objectForKey: @"InternalTorrentPath"]
+                hash: [history objectForKey: @"TorrentHash"]
+                torrentStruct: NULL lib: lib
                 publicTorrent: [history objectForKey: @"PublicCopy"]
+                publicTorrentLocation: [history objectForKey: @"TorrentPath"]
                 downloadFolder: [history objectForKey: @"DownloadFolder"]
                 useIncompleteFolder: [history objectForKey: @"UseIncompleteFolder"]
                 incompleteFolder: [history objectForKey: @"IncompleteFolder"]
-                ratioSetting: [history objectForKey: @"RatioSetting"]
-                ratioLimit: [history objectForKey: @"RatioLimit"]
                 waitToStart: [history objectForKey: @"WaitToStart"]
-                orderValue: [history objectForKey: @"OrderValue"]
                 groupValue: [history objectForKey: @"GroupValue"]
                 addedTrackers: [history objectForKey: @"AddedTrackers"]];
     
@@ -132,7 +135,7 @@ int trashDataFile(const char * filename)
     {
         //start transfer
         NSNumber * active;
-        if ((active = [history objectForKey: @"Active"]) && [active boolValue])
+        if (!pause && (active = [history objectForKey: @"Active"]) && [active boolValue])
         {
             fStat = tr_torrentStat(fHandle);
             [self startTransfer];
@@ -146,6 +149,21 @@ int trashDataFile(const char * filename)
             tr_torrentSetActivityDate(fHandle, [date timeIntervalSince1970]);
         if ((date = [history objectForKey: @"DateCompleted"]))
             tr_torrentSetDoneDate(fHandle, [date timeIntervalSince1970]);
+        
+        //upgrading from versions < 1.60: get old stop ratio settings
+        NSNumber * ratioSetting;
+        if ((ratioSetting = [history objectForKey: @"RatioSetting"]))
+        {
+            switch ([ratioSetting intValue])
+            {
+                case NSOnState: [self setRatioSetting: TR_RATIOLIMIT_SINGLE]; break;
+                case NSOffState: [self setRatioSetting: TR_RATIOLIMIT_UNLIMITED]; break;
+                case NSMixedState: [self setRatioSetting: TR_RATIOLIMIT_GLOBAL]; break;
+            }
+        }
+        NSNumber * ratioLimit;
+        if ((ratioLimit = [history objectForKey: @"RatioLimit"]))
+            [self setRatioLimit: [ratioLimit floatValue]];
     }
     return self;
 }
@@ -153,15 +171,13 @@ int trashDataFile(const char * filename)
 - (NSDictionary *) history
 {
     NSMutableDictionary * history = [NSMutableDictionary dictionaryWithObjectsAndKeys:
-                    [NSNumber numberWithBool: fPublicTorrent], @"PublicCopy",
+                    [self torrentLocation], @"InternalTorrentPath",
                     [self hashString], @"TorrentHash",
+                    [NSNumber numberWithBool: fPublicTorrent], @"PublicCopy",
                     fDownloadFolder, @"DownloadFolder",
                     [NSNumber numberWithBool: fUseIncompleteFolder], @"UseIncompleteFolder",
                     [NSNumber numberWithBool: [self isActive]], @"Active",
-                    [NSNumber numberWithInt: fRatioSetting], @"RatioSetting",
-                    [NSNumber numberWithFloat: fRatioLimit], @"RatioLimit",
                     [NSNumber numberWithBool: fWaitToStart], @"WaitToStart",
-                    [NSNumber numberWithInt: fOrderValue], @"OrderValue",
                     [NSNumber numberWithInt: fGroupValue], @"GroupValue",
                     [NSNumber numberWithBool: fAddedTrackers], @"AddedTrackers", nil];
     
@@ -282,19 +298,6 @@ int trashDataFile(const char * filename)
     
     fStat = tr_torrentStat(fHandle);
     
-    //check to stop for ratio
-    CGFloat stopRatio;
-    if ([self isSeeding] && (stopRatio = [self actualStopRatio]) != INVALID && [self ratio] >= stopRatio)
-    {
-        [self setRatioSetting: NSOffState];
-        [[NSNotificationCenter defaultCenter] postNotificationName: @"TorrentStoppedForRatio" object: self];
-        
-        [self stopTransfer];
-        fStat = tr_torrentStat(fHandle);
-        
-        fFinishedSeeding = YES;
-    }
-    
     //check if stalled (stored because based on time and needs to check if it was previously stalled)
     fStalled = [self isActive] && [fDefaults boolForKey: @"CheckStalled"]
                 && [self stalledMinutes] > [fDefaults integerForKey: @"StalledMinutes"];
@@ -362,56 +365,45 @@ int trashDataFile(const char * filename)
     return fStat->ratio;
 }
 
-- (NSInteger) ratioSetting
+- (tr_ratiolimit) ratioSetting
 {
-    return fRatioSetting;
+    return tr_torrentGetRatioMode(fHandle);
 }
 
-- (void) setRatioSetting: (NSInteger) setting
+- (void) setRatioSetting: (tr_ratiolimit) setting
 {
-    fRatioSetting = setting;
+    tr_torrentSetRatioMode(fHandle, setting);
 }
 
 - (CGFloat) ratioLimit
 {
-    return fRatioLimit;
+    return tr_torrentGetRatioLimit(fHandle);
 }
 
 - (void) setRatioLimit: (CGFloat) limit
 {
-    if (limit >= 0)
-        fRatioLimit = limit;
+    NSAssert(limit >= 0, @"Ratio cannot be negative");
+    tr_torrentSetRatioLimit(fHandle, limit);
 }
 
-- (CGFloat) actualStopRatio
+- (BOOL) seedRatioSet
 {
-    if (fRatioSetting == NSOnState)
-        return fRatioLimit;
-    else if (fRatioSetting == NSMixedState && [fDefaults boolForKey: @"RatioCheck"])
-        return [fDefaults floatForKey: @"RatioLimit"];
-    else
-        return INVALID;
+    return tr_torrentGetSeedRatio(fHandle, NULL);
 }
 
 - (CGFloat) progressStopRatio
 {
-    CGFloat stopRatio, ratio;
-    if ((stopRatio = [self actualStopRatio]) == INVALID || (ratio = [self ratio]) >= stopRatio)
-        return 1.0;
-    else if (stopRatio > 0.0)
-        return ratio / stopRatio;
-    else
-        return 0.0;
+    return fStat->percentRatio;
 }
 
-- (tr_speedlimit) speedMode: (BOOL) upload
+- (BOOL) usesSpeedLimit: (BOOL) upload
 {
-    return tr_torrentGetSpeedMode(fHandle, upload ? TR_UP : TR_DOWN);
+    return tr_torrentUsesSpeedLimit(fHandle, upload ? TR_UP : TR_DOWN);
 }
 
-- (void) setSpeedMode: (tr_speedlimit) mode upload: (BOOL) upload
+- (void) setUseSpeedLimit: (BOOL) use upload: (BOOL) upload
 {
-    tr_torrentSetSpeedMode(fHandle, upload ? TR_UP : TR_DOWN, mode);
+    tr_torrentUseSpeedLimit(fHandle, upload ? TR_UP : TR_DOWN, use);
 }
 
 - (NSInteger) speedLimit: (BOOL) upload
@@ -424,6 +416,16 @@ int trashDataFile(const char * filename)
     tr_torrentSetSpeedLimit(fHandle, upload ? TR_UP : TR_DOWN, limit);
 }
 
+- (BOOL) usesGlobalSpeedLimit
+{
+    return tr_torrentUsesSessionLimits(fHandle);
+}
+
+- (void) setUseGlobalSpeedLimit: (BOOL) use
+{
+    tr_torrentUseSessionLimits(fHandle, use);
+}
+
 - (void) setMaxPeerConnect: (uint16_t) count
 {
     NSAssert(count > 0, @"max peer count must be greater than 0");
@@ -446,6 +448,16 @@ int trashDataFile(const char * filename)
     return fWaitToStart;
 }
 
+- (tr_priority_t) priority
+{
+    return tr_torrentGetPriority(fHandle);
+}
+
+- (void) setPriority: (tr_priority_t) priority
+{
+    return tr_torrentSetPriority(fHandle, priority);
+}
+
 - (void) revealData
 {
     [[NSWorkspace sharedWorkspace] selectFile: [self dataLocation] inFileViewerRootedAtPath: nil];
@@ -556,10 +568,7 @@ int trashDataFile(const char * filename)
     NSString * volumeName;
     if ((volumeName = [[fileManager componentsToDisplayForPath: downloadFolder] objectAtIndex: 0]))
     {
-        BOOL onLeopard = [NSApp isOnLeopardOrBetter];
-        
-        NSDictionary * systemAttributes = onLeopard ? [fileManager attributesOfFileSystemForPath: downloadFolder error: NULL]
-                                            : [fileManager fileSystemAttributesAtPath: downloadFolder];
+        NSDictionary * systemAttributes = [fileManager attributesOfFileSystemForPath: downloadFolder error: NULL];
         uint64_t remainingSpace = [[systemAttributes objectForKey: NSFileSystemFreeSize] unsignedLongLongValue];
         
         //if the remaining space is greater than the size left, then there is enough space regardless of preallocation
@@ -575,17 +584,12 @@ int trashDataFile(const char * filename)
             [alert addButtonWithTitle: NSLocalizedString(@"OK", "Torrent disk space alert -> button")];
             [alert addButtonWithTitle: NSLocalizedString(@"Download Anyway", "Torrent disk space alert -> button")];
             
-            if (onLeopard)
-            {
-                [alert setShowsSuppressionButton: YES];
-                [[alert suppressionButton] setTitle: NSLocalizedString(@"Do not check disk space again",
-                                                        "Torrent disk space alert -> button")];
-            }
-            else
-                [alert addButtonWithTitle: NSLocalizedString(@"Always Download", "Torrent disk space alert -> button")];
+            [alert setShowsSuppressionButton: YES];
+            [[alert suppressionButton] setTitle: NSLocalizedString(@"Do not check disk space again",
+                                                    "Torrent disk space alert -> button")];
 
             NSInteger result = [alert runModal];
-            if ((onLeopard ? [[alert suppressionButton] state] == NSOnState : result == NSAlertThirdButtonReturn))
+            if ([[alert suppressionButton] state] == NSOnState)
                 [fDefaults setBool: NO forKey: @"WarningRemainingSpace"];
             [alert release];
             
@@ -769,7 +773,7 @@ int trashDataFile(const char * filename)
 - (NSMutableArray *) allTrackers: (BOOL) separators
 {
     const NSInteger count = fInfo->trackerCount;
-    const NSInteger capacity = separators ? count + fInfo->trackers[count-1].tier + 1 : count;
+    const NSInteger capacity = (separators && count > 0) ? count + fInfo->trackers[count-1].tier + 1 : count;
     NSMutableArray * allTrackers = [NSMutableArray arrayWithCapacity: capacity];
     
     for (NSInteger i = 0, tier = -1; i < count; i++)
@@ -914,24 +918,6 @@ int trashDataFile(const char * filename)
     return fStat->eta;
 }
 
-- (NSInteger) etaRatio
-{
-    if (![self isSeeding])
-        return TR_ETA_UNKNOWN;
-    
-    CGFloat uploadRate = [self uploadRate];
-    if (uploadRate < 0.1)
-        return TR_ETA_UNKNOWN;
-    
-    CGFloat stopRatio = [self actualStopRatio], ratio = [self ratio];
-    if (stopRatio == INVALID || ratio >= stopRatio)
-        return TR_ETA_UNKNOWN;
-    
-    CGFloat haveDownloaded = (CGFloat)([self downloadedTotal] > 0 ? [self downloadedTotal] : [self haveVerified]);
-    CGFloat needUploaded = haveDownloaded * (stopRatio - ratio);
-    return needUploaded / uploadRate / 1024.0;
-}
-
 - (CGFloat) notAvailableDesired
 {
     return 1.0 - (CGFloat)fStat->desiredAvailable / [self sizeLeft];
@@ -959,7 +945,7 @@ int trashDataFile(const char * filename)
 
 - (BOOL) allDownloaded
 {
-    return [self progressDone] >= 1.0;
+    return [self sizeLeft] == 0;
 }
 
 - (BOOL) isComplete
@@ -1097,12 +1083,8 @@ int trashDataFile(const char * filename)
     }
     
     //add time when downloading
-    if (fStat->activity == TR_STATUS_DOWNLOAD || ([self isSeeding]
-        && (fRatioSetting == NSOnState || (fRatioSetting == NSMixedState && [fDefaults boolForKey: @"RatioCheck"]))))
-    {
-        NSInteger eta = fStat->activity == TR_STATUS_DOWNLOAD ? [self eta] : [self etaRatio];
-        string = [string stringByAppendingFormat: @" - %@", [self etaString: eta]];
-    }
+    if (fStat->activity == TR_STATUS_DOWNLOAD || ([self isSeeding] && [self seedRatioSet]))
+        string = [string stringByAppendingFormat: @" - %@", [self etaString]];
     
     return string;
 }
@@ -1240,11 +1222,10 @@ int trashDataFile(const char * filename)
 
 - (NSString *) remainingTimeString
 {
-    if (![self isActive] || ([self isSeeding]
-        && !(fRatioSetting == NSOnState || (fRatioSetting == NSMixedState && [fDefaults boolForKey: @"RatioCheck"]))))
+    if (fStat->activity == TR_STATUS_DOWNLOAD || ([self isSeeding] && [self seedRatioSet]))
+        return [self etaString];
+    else
         return [self shortStatusString];
-    
-    return [self etaString: [self isSeeding] ? [self etaRatio] : [self eta]];
 }
 
 - (NSString *) stateString
@@ -1309,6 +1290,11 @@ int trashDataFile(const char * filename)
     return fStat->peersFrom[TR_PEER_FROM_PEX];
 }
 
+- (NSInteger) totalPeersDHT
+{
+    return fStat->peersFrom[TR_PEER_FROM_DHT];
+}
+
 - (NSInteger) totalPeersKnown
 {
     return fStat->peersKnown;
@@ -1374,16 +1360,6 @@ int trashDataFile(const char * filename)
     return fStat->swarmSpeed;
 }
 
-- (NSInteger) orderValue
-{
-    return fOrderValue;
-}
-
-- (void) setOrderValue: (NSInteger) orderValue
-{
-    fOrderValue = orderValue;
-}
-
 - (NSInteger) groupValue
 {
     return fGroupValue;
@@ -1401,7 +1377,7 @@ int trashDataFile(const char * filename)
 
 - (void) checkGroupValueForRemoval: (NSNotification *) notification
 {
-    if (fGroupValue != -1 && [[[notification userInfo] objectForKey: @"Indexes"] containsIndex: fGroupValue])
+    if (fGroupValue != -1 && [[[notification userInfo] objectForKey: @"Index"] intValue] == fGroupValue)
         fGroupValue = -1;
 }
 
@@ -1501,7 +1477,7 @@ int trashDataFile(const char * filename)
     [[NSNotificationCenter defaultCenter] postNotificationName: @"TorrentFileCheckChange" object: self];
 }
 
-- (void) setFilePriority: (NSInteger) priority forIndexes: (NSIndexSet *) indexSet
+- (void) setFilePriority: (tr_priority_t) priority forIndexes: (NSIndexSet *) indexSet
 {
     const NSUInteger count = [indexSet count];
     tr_file_index_t * files = malloc(count * sizeof(tr_file_index_t));
@@ -1512,7 +1488,7 @@ int trashDataFile(const char * filename)
     free(files);
 }
 
-- (BOOL) hasFilePriority: (NSInteger) priority forIndexes: (NSIndexSet *) indexSet
+- (BOOL) hasFilePriority: (tr_priority_t) priority forIndexes: (NSIndexSet *) indexSet
 {
     for (NSInteger index = [indexSet firstIndex]; index != NSNotFound; index = [indexSet indexGreaterThanIndex: index])
         if (priority == tr_torrentGetFilePriority(fHandle, index) && [self canChangeDownloadCheckForFile: index])
@@ -1530,7 +1506,7 @@ int trashDataFile(const char * filename)
         if (![self canChangeDownloadCheckForFile: index])
             continue;
         
-        NSInteger priority = tr_torrentGetFilePriority(fHandle, index);
+        const tr_priority_t priority = tr_torrentGetFilePriority(fHandle, index);
         if (priority == TR_PRI_LOW)
         {
             if (low)
@@ -1550,7 +1526,7 @@ int trashDataFile(const char * filename)
             normal = YES;
         }
         
-        [priorities addObject: [NSNumber numberWithInt: priority]];
+        [priorities addObject: [NSNumber numberWithInteger: priority]];
         if (low && normal && high)
             break;
     }
@@ -1559,19 +1535,19 @@ int trashDataFile(const char * filename)
 
 - (NSDate *) dateAdded
 {
-    time_t date = fStat->addedDate;
+    const time_t date = fStat->addedDate;
     return [NSDate dateWithTimeIntervalSince1970: date];
 }
 
 - (NSDate *) dateCompleted
 {
-    time_t date = fStat->doneDate;
+    const time_t date = fStat->doneDate;
     return date != 0 ? [NSDate dateWithTimeIntervalSince1970: date] : nil;
 }
 
 - (NSDate *) dateActivity
 {
-    time_t date = fStat->activityDate;
+    const time_t date = fStat->activityDate;
     return date != 0 ? [NSDate dateWithTimeIntervalSince1970: date] : nil;
 }
 
@@ -1618,23 +1594,21 @@ int trashDataFile(const char * filename)
 
 @implementation Torrent (Private)
 
-//if a hash is given, attempt to load that; otherwise, attempt to open file at path
-- (id) initWithHash: (NSString *) hashString path: (NSString *) path torrentStruct: (tr_torrent *) torrentStruct lib: (tr_session *) lib
-        publicTorrent: (NSNumber *) publicTorrent
+- (id) initWithPath: (NSString *) path hash: (NSString *) hashString torrentStruct: (tr_torrent *) torrentStruct lib: (tr_session *) lib
+        publicTorrent: (NSNumber *) publicTorrent publicTorrentLocation: (NSString *) publicTorrentLoc
         downloadFolder: (NSString *) downloadFolder
         useIncompleteFolder: (NSNumber *) useIncompleteFolder incompleteFolder: (NSString *) incompleteFolder
-        ratioSetting: (NSNumber *) ratioSetting ratioLimit: (NSNumber *) ratioLimit
         waitToStart: (NSNumber *) waitToStart
-        orderValue: (NSNumber *) orderValue groupValue: (NSNumber *) groupValue addedTrackers: (NSNumber *) addedTrackers
+        groupValue: (NSNumber *) groupValue addedTrackers: (NSNumber *) addedTrackers
 {
     if (!(self = [super init]))
         return nil;
     
     fDefaults = [NSUserDefaults standardUserDefaults];
 
-    fPublicTorrent = path && (publicTorrent ? [publicTorrent boolValue] : ![fDefaults boolForKey: @"DeleteOriginalTorrent"]);
+    fPublicTorrent = publicTorrentLoc && (publicTorrent ? [publicTorrent boolValue] : ![fDefaults boolForKey: @"DeleteOriginalTorrent"]);
     if (fPublicTorrent)
-        fPublicTorrentLocation = [path retain];
+        fPublicTorrentLocation = [publicTorrentLoc retain];
     
     fDownloadFolder = downloadFolder ? downloadFolder : [fDefaults stringForKey: @"DownloadFolder"];
     fDownloadFolder = [[fDownloadFolder stringByExpandingTildeInPath] retain];
@@ -1663,32 +1637,29 @@ int trashDataFile(const char * filename)
         tr_ctorSetPaused(ctor, TR_FORCE, YES);
         tr_ctorSetPeerLimit(ctor, TR_FALLBACK, [fDefaults integerForKey: @"PeersTorrent"]);
         
-        tr_info info;
-        if (hashString)
+        int result = TR_EINVALID;
+        if (path)
+            result = tr_ctorSetMetainfoFromFile(ctor, [path UTF8String]);
+        
+        //backup - shouldn't be needed after upgrade to 1.70
+        if (result != TR_OK && hashString)
+            result = tr_ctorSetMetainfoFromHash(ctor, [hashString UTF8String]);
+        
+        if (result == TR_OK)
         {
-            tr_ctorSetMetainfoFromHash(ctor, [hashString UTF8String]);
-            if (tr_torrentParse(lib, ctor, &info) == TR_OK)
+            tr_info info;
+            result = tr_torrentParse(ctor, &info);
+            
+            if (result == TR_OK)
             {
                 NSString * currentDownloadFolder = [self shouldUseIncompleteFolderForName: [NSString stringWithUTF8String: info.name]]
                                                     ? fIncompleteFolder : fDownloadFolder;
                 tr_ctorSetDownloadDir(ctor, TR_FORCE, [currentDownloadFolder UTF8String]);
                 
-                fHandle = tr_torrentNew(lib, ctor, NULL);
+                fHandle = tr_torrentNew(ctor, NULL);
             }
-            tr_metainfoFree(&info);
-        }
-        if (!fHandle && path)
-        {
-            tr_ctorSetMetainfoFromFile(ctor, [path UTF8String]);
-            if (tr_torrentParse(lib, ctor, &info) == TR_OK)
-            {
-                NSString * currentDownloadFolder = [self shouldUseIncompleteFolderForName: [NSString stringWithUTF8String: info.name]]
-                                                    ? fIncompleteFolder : fDownloadFolder;
-                tr_ctorSetDownloadDir(ctor, TR_FORCE, [currentDownloadFolder UTF8String]);
-                
-                fHandle = tr_torrentNew(lib, ctor, NULL);
-            }
-            tr_metainfoFree(&info);
+            if (result != TR_EINVALID)
+                tr_metainfoFree(&info);
         }
         
         tr_ctorFree(ctor);
@@ -1703,23 +1674,21 @@ int trashDataFile(const char * filename)
     }
     
     tr_torrentSetCompletenessCallback(fHandle, completenessChangeCallback, self);
+    tr_torrentSetRatioLimitHitCallback(fHandle, ratioLimitHitCallback, self);
     
     fNameString = [[NSString alloc] initWithUTF8String: fInfo->name];
     fHashString = [[NSString alloc] initWithUTF8String: fInfo->hashString];
 	
-    fRatioSetting = ratioSetting ? [ratioSetting intValue] : NSMixedState;
-    fRatioLimit = ratioLimit ? [ratioLimit floatValue] : [fDefaults floatForKey: @"RatioLimit"];
     fFinishedSeeding = NO;
     
     fWaitToStart = waitToStart && [waitToStart boolValue];
     fResumeOnWake = NO;
-    
-    fOrderValue = orderValue ? [orderValue intValue] : tr_sessionCountTorrents(lib) - 1;
-    fGroupValue = groupValue ? [groupValue intValue] : -1;
-    
-    fAddedTrackers = addedTrackers ? [addedTrackers boolValue] : NO; 
-    
+	
     [self createFileList];
+	
+    fGroupValue = groupValue ? [groupValue intValue] : [[GroupsController groups] groupIndexForTorrent: self];
+    
+    fAddedTrackers = addedTrackers ? [addedTrackers boolValue] : NO;    
     
     [[NSNotificationCenter defaultCenter] addObserver: self selector: @selector(checkGroupValueForRemoval:)
         name: @"GroupValueRemoved" object: nil];
@@ -1752,9 +1721,8 @@ int trashDataFile(const char * filename)
             if ([pathComponents count] > 0)
             {
                 //determine if folder node already exists
-                NSEnumerator * enumerator = [fileList objectEnumerator];
                 FileListNode * node;
-                while ((node = [enumerator nextObject]))
+                for (node in fileList)
                     if ([[node name] isEqualToString: name] && [node isFolder])
                         break;
                 
@@ -1803,8 +1771,7 @@ int trashDataFile(const char * filename)
     FileListNode * node = nil;
     if (isFolder)
     {
-        NSEnumerator * enumerator = [[parent children] objectEnumerator];
-        while ((node = [enumerator nextObject]))
+        for (node in [parent children])
             if ([[node name] isEqualToString: name] && [node isFolder])
                 break;
     }
@@ -1902,7 +1869,16 @@ int trashDataFile(const char * filename)
     [status release];
     
     [self update];
-} 
+}
+
+- (void) ratioLimitHit
+{
+    fStat = tr_torrentStat(fHandle);
+    
+    [[NSNotificationCenter defaultCenter] postNotificationName: @"TorrentStoppedForRatio" object: self];
+    
+    fFinishedSeeding = YES;
+}
 
 - (void) quickPause
 {
@@ -1910,14 +1886,14 @@ int trashDataFile(const char * filename)
         return;
 
     fQuickPauseDict = [[NSDictionary alloc] initWithObjectsAndKeys:
-                    [NSNumber numberWithInt: [self speedMode: YES]], @"UploadSpeedMode",
+                    [NSNumber numberWithInt: [self usesSpeedLimit: YES]], @"UploadUsesSpeedLimit",
                     [NSNumber numberWithInt: [self speedLimit: YES]], @"UploadSpeedLimit",
-                    [NSNumber numberWithInt: [self speedMode: NO]], @"DownloadSpeedMode",
+                    [NSNumber numberWithInt: [self usesSpeedLimit: NO]], @"DownloadUsesSpeedLimit",
                     [NSNumber numberWithInt: [self speedLimit: NO]], @"DownloadSpeedLimit", nil];
     
-    [self setSpeedMode: TR_SPEEDLIMIT_SINGLE upload: YES];
+    [self setUseSpeedLimit: YES upload: YES];
     [self setSpeedLimit: 0 upload: YES];
-    [self setSpeedMode: TR_SPEEDLIMIT_SINGLE upload: NO];
+    [self setUseSpeedLimit: YES upload: NO];
     [self setSpeedLimit: 0 upload: NO];
 }
 
@@ -1926,17 +1902,18 @@ int trashDataFile(const char * filename)
     if (!fQuickPauseDict)
         return;
     
-    [self setSpeedMode: [[fQuickPauseDict objectForKey: @"UploadSpeedMode"] intValue] upload: YES];
+    [self setUseSpeedLimit: [[fQuickPauseDict objectForKey: @"UploadUsesSpeedLimit"] intValue] upload: YES];
     [self setSpeedLimit: [[fQuickPauseDict objectForKey: @"UploadSpeedLimit"] intValue] upload: YES];
-    [self setSpeedMode: [[fQuickPauseDict objectForKey: @"DownloadSpeedMode"] intValue] upload: NO];
+    [self setUseSpeedLimit: [[fQuickPauseDict objectForKey: @"DownloadUsesSpeedLimit"] intValue] upload: NO];
     [self setSpeedLimit: [[fQuickPauseDict objectForKey: @"DownloadSpeedLimit"] intValue] upload: NO];
     
     [fQuickPauseDict release];
     fQuickPauseDict = nil;
 }
 
-- (NSString *) etaString: (NSInteger) eta
+- (NSString *) etaString
 {
+    const NSInteger eta = [self eta];
     switch (eta)
     {
         case TR_ETA_NOT_AVAIL:
@@ -1952,17 +1929,14 @@ int trashDataFile(const char * filename)
 {
     //get count
     NSInteger count = 0;
-    NSEnumerator * enumerator = [trackers objectEnumerator];
-    id object;
-    while ((object = [enumerator nextObject]))
+    for (id object in trackers)
         if (![object isKindOfClass: [NSNumber class]])
             count++;
     
     //recreate the tracker structure
     tr_tracker_info * trackerStructs = tr_new(tr_tracker_info, count);
     NSInteger tier = 0, i = 0;
-    enumerator = [trackers objectEnumerator];
-    while ((object = [enumerator nextObject]))
+    for (id object in trackers)
     {
         if (![object isKindOfClass: [NSNumber class]])
         {
@@ -1986,24 +1960,15 @@ int trashDataFile(const char * filename)
         files: [NSArray arrayWithObject: [path lastPathComponent]] tag: nil])
     {
         //if cannot trash, just delete it (will work if it's on a remote volume)
-        if ([NSApp isOnLeopardOrBetter])
-        {
-            NSError * error;
-            if (![[NSFileManager defaultManager] removeItemAtPath: path error: &error])
-                NSLog(@"Could not trash %@: %@", path, [error localizedDescription]);
-        }
-        else
-        {
-            if (![[NSFileManager defaultManager] removeFileAtPath: path handler: nil])
-                NSLog(@"Could not trash %@", path);
-        }
+        NSError * error;
+        if (![[NSFileManager defaultManager] removeItemAtPath: path error: &error])
+            NSLog(@"Could not trash %@: %@", path, [error localizedDescription]);
     }
 }
 
 - (void) setTimeMachineExclude: (BOOL) exclude forPath: (NSString *) path
 {
-    if ([NSApp isOnLeopardOrBetter])
-        CSBackupSetItemExcluded((CFURLRef)[NSURL fileURLWithPath: path], exclude, true);
+    CSBackupSetItemExcluded((CFURLRef)[NSURL fileURLWithPath: path], exclude, true);
 }
 
 @end
