@@ -1,7 +1,7 @@
 /******************************************************************************
- * $Id: TorrentCell.m 7032 2008-11-04 01:31:24Z livings124 $
+ * $Id: TorrentCell.m 8260 2009-04-20 02:45:27Z livings124 $
  *
- * Copyright (c) 2006-2008 Transmission authors and contributors
+ * Copyright (c) 2006-2009 Transmission authors and contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -25,11 +25,8 @@
 #import "TorrentCell.h"
 #import "TorrentTableView.h"
 #import "GroupsController.h"
-#import "NSApplicationAdditions.h"
 #import "NSStringAdditions.h"
-#import "NSBezierPathAdditions.h"
 #import "ProgressGradients.h"
-#import "CTGradient.h"
 
 #define BAR_HEIGHT 12.0f
 
@@ -39,6 +36,9 @@
 #define NORMAL_BUTTON_WIDTH 14.0f
 #define ACTION_BUTTON_WIDTH 16.0f
 
+#define PRIORITY_ICON_WIDTH 14.0f
+#define PRIORITY_ICON_HEIGHT 14.0f
+
 //ends up being larger than font height
 #define HEIGHT_TITLE 16.0f
 #define HEIGHT_STATUS 12.0f
@@ -46,6 +46,7 @@
 #define PADDING_HORIZONTAL 3.0f
 #define PADDING_BETWEEN_IMAGE_AND_TITLE 5.0f
 #define PADDING_BETWEEN_IMAGE_AND_BAR 7.0f
+#define PADDING_BETWEEN_TITLE_AND_PRIORITY 4.0f
 #define PADDING_ABOVE_TITLE 4.0f
 #define PADDING_ABOVE_MIN_STATUS 4.0f
 #define PADDING_BETWEEN_TITLE_AND_MIN_STATUS 2.0f
@@ -100,7 +101,13 @@
         [paragraphStyle release];
         
         fBluePieceColor = [[NSColor colorWithCalibratedRed: 0.0f green: 0.4f blue: 0.8f alpha: 1.0f] retain];
-        fBarBorderColor = [[NSColor colorWithDeviceWhite: 0.0f alpha: 0.2f] retain];
+        fBarBorderColor = [[NSColor colorWithCalibratedWhite: 0.0f alpha: 0.2f] retain];
+        
+        fHighPriorityImage = [[NSImage imageNamed: @"PriorityHigh.png"] copy];
+        [fHighPriorityImage setFlipped: YES];
+        
+        fLowPriorityImage = [[NSImage imageNamed: @"PriorityLow.png"] copy];
+        [fLowPriorityImage setFlipped: YES];
     }
 	return self;
 }
@@ -294,8 +301,7 @@
     }
     else;
     
-    if ([NSApp isOnLeopardOrBetter])
-        [controlView updateTrackingAreas];
+    [controlView updateTrackingAreas];
     
     return YES;
 }
@@ -379,7 +385,7 @@
 {
     Torrent * torrent = [self representedObject];
     
-    BOOL minimal = [fDefaults boolForKey: @"SmallView"];
+    const BOOL minimal = [fDefaults boolForKey: @"SmallView"];
     
     //group coloring
     NSRect iconRect = [self iconRectForBounds: cellFrame];
@@ -399,20 +405,21 @@
                 * darkGroupColor = [groupColor blendedColorWithFraction: 0.2f ofColor: [NSColor whiteColor]];
         
         //border
-        NSBezierPath * bp = [NSBezierPath bezierPathWithRoundedRect: groupRect radius: radius];
+        NSBezierPath * bp = [NSBezierPath bezierPathWithRoundedRect: groupRect xRadius: radius yRadius: radius];
         [darkGroupColor set];
         [bp setLineWidth: 2.0f];
         [bp stroke];
         
         //inside
-        bp = [NSBezierPath bezierPathWithRoundedRect: groupRect radius: radius];
-        CTGradient * gradient = [CTGradient gradientWithBeginningColor: [groupColor blendedColorWithFraction: 0.7f
+        bp = [NSBezierPath bezierPathWithRoundedRect: groupRect xRadius: radius yRadius: radius];
+        NSGradient * gradient = [[NSGradient alloc] initWithStartingColor: [groupColor blendedColorWithFraction: 0.7f
                                     ofColor: [NSColor whiteColor]] endingColor: darkGroupColor];
-        [gradient fillBezierPath: bp angle: 90.0f];
+        [gradient drawInBezierPath: bp angle: 90.0f];
+        [gradient release];
     }
     
     //error image
-    BOOL error = [torrent isError];
+    const BOOL error = [torrent isError];
     if (error && !fErrorImage)
     {
         fErrorImage = [NSImage imageNamed: @"Error.png"];
@@ -435,12 +442,8 @@
     
     //text color
     NSColor * titleColor, * statusColor;
-    if ([self isHighlighted]
-        && [[self highlightColorWithFrame: cellFrame inView: controlView] isEqual: [NSColor alternateSelectedControlColor]])
-    {
-        titleColor = [NSColor whiteColor];
-        statusColor = [NSColor whiteColor];
-    }
+    if ([self backgroundStyle] == NSBackgroundStyleDark)
+        titleColor = statusColor = [NSColor whiteColor];
     else
     {
         titleColor = [NSColor controlTextColor];
@@ -462,6 +465,17 @@
     NSRect titleRect = [self rectForTitleWithString: titleString basedOnMinimalStatusRect: minimalStatusRect inBounds: cellFrame];
     [titleString drawInRect: titleRect];
     
+    //priority icon
+    if ([torrent priority] != TR_PRI_NORMAL)
+    {
+        NSImage * priorityImage = [torrent priority] == TR_PRI_HIGH ? fHighPriorityImage : fLowPriorityImage;
+        
+        NSRect priorityRect = NSMakeRect(NSMaxX(titleRect) + PADDING_BETWEEN_TITLE_AND_PRIORITY,
+                                titleRect.origin.y - (PRIORITY_ICON_HEIGHT - titleRect.size.height) / 2.0,
+                                PRIORITY_ICON_WIDTH, PRIORITY_ICON_HEIGHT);
+        [priorityImage drawInRect: priorityRect fromRect: NSZeroRect operation: NSCompositeSourceOver fraction: 1.0];
+    }
+    
     //progress
     if (!minimal)
     {
@@ -589,7 +603,7 @@
             noIncludeRect.origin.x += barRect.size.width - rightNoIncludeWidth;
             noIncludeRect.size.width = rightNoIncludeWidth;
             
-            [[ProgressGradients progressLightGrayGradient] fillRect: noIncludeRect angle: 90];
+            [[ProgressGradients progressLightGrayGradient] drawInRect: noIncludeRect angle: 90];
         }
         
         if (rightWidth > 0)
@@ -606,7 +620,7 @@
                     notAvailableRect.origin.x += leftWidth + rightWidth;
                     notAvailableRect.size.width = notAvailableWidth;
                     
-                    [[ProgressGradients progressRedGradient] fillRect: notAvailableRect angle: 90];
+                    [[ProgressGradients progressRedGradient] drawInRect: notAvailableRect angle: 90];
                 }
             }
             
@@ -616,7 +630,7 @@
                 includeRect.origin.x += leftWidth;
                 includeRect.size.width = rightWidth;
                 
-                [[ProgressGradients progressWhiteGradient] fillRect: includeRect angle: 90];
+                [[ProgressGradients progressWhiteGradient] drawInRect: includeRect angle: 90];
             }
         }
     }
@@ -629,7 +643,7 @@
         if ([torrent isActive])
         {
             if ([torrent isChecking])
-                [[ProgressGradients progressYellowGradient] fillRect: completeRect angle: 90];
+                [[ProgressGradients progressYellowGradient] drawInRect: completeRect angle: 90];
             else if ([torrent isSeeding])
             {
                 NSInteger ratioLeftWidth = leftWidth * (1.0f - [torrent progressStopRatio]);
@@ -641,30 +655,30 @@
                     ratioLeftRect.origin.x += leftWidth;
                     ratioLeftRect.size.width = ratioLeftWidth;
                     
-                    [[ProgressGradients progressLightGreenGradient] fillRect: ratioLeftRect angle: 90];
+                    [[ProgressGradients progressLightGreenGradient] drawInRect: ratioLeftRect angle: 90];
                 }
                 
                 if (leftWidth > 0)
                 {
                     completeRect.size.width = leftWidth;
                     
-                    [[ProgressGradients progressGreenGradient] fillRect: completeRect angle: 90];
+                    [[ProgressGradients progressGreenGradient] drawInRect: completeRect angle: 90];
                 }
             }
             else
-                [[ProgressGradients progressBlueGradient] fillRect: completeRect angle: 90];
+                [[ProgressGradients progressBlueGradient] drawInRect: completeRect angle: 90];
         }
         else
         {
             if ([torrent waitingToStart])
             {
                 if ([torrent progressLeft] <= 0.0f)
-                    [[ProgressGradients progressDarkGreenGradient] fillRect: completeRect angle: 90];
+                    [[ProgressGradients progressDarkGreenGradient] drawInRect: completeRect angle: 90];
                 else
-                    [[ProgressGradients progressDarkBlueGradient] fillRect: completeRect angle: 90];
+                    [[ProgressGradients progressDarkBlueGradient] drawInRect: completeRect angle: 90];
             }
             else
-                [[ProgressGradients progressGrayGradient] fillRect: completeRect angle: 90];
+                [[ProgressGradients progressGrayGradient] drawInRect: completeRect angle: 90];
         }
     }
 }
@@ -725,7 +739,7 @@
 - (NSRect) rectForTitleWithString: (NSAttributedString *) string basedOnMinimalStatusRect: (NSRect) statusRect
             inBounds: (NSRect) bounds
 {
-    BOOL minimal = [fDefaults boolForKey: @"SmallView"];
+    const BOOL minimal = [fDefaults boolForKey: @"SmallView"];
     
     NSRect result = bounds;
     result.origin.y += PADDING_ABOVE_TITLE;
@@ -733,7 +747,8 @@
     
     result.size = [string size];
     result.size.width = MIN(result.size.width, NSMaxX(bounds) - result.origin.x - PADDING_HORIZONTAL
-                            - (minimal ? PADDING_BETWEEN_TITLE_AND_MIN_STATUS + statusRect.size.width : 0));
+                - (minimal ? PADDING_BETWEEN_TITLE_AND_MIN_STATUS + statusRect.size.width : 0.0)
+                - ([[self representedObject] priority] != TR_PRI_NORMAL ? PRIORITY_ICON_WIDTH + PADDING_BETWEEN_TITLE_AND_PRIORITY: 0.0));
     
     return result;
 }
@@ -791,7 +806,7 @@
             return NSLocalizedString(@"Pause the transfer", "Torrent Table -> tooltip");
         else
         {
-            if ([[NSApp currentEvent] modifierFlags] & NSAlternateKeyMask && [fDefaults boolForKey: @"Queue"])
+            if ([[NSApp currentEvent] modifierFlags] & NSAlternateKeyMask)
                 return NSLocalizedString(@"Resume the transfer right away", "Torrent cell -> button info");
             else if ([torrent waitingToStart])
                 return NSLocalizedString(@"Stop waiting to start", "Torrent cell -> button info");
